#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>
#include <hpdir.h>

int hpd_rcl_real (char *name, double *val)
{
	int Ret = HPD_OK;

	if (name && val)
	{
		SAT_OBJ_DSCR *dirobj;

		// Gets the dir object		
		if (dirobj = sat_dir_rclobject (name))
		{	
			int type;
			hpobj_t obj;
			int incr;
			obj.type = 0;
	
			// Gets the type of the object
			type = sat_peek (dirobj->addr, 5);

			// Deals with the type
			switch (type)
			{
				case DOREAL :		// Requested type => no conversion needed
				{
					// Decodes the object
					(void) hps_decode_real (dirobj->addr, &obj, &incr);
					*val = obj.data.doreal;
						
					break;
				}
					
				case DOHXS :		// Conversion needed if authorized
				{
					if (hps_get_convert ())	// Checks for conversion flag
					{
						// Decodes the object
						(void) hps_decode_uint (dirobj->addr, &obj, &incr);
						*val = (double) obj.data.dohxs;
					}
					else
					{
						Ret = HPD_ERROR_TYPE;
					}

					break;
				}

				case DOINT :		// Conversion needed if authorized
				{
					if (hps_get_convert ())	// Checks for conversion flag
					{
						// Decodes the object
						(void) hps_decode_int (dirobj->addr, &obj, &incr);
						*val = (double) obj.data.doint;
					}
					else
					{
						Ret = HPD_ERROR_TYPE;
					}

					break;
				}
					
				case DOCMP :		// Conversion needed if authorized
				{
					if (hps_get_convert ())	// Checks for conversion flag
					{
						// Decodes the object
						(void) hps_decode_complex (dirobj->addr, &obj, &incr);
						if (obj.data.docmp.im == 0.)
						{
							*val = obj.data.docmp.re;
						}
						else
						{
							Ret = HPD_ERROR_TYPE;
						}
					}
					else
					{
						Ret = HPD_ERROR_TYPE;
					}

					break;
				}

				default :				// No conversion possible : type mismatch
				{
					Ret = HPD_ERROR_TYPE;
					break;
				}
			}
		}
		else
		{
			Ret = HPD_ERROR_NOT_FOUND;
		}
	}
	else
	{
		Ret = HPD_ERROR_PAR;
	}
	
	return (Ret);
}
