#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_code (hpobj_t *obj, unsigned *address)
{
	int Ret = HPS_OK;
	if (obj)
	{
		unsigned addr;			// Target address (in Saturn memory)
		int size = 0;			// Size of the object

		// Computes the size of the object
		if ((Ret = hps_objsize (obj, &size)) == HPS_OK)
		{
			if (* address)
			{
				// The data will be encoded in an existing object
				addr = * address;
			}
			else
			{
				// Tries to create a new object
				* address = addr = sat_createtemp (size);
			}			

			if (addr)
			{
				switch (obj->type)
				{
					case DOINT :
					{
						hps_code_int (obj->data.doint, address);
						
						break;
					}	
				
					case DOHXS :
					{
						hps_code_uint (obj->data.dohxs, address);
										
						break;
					}	

					case DOREAL :
					{
						hps_code_real (obj->data.doreal, address);

						break;
					}
					
					case DOCSTR :
					{
						hps_code_str (obj->data.docstr, address);

						break;
					}
					
					case DOCMP :
					{
						hps_code_complex (&(obj->data.docmp), address);
			
						break;
					}
					
					case DOLIST :
					{
						hps_code_list (obj->data.dolist, address);

						break;
					}
					
					case DOMATRIX :
					{
						hps_code_array (obj->data.domatrix, address);
			
						break;
					}
					
					case DOIDNT :
					{
						hps_code_ident (obj->data.doidnt, address);

						break;
					}
			
					case DOSYMB :
					{
						hps_code_symb (obj->data.dosymb, address);

						break;
					}

					default :
					{
						Ret = HPS_ERROR_NOT_IMPLEMENTED;
						break;
					}
				}
			}
			else
			{
				Ret = HPS_ERROR_SAT_MEMORY;
			}
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
		
	return (Ret);
}
