#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>
#include <hpparser.h>

int nbdigit (double x, int base)
{
	int nb = 1;
	
	while ((int) (x /= base)) nb++;
	
	return (nb);
}

int hps_objsize (hpobj_t *obj, int *size)
{
	int Ret = HPS_OK;

	if (obj)
	{
		switch (obj->type)
		{
			case DOINT :
			{
				LONGLONG val = obj->data.doint;
				if (val != 0)
				{
					*size = nbdigit ((double) val, 10);
				}
				else
				{
					*size = 0;
				}
				*size += 11;						// 5 (prologue) + 5 (size) + 1 (sign)
				
				break;
			}
			
			case DOHXS :
			{
				*size = 26;							// 5 (prologue) + 5 (size) + 16 (data)
				
				break;
			}
			
			case DOREAL :
			{
				*size = 21;							// 5 (prologue) + 3 (exponent) + 12 (mantissa) + 1 (sign)
				
				break;
			}
			
			case DOCSTR :
			{
				char *str = obj->data.docstr;
				*size = strlen (str) * 2 + 10;	// Number of chars * 2 + 5 (prologue) + 5 (size)
				
				break;
			}
			
			case DOCMP :
			{
				*size = 37;								// 5 (prologue) + 2 * 16 (real w/o prologue)
				
				break;
			}
			
			case DOLIST :
			{
				list_t *cur = obj->data.dolist;
				int s;

				// Computes the size of the object
				*size = 0;
				while (cur)
				{
					if ((Ret = hps_objsize (cur->obj, &s)) == HPS_OK)
					{
						*size += s;
					}
					else
					{
						break;
					}
					
					cur = cur->next;
				}
				
				*size += 10;					// Size of the objects + 5 (prologue) + 5 (epilogue)
				
				break;
			}
			
			case DOMATRIX :
			{
				array_t *array = obj->data.domatrix;
				array_t *cur = array;
				int s;

				*size = 0;
				while (cur)
				{
					if (cur->obj)
					{
						if ((Ret = hps_objsize (cur->obj, &s)) == HPS_OK)
						{
							*size += s;
						}
						else
						{
							break;
						}
					}
									
					cur = cur->next;
				}

				*size += 10;					// Size of the objects + 5 (prologue) + 5 (epilogue)

				break;
			}		

			case DOIDNT :
			{
				char *ident = obj->data.doidnt;
				
				*size = strlen (ident)*2 + 5 + 2;		// Size of the ident + 5 (prologue) + 2 (length)
				
				break;
			}
			
			case DOFUNC :
			{
				*size = 5;

				break;
			}
			
			case DOSYMB :
			{
				int level = 1;
				entry_t *stack = obj->data.dosymb;
				element_t element;

				*size = 0;
				if (stack)
				{
					while ((prs_alg_get_element (stack, level++, &element) == PRS_OK) && (Ret == HPS_OK))
					{
					   switch (element.Type)
					   {
							case C_TypeVal :
							{
								*size += 21;				// Size of a DOREAL
								break;
							}
	
							case C_TypeIdent :
							{
								*size += strlen (element.data.Ident)*2 + 7;	// Size of a DOIDENT
								break;
							}
	
							case C_TypeFunc :
							{
								*size += 5;				// Size of a DOFUNC
								break;
							}
							
							case C_TypeSymb :
							{
								hpobj_t *obj;
								if ((Ret = hps_obj_create (&obj)) == HPS_OK)
								{
									int s;
									
									obj->type = DOSYMB;
									obj->data.dosymb = element.data.Symb;
									if ((Ret = hps_objsize (obj, &s)) == HPS_OK)
									{
										*size += s;
									}
									
									(void) hps_obj_destroy (&obj);
								}
								break;
							}
						}
					}
	
					*size += 10;				// Size of the objects + 5 (prologue) + 5 (epilogue)
				}
				else
				{
					Ret = HPS_ERROR_PAR;
				}
				
			   break;
			}

			default :
			{
				Ret = HPS_ERROR_NOT_IMPLEMENTED;
				break;
			}
		}
	}
	else
	{
		*size = 0;
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}

int hps_objsize_type (int type, void *data, int *size)
{
	hpobj_t obj;
	
	obj.type = type;
	(void) memcpy ((void *) &(obj.data), data, sizeof (obj.data));
	
	return (hps_objsize (&obj, size));
}
