#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_pick_int (int level, LONGLONG *val)
{
	int Ret;

	if (val)
	{
		int depth;
		
		// Gets the stack depth
		(void) hps_depth (&depth);
		
		// Checks that the asked level exists
		if ((level > 0) && (level <= depth))
		{	
			int type;
	
			// Gets the type of the object at the indicated level
			if ((Ret = hps_get_type (level, &type)) == HPS_OK)
			{
				// Deals with the type
				switch (type)
				{
					case DOINT :		// Requested type => no conversion needed
					{
						hpobj_t obj;
						int incr;
						
						// Gets the address of the object located at this level
						unsigned address = sat_peek (sat_peek (SAT_DSKTOP, 5) + (level-1) * 5, 5);
						
						// Tries to decode the object
						obj.type = 0;
						Ret = hps_decode_int (address, &obj, &incr);
						*val = obj.data.doint;
						
						break;
					}
					
					case DOHXS :		// Conversion needed if authorized
					{
						if (hps_get_convert ())	// Checks for conversion flag
						{
							// Converts the data
							ULONGLONG data;
							Ret = hps_pick_uint (level, &data);
							*val = (LONGLONG) data;
						}
						else
						{
							Ret = HPS_ERROR_TYPE;
						}
						break;
					}

					case DOREAL :		// Conversion needed if authorized
					{
						if (hps_get_convert ())	// Checks for conversion flag
						{
							// Converts the data
							double data;
							Ret = hps_pick_real (level, &data);
							*val = (LONGLONG) data;
						}
						else
						{
							Ret = HPS_ERROR_TYPE;
						}
						break;
					}
					
					default :				// No conversion possible : type mismatch
					{
						Ret = HPS_ERROR_TYPE;
						break;
					}
				}
			}
		}
		else
		{
			Ret = HPS_ERROR_STACK;
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}
