#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_pick_real (int level, double *val)
{
	int Ret;

	if (val)
	{
		int depth;
		
		// Gets the stack depth
		(void) hps_depth (&depth);
		
		// Checks that the asked level exists
		if ((level > 0) && (level <= depth))
		{
			int type;
			
			// Gets the type of the object at the indicated level
			if ((Ret = hps_get_type (level, &type)) == HPS_OK)
			{
				int convertflag = hps_get_convert ();
				switch (type)
				{
					case DOREAL :		// Requested type => no conversion needed
					{
						hpobj_t obj;
						int incr;
						
						// Gets the address of the object located at this level
						unsigned address = sat_peek (sat_peek (SAT_DSKTOP, 5) + (level-1) * 5, 5);
						
						// Tries to decode the object
						obj.type = 0;
						Ret = hps_decode_real (address, &obj, &incr);
						*val = obj.data.doreal;
	
						break;
					}
					
					case DOINT :		// Conversion needed if authorized
					{
						if (convertflag)
						{
							// Converts the data
							LONGLONG data;
							Ret = hps_pick_int (level, &data);
							*val = (double) data;
						}
						else
						{
							Ret = HPS_ERROR_TYPE;
						}
						break;
					}
					
					case DOHXS :		// Conversion needed if authorized
					{
						if (convertflag)
						{
							// Converts the data
							ULONGLONG data;
							Ret = hps_pick_uint (level, &data);
							*val = (double) data;
						}
						else
						{
							Ret = HPS_ERROR_TYPE;
						}
						break;
					}
					
					case DOCMP :		// Conversion needed if authorized
					{
						if (convertflag)
						{
							// Converts the data
							complex_t data;
							Ret = hps_pick_complex (level, &data);
							if (data.im == 0.)
							{
								*val = data.re;
							}
							else
							{
								Ret = HPS_ERROR_TYPE;
							}
						}
						else
						{
							Ret = HPS_ERROR_TYPE;
						}
						break;
					}

					default :				// No conversion possible : type mismatch
					{
						Ret = HPS_ERROR_TYPE;
						break;
					}
				}
			}
		}
		else
		{
			Ret = HPS_ERROR_STACK;
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}
