#ifndef __HPGRAPHICS_H
#define __HPGRAPHICS_H	1

typedef struct hpg_graphics hpg_t;

typedef struct hpg_font hpg_font_t;

typedef struct hpg_pattern hpg_pattern_t;

extern hpg_t *hpg_stdscreen;

#define HPG_MODE_PAINT 0
#define HPG_MODE_XOR 1


#define HPG_COLOR_WHITE    0x00
#define HPG_COLOR_GRAY_1   0x11
#define HPG_COLOR_GRAY_2   0x22
#define HPG_COLOR_GRAY_3   0x33
#define HPG_COLOR_GRAY_4   0x44
#define HPG_COLOR_GRAY_5   0x55
#define HPG_COLOR_GRAY_6   0x66
#define HPG_COLOR_GRAY_7   0x77
#define HPG_COLOR_GRAY_8   0x88
#define HPG_COLOR_GRAY_9   0x99
#define HPG_COLOR_GRAY_10  0xAA
#define HPG_COLOR_GRAY_11  0xBB
#define HPG_COLOR_GRAY_12  0xCC
#define HPG_COLOR_GRAY_13  0xDD
#define HPG_COLOR_GRAY_14  0xEE
#define HPG_COLOR_BLACK    0xFF

#define HPG_INDICATOR_REMOTE  0
#define HPG_INDICATOR_LSHIFT  1
#define HPG_INDICATOR_RSHIFT  2
#define HPG_INDICATOR_ALPHA   3
#define HPG_INDICATOR_BATTERY 4
#define HPG_INDICATOR_WAIT    5


void hpg_init(void);
void hpg_cleanup(void);

void hpg_set_mode_mono(int dbuf);
void hpg_set_mode_gray4(int dbuf);
void hpg_set_mode_gray16(int dbuf);

int hpg_get_width(hpg_t *g);
int hpg_get_height(hpg_t *g);

void hpg_set_indicator(unsigned char indicator, unsigned char color);

void hpg_flip(void);

void hpg_clip_reset(hpg_t *g);
void hpg_clip_set(hpg_t *g, int x1, int y1, int x2, int y2);
void hpg_clip(hpg_t *g, int x1, int y1, int x2, int y2);


void hpg_clear_on(hpg_t *g);
void hpg_clear(void);

void hpg_draw_pixel_on(hpg_t *g, int x, int y);
void hpg_draw_pixel(int x, int y);

void hpg_draw_line_on(hpg_t *g, int x1, int y1, int x2, int y2);
void hpg_draw_line(int x1, int y1, int x2, int y2);

void hpg_draw_rect_on(hpg_t *g, int x1, int y1, int x2, int y2);
void hpg_draw_rect(int x1, int y1, int x2, int y2);
void hpg_fill_rect_on(hpg_t *g, int x1, int y1, int x2, int y2);
void hpg_fill_rect(int x1, int y1, int x2, int y2);

void hpg_draw_circle_on(hpg_t *g, int cx, int cy, int r);
void hpg_draw_circle(int cx, int cy, int r);
void hpg_fill_circle_on(hpg_t *g, int cx, int cy, int r);
void hpg_fill_circle(int cx, int cy, int r);

void hpg_draw_polygon_on(hpg_t *g, int vx[], int vy[], int len);
void hpg_draw_polygon(int vx[], int vy[], int len);
void hpg_fill_polygon_on(hpg_t *g, int vx[], int vy[], int len);
void hpg_fill_polygon(int vx[], int vy[], int len);


hpg_font_t *hpg_get_minifont(void);
hpg_font_t *hpg_get_bigfont(void);

int hpg_font_get_height(hpg_font_t *font);
int hpg_font_get_advance(hpg_font_t *font);

void hpg_draw_letter_on(hpg_t *g, char a, int x, int y);
void hpg_draw_letter(char a, int x, int y);

void hpg_draw_text_on(hpg_t *g, char *s, int x, int y);
void hpg_draw_text(char *s, int x, int y);


unsigned char hpg_get_color(hpg_t *g);
void hpg_set_color(hpg_t *g, unsigned char color);


unsigned char hpg_get_mode(hpg_t *g);
void hpg_set_mode(hpg_t *g, unsigned char mode);



hpg_pattern_t *hpg_get_pattern(hpg_t *g);
void hpg_set_pattern(hpg_t *g, hpg_pattern_t *pattern);

hpg_font_t *hpg_get_font(hpg_t *g);
void hpg_set_font(hpg_t *g, hpg_font_t *font);

hpg_pattern_t *hpg_alloc_pattern(char *buffer, int height, int fixed);
void hpg_free_pattern(hpg_pattern_t *pattern);

hpg_font_t *hpg_alloc_font(char *buffer, int count, int height, int advance);
void hpg_free_font(hpg_font_t *font);


unsigned char hpg_get_pixel(hpg_t *g, int x, int y);


hpg_t *hpg_alloc_mono_image(int width, int height);
hpg_t *hpg_alloc_gray4_image(int width, int height);
hpg_t *hpg_alloc_gray16_image(int width, int height);
void hpg_free_image(hpg_t *img);


void hpg_blit(hpg_t *src, int sx, int sy, int w, int h,
              hpg_t *dst, int dx, int dy);


hpg_t *hpg_load_xpm_mono(char *xpm[]);
hpg_t *hpg_load_xpm_gray4(char *xpm[]);
hpg_t *hpg_load_xpm_gray16(char *xpm[]);


#endif /* __HPGRAPHICS_H */
 
