#ifndef _HPKEYB49_H
#define _HPKEYB49_H	1

#ifndef _LONGLONG_DEF
typedef unsigned long long ULONGLONG;
typedef long long LONGLONG;
#define _LONGLONG_DEF
#endif


extern int keyb_isKeyPressed(unsigned char col, unsigned char row);
extern int keyb_isAnyKeyPressed(void);

extern void keyb_waitKeyPressed(void);

extern int keyb_isLeft(void);
extern int keyb_isRight(void);
extern int keyb_isUp(void);
extern int keyb_isDown(void);
extern int keyb_isAlpha(void);
extern int keyb_isLS(void);
extern int keyb_isRS(void);
extern int keyb_isON(void);

extern int keyb_getkey(int wait);


#define KB_A   41
#define KB_B   42
#define KB_C   43
#define KB_D   44
#define KB_E   45
#define KB_F   46
#define KB_G   47
#define KB_H   53
#define KB_I   54
#define KB_J   55
#define KB_K   57
#define KB_L   58
#define KB_M   33
#define KB_N   25
#define KB_O   17
#define KB_P    9
#define KB_Q   34
#define KB_R   26
#define KB_S   18
#define KB_T   10
#define KB_U    2
#define KB_V   35
#define KB_W   27
#define KB_X   19
#define KB_Y   11
#define KB_Z KB_DIV
#define KB_0   31
#define KB_1   30
#define KB_2   22
#define KB_3   14
#define KB_4   29
#define KB_5   21
#define KB_6   13
#define KB_7   28
#define KB_8   20
#define KB_9   12
#define KB_BKS  1
#define KB_DIV  3
#define KB_MUL  4
#define KB_ADD  6
#define KB_SUB  5
#define KB_DOT 23
#define KB_SPC 15
#define KB_ENT  7
#define KB_UP  49
#define KB_DN  51
#define KB_LF  50
#define KB_RT  52
#define KB_ALPHA   60
#define KB_LSHIFT  61
#define KB_RSHIFT  62
#define KB_ON  63


#define KB_MASK64(a) (((ULONGLONG)1)<<(a))
#define KB_MASK32(a) (((unsigned int)1)<<(a&31))
#define KB_MASK8(a)  (((unsigned int)1)<<(a&7))

/* COMMON KEYBOARD MASKS */
/* ALL CURSOR KEYS */

#define KB_CURS ((ULONGLONG)0x001e000000000000)
#define KBH_CURS32 ((unsigned int)0x001e0000)
#define KB_FUNC ((ULONGLONG)0x00007e0000000000)
/* USE WITH HIWORD ONLY  */
#define KBH_FUNC32 ((unsigned int)0x00007e00)


/* SHIFT CONSTANTS FOR HIGH-LEVEL KEYBOARD FUNCTIONS */

#define SHFT_ALPHA  0x10000000
#define SHFT_RS     0x20000000
#define SHFT_LS     0x40000000
#define SHFT_ON     0x80000000
#define SHFT_ANY    0xf0000000

#define KEYCODE(shftcode)   ((shftcode) & 0x000000ff)
#define SHFTPLANE(shftcode) ((shftcode) & 0xf0000000)


/* KEYMATRIX TYPE DEFINITION */
/*!
 * \brief A matrix of simultaneous key states.
 *
 * This data type is used to represent the complete state of the keyboard.
 * It can be accessed as either a single 64-bit mask, a pair of two 32-bit
 * masks representing the high and low halves of the entire mask, or as
 * individual bytes, each representing a logical column of the keyboard.
 */
typedef union {
    unsigned long long full;
/*
#warning FIXME: endianness
*/
    struct {
	unsigned int loword, hiword;
    }
#ifdef __STRICT_ANSI__
    words	/* No unnamed struct in ANSI C */
#endif
    ;
    unsigned char bytes[8];
} keymatrix;


/* SCANS THE KEYBOARD AND STORES THE 64-BIT MATRIX IN ptr */
extern void keyb_getmatrix(keymatrix * ptr);
extern int keyb_isKeyPrM(int keycode);
extern int keyb_isShiftKeyPrM(int shiftcode);

extern int keyb_getkeyM(int wait);
extern int keyb_getnextkey(keymatrix * ptr, int keycode);

#define WAIT_CANCEL { while(keyb_isON()) ; while(! keyb_isON()) ; }

#endif
