#ifndef _KOS_H
#define _KOS_H	1

#include <stdio.h>
#include <time.h>


typedef unsigned int U32;
typedef int S32;
typedef unsigned short U16;
typedef short S16;
typedef unsigned char U8,*P_U8;
typedef char S8;
typedef char BYTE;
typedef char *P_BYTE;
typedef void *P_VOID;

/* typedef int BOOL; */

#ifndef SIZE_T
#define SIZE_T size_t
#endif


#ifndef NULL
#define NULL 0
#endif

#define TIME_T time_t

typedef U32 (*FUNC_PTR)();
typedef U32 (*FUNC0PTR)();
typedef U32 (*FUNC1PTR)(U32);
typedef U32 (*FUNC2PTR)(U32,U32);
typedef U32 (*FUNC3PTR)(U32,U32,U32);



#ifndef FALSE
#define FALSE 0
#define TRUE (! FALSE)
#endif




/* The KOS Date structure */
typedef struct
{
    U16 year;
    U8 month;
    U8 day;
}
KDATE;

/* The KOS Time structure */
typedef struct
{
    U8 hours;
    U8 minutes;
    U8 seconds;
    U8 unknown1;
    U32 unknown2;
} KTIME;

/* The KOS Date&Time Structure (nesting) */
typedef struct
{
    KDATE date;
    KTIME time;
}
KDATETIME, *P_KDATETIME;


#define KOS_MEM_FREE                        0x0000
#define KOS_MEM_GLOBAL                      0x1000
#define KOS_MEM_LOCAL                       0x2000
#define KOS_MEM_HEAP0                       0x0100


#define KOS_MODULE_FILESYSTEM 1

#define KOS_FUNC_FOPEN    0
#define KOS_FUNC_FREAD    1
#define KOS_FUNC_FWRITE    2
#define KOS_FUNC_FCLOSE    3
#define KOS_FUNC_FSEEK    4
#define KOS_FUNC_FDELETE 5
#define KOS_FUNC_FRENAME        6
#define KOS_FUNC_CHDIR            7
#define KOS_FUNC_GETFILEINFO    8
#define KOS_FUNC_GETFIRSTDIR    9
#define KOS_FUNC_GETNEXTDIR        10
#define KOS_FUNC_GETPREVDIR        11
#define KOS_FUNC_GETFIRSTFILE    12
#define KOS_FUNC_GETNEXTFILE    13
#define KOS_FUNC_GETPREVFILE    14
#define KOS_FUNC_FATTR            15
#define KOS_FUNC_GETSIZE        16
#define KOS_FUNC_CHECKINSERT    17
#define KOS_FUNC_CHECKPROTECT    18



#define KOS_READMODE 1        
#define KOS_WRITEMODE 2



/* Filesystem (fs.h) */

#define FAT32_SUPPORT  1


#define NEW_FILE_NAME


typedef struct 
{
       KDATETIME       last_modifier;
       KDATETIME       create_date;
       KDATE           last_access_date;
#ifdef  NEW_FILE_NAME
       BYTE            file_name[13];
#else
       BYTE            extension[4];
       BYTE            name[9];
#endif
       U8               attribute;
       U16             ext_attr;
       U32             size;
}
FILE_INFO, DIR_INFO, *P_FILE_INFO, *P_DIR_INFO;



typedef FILE *H_FILE;


#define KOS_READONLY_ATTRIBUTE (1)
#define KOS_HIDDEN_ATTRIBUTE (1<<1)
#define KOS_SYSTEM_ATTRIBUTE (1<<2)
#define KOS_ARCHIVE_ATTRIBUTE (1<<5)

#define FILE_READONLY   0x00000001      /* mode parameter in kos_fopen */
#define FILE_READWRITE  0x00000002
#define FILE_APPEND     0x00000003

#define FS_SEEK_SET             SEEK_SET
#define FS_SEEK_CUR             SEEK_CUR
#define FS_SEEK_END             SEEK_END

#define ERR_FS_INVALID_NAME                     -1
#define ERR_FS_FILE_OPENED                      -2
#define ERR_FS_INVALID_HANDLE                   -3
#define ERR_FS_INVALID_INDEX                    -4
#define ERR_FS_INVALID_MODE                     -5
#define ERR_FS_DISK_FULL                        -6
#define ERR_FS_DISK_ERROR                       -7
#define ERR_FS_DISK_CHANGE                      -8
#define ERR_FS_NO_SDCARD                        -9
#define ERR_FS_NO_MEMORY                       -10
#define ERR_FS_NOT_SUPPORT                     -11
#define ERR_FS_PROTECT                         -13
#define ERR_FS_NOTFIND                         -14
#define ERR_FS_DISK_REMOVE                     -15
#define ERR_FS_FILE_NOTEXIST                   -16
#define ERR_FS_UNKNOWN                        -100



/* Errors */

#define SUCCESS 0
#define KOS_ERR_INVALIDPTR    -200 /* FIXME: random guess */

/* Kernel */

FUNC_PTR kos_GetFuncEntry(U32 module, U32 index);


/* Filesystem */

H_FILE kos_fopen(P_BYTE fname, U8 mode);
size_t kos_fread(H_FILE fd, P_BYTE buf, U32 length);
size_t kos_fwrite(H_FILE fd, P_BYTE buf, U32 length);
S32 kos_fseek(H_FILE fd, S32 offset, U32 mode);
S32 kos_fclose(H_FILE fd);
S32 kos_fdelete(P_BYTE fname);


/* CHANGED TO A MACRO */
#define fdelete(a) kos_fdelete(a);

S32 kos_frename(P_BYTE oldname, P_BYTE newname);
S32 kos_chdir(P_BYTE pDir);
S32 kos_GetSpecialFileInfo(P_BYTE fname, P_FILE_INFO pInfo);
S32 kos_GetFirstDirInfo(P_DIR_INFO pInfo);
S32 kos_GetNextDirInfo(P_DIR_INFO pInfo, U32 index);
S32 kos_GetPrevDirInfo(P_DIR_INFO pInfo, U32 index);
S32 kos_GetFirstFileInfo(P_FILE_INFO pInfo);
S32 kos_GetNextFileInfo(P_FILE_INFO pInfo, U32 index);
S32 kos_GetPrevFileInfo(P_FILE_INFO pInfo, U32 index);
S32 kos_fattr(P_BYTE fname, U8 attr);
S32 kos_GetDiskFreeSize(void);
S32 kos_CheckDiskInsert(void);
S32 kos_CheckDiskProtect(void);
void kos_ResetSD(void);

#define kos_malloc malloc
#define kos_free free;


/* display routines */

void kos_ClearLcd(void);
U32 kos_PutString(U32 dx, U32 dy, P_VOID ptr);
void kos_PutHexNumber(U32 dx, U32 dy, U32 value);
void kos_PutDecNumber(U32 dx, U32 dy, U32 value, U32 bit);

U32 kos_LCDGetContrast(void);
void kos_LCDSetContrast(U32 contrast);

/* misc routines */

P_BYTE kos_getSerialNumber(void);
void kos_Reset(void);
S32 kos_beep(U32 freq, TIME_T time, U32 override);


/* Time */

U32 kos_GetSysTickTime();


#endif
