#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <glib/gthread.h>
#include <gdk/gdk.h>
#include "gui_x11.h"
#include "../include/hpapine_gui.h"
#include "../include/_hpapine_version.h"

#ifndef G_THREADS_ENABLED
#error Please install a Glib with thread support.
#endif

#ifdef GDK_WINDOWING_X11
#define __GDK_BACKEND__ "X11"
#endif

#ifdef GDK_WINDOWING_WIN32
#define __GDK_BACKEND__ "Win32"
#endif

#ifndef __GDK_BACKEND__
#define __GDK_BACKEND__ "Unknown"
#endif


const char *gui_name = "GDK (" __GDK_BACKEND__ ")";


/* Window geometry */
static unsigned int winw;
static unsigned int winh;
static unsigned int winzoom = DEFAULT_WINZOOM;

/* */
static void *gui_loop(void *arg);



/* The main window and graphic context */
static GdkWindow *window = NULL;
static GdkGC *gc = NULL;



void gui_run(void)
{
    if (window)
	return;

    hpapine_print_version();

    /* Multi-threading support */
    g_thread_init(NULL);
    if (!g_thread_supported()) {
	fprintf(stderr,
		"Please install a GLib with thread support. Exiting.\n");
	exit(1);
    }
    gdk_threads_init();

    gdk_init(NULL, NULL);

    /* Window attributes */
    winw = GUI_WIN_W * winzoom;
    winh = GUI_WIN_H * winzoom;

    GdkWindowAttr attributes;
    attributes.title = "hpapine";
    attributes.event_mask =
	GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK | GDK_FOCUS_CHANGE_MASK;
    attributes.wclass = GDK_INPUT_OUTPUT;
    attributes.window_type = GDK_WINDOW_TOPLEVEL;
    attributes.width = winw;
    attributes.height = winh;
    attributes.visual = gdk_visual_get_system();
    attributes.colormap = gdk_colormap_get_system();

    /* Creating the window */
    window =
	gdk_window_new(NULL, &attributes,
		       GDK_WA_TITLE | GDK_WA_VISUAL | GDK_WA_COLORMAP);
    gdk_window_show(window);
    gc = gdk_gc_new(window);
    gdk_flush();

    /* Launch the gui thread */
    if (!g_thread_create(gui_loop, NULL, TRUE, NULL)) {
	fprintf(stderr, "g_thread_create() fails\n");
	exit(1);
    }

    return;
}

/*
static void gui_close(void)
{
    return;
}
*/


static void *gui_loop(void *arg)
{
    guint8 *graybuf = g_malloc0(131 * 80 * winzoom * winzoom);
    unsigned int counter = 0;

    while (1) {
	/*
	 * Wait 0.005 s
	 */
	usleep(5000);

	gdk_threads_enter();

	/*
	 * Screen painting every 0.025 s
	 */
	if (counter++ % 5 == 0) {
	    unsigned int i, j;
	    for (i = 0; i < 131; i++) {
		for (j = 0; j < 80; j++) {
		    unsigned int c, ii, jj;
		    c = screen_getpixlum(i, j);
		    for (ii = 0; ii < winzoom; ii++)
			for (jj = 0; jj < winzoom; jj++)
			    graybuf[(winzoom * j + jj) * winzoom * 131 +
				    (winzoom * i + ii)] = c << 4;
		}
	    }
	    gdk_draw_gray_image(window, gc, 0, 0, 131 * winzoom,
				80 * winzoom, GDK_RGB_DITHER_NORMAL,
				graybuf, 131 * winzoom);
	    gdk_flush();
	}


	/*
	 * Process events
	 */
	GdkEvent *event;
	while ((event = gdk_event_get())) {
	    switch (event->type) {
	    case GDK_KEY_PRESS:
		keyval_press(event->key.keyval);
		break;
	    case GDK_KEY_RELEASE:
		keyval_release(event->key.keyval);
		break;
	    case GDK_FOCUS_CHANGE:
		keyval_releaseall();
		break;
	    default:
		break;
	    }

	    gdk_event_free(event);
	}

        gdk_threads_leave();
    }
}

void gui_change_screenbuf(unsigned char *new)
{
    gdk_threads_enter();
    screen_buf = new;
    gdk_threads_leave();
}



void gui_bell(int tone, int duration)
{
    fprintf(stderr, "\a");
    return;
}
