#include <stdio.h>
#include <gdk/gdkkeysyms.h>
#include "../include/hpapine_gui.h"
#include "gui_x11.h"




static unsigned long long keystate;
static char keyval_to_code(guint keyval);



void gui_getmatrix(unsigned long long *km)
{
    gui_run();			/* ensure the gui is properly initialized */
    *km = keystate;
}

void keyval_release(guint keyval)
{
    char code = keyval_to_code(keyval);
    if (code == 0)
	return;
    keystate = keystate & (~(1LL << code));
}

void keyval_press(guint keyval)
{
    char code = keyval_to_code(keyval);
    if (code == 0)
	return;
    keystate = keystate | (1LL << code);
}

void keyval_releaseall(void)
{
    keystate = 0LL;
}


static char keyval_to_code(guint keyval)
{
    const static char tab_09[] = {
	31,			/* 0 */
	30,			/* 1 */
	22,			/* 2 */
	14,			/* 3 */
	29,			/* 4 */
	21,			/* 5 */
	13,			/* 6 */
	28,			/* 7 */
	20,			/* 8 */
	19			/* 9 */
    };
    const static char tab_az[] = {
	41,			/* A */
	42,			/* B */
	43,			/* C */
	44,			/* D */
	45,			/* E */
	46,			/* F */
	47,			/* G */
	53,			/* H */
	54,			/* I */
	55,			/* J */
	57,			/* K */
	58,			/* L */
	33,			/* M */
	25,			/* N */
	17,			/* O */
	9,			/* P */
	34,			/* Q */
	26,			/* R */
	18,			/* S */
	10,			/* T */
	2,			/* U */
	35,			/* V */
	27,			/* W */
	19,			/* X */
	11,			/* Y */
	3			/* Z */
    };

    if (GDK_a <= keyval && keyval <= GDK_z) {	/* a-z */
	return tab_az[keyval - GDK_a];
    } else if (GDK_A <= keyval && keyval <= GDK_Z) {	/* A-Z */
	return tab_az[keyval - GDK_A];
    } else if (GDK_0 <= keyval && keyval <= GDK_9) {	/* 0-9 */
	return tab_09[keyval - GDK_0];
    } else if (GDK_KP_0 <= keyval && keyval <= GDK_KP_9) {	/* KP_0-KP_9 */
	return tab_09[keyval - GDK_KP_0];
    } else if (GDK_F1 <= keyval && keyval <= GDK_F6) {	/* F1-F6 */
	return tab_az[keyval - GDK_F1];
    } else {
	switch (keyval) {
	case GDK_KP_Up:
	case GDK_Up:
	    return 49;

	case GDK_KP_Left:
	case GDK_Left:
	    return 50;

	case GDK_KP_Down:
	case GDK_Down:
	    return 51;

	case GDK_KP_Right:
	case GDK_Right:
	    return 52;

	case GDK_space:
	    return 15;

	case GDK_KP_Decimal:
	case GDK_period:
	case GDK_comma:
	case GDK_semicolon:
	    return 23;

	case GDK_KP_Enter:
	case GDK_Return:
	    return 7;

	case GDK_plus:
	case GDK_KP_Add:
	    return 6;

	case GDK_minus:
	case GDK_KP_Subtract:
	    return 5;

	case GDK_asterisk:
	case GDK_KP_Multiply:
	    return 4;

	case GDK_slash:
	case GDK_KP_Divide:
	    return 3;

	case GDK_Shift_L:
	    return 61;

	case GDK_Control_L:
	    return 62;

	case GDK_Tab:
	    return 60;

	case GDK_BackSpace:
	    return 1;

	case GDK_Escape:
	    return 63;

	default:
	    return 0;
	}
    }
}
