#include <stdio.h>
#include "../include/hpapine_gui.h"
#include "gui_x11.h"


static unsigned long long keystate;
static char KeysymToCode(KeySym ks);



void gui_getmatrix(unsigned long long *km)
{
    gui_run();			/* ensure the gui is properly initialized */
    *km = keystate;
}

void keysym_release(KeySym ks)
{
    char code = KeysymToCode(ks);
    if (code == 0)
	return;
    keystate = keystate & (~(1LL << code));
}

void keysym_press(KeySym ks)
{
    char code = KeysymToCode(ks);
    if (code == 0)
	return;
    keystate = keystate | (1LL << code);
}

void keysym_releaseall(void)
{
    keystate = 0LL;
}


static char KeysymToCode(KeySym ks)
{
    const static char tab_09[] = {
	31,			/* 0 */
	30,			/* 1 */
	22,			/* 2 */
	14,			/* 3 */
	29,			/* 4 */
	21,			/* 5 */
	13,			/* 6 */
	28,			/* 7 */
	20,			/* 8 */
	19			/* 9 */
    };
    const static char tab_az[] = {
	41,			/* A */
	42,			/* B */
	43,			/* C */
	44,			/* D */
	45,			/* E */
	46,			/* F */
	47,			/* G */
	53,			/* H */
	54,			/* I */
	55,			/* J */
	57,			/* K */
	58,			/* L */
	33,			/* M */
	25,			/* N */
	17,			/* O */
	9,			/* P */
	34,			/* Q */
	26,			/* R */
	18,			/* S */
	10,			/* T */
	2,			/* U */
	35,			/* V */
	27,			/* W */
	19,			/* X */
	11,			/* Y */
	3			/* Z */
    };

    if (XK_a <= ks && ks <= XK_z) {	/* a-z */
	return tab_az[ks - XK_a];
    } else if (XK_A <= ks && ks <= XK_Z) {	/* A-Z */
	return tab_az[ks - XK_A];
    } else if (XK_0 <= ks && ks <= XK_9) {	/* 0-9 */
	return tab_09[ks - XK_0];
    } else if (XK_KP_0 <= ks && ks <= XK_KP_9) {	/* KP_0-KP_9 */
	return tab_09[ks - XK_KP_0];
    } else if (XK_F1 <= ks && ks <= XK_F6) {	/* F1-F6 */
	return tab_az[ks - XK_F1];
    } else {
	switch (ks) {
	case XK_KP_Up:
	case XK_Up:
	    return 49;

	case XK_KP_Left:
	case XK_Left:
	    return 50;

	case XK_KP_Down:
	case XK_Down:
	    return 51;

	case XK_KP_Right:
	case XK_Right:
	    return 52;

	case XK_space:
	    return 15;

	case XK_KP_Decimal:
	case XK_period:
	case XK_comma:
	case XK_semicolon:
	    return 23;

	case XK_KP_Enter:
	case XK_Return:
	    return 7;

	case XK_plus:
	case XK_KP_Add:
	    return 6;

	case XK_minus:
	case XK_KP_Subtract:
	    return 5;

	case XK_asterisk:
	case XK_KP_Multiply:
	    return 4;

	case XK_slash:
	case XK_KP_Divide:
	    return 3;

	case XK_Shift_L:
	    return 61;

	case XK_Control_L:
	    return 62;

	case XK_Tab:
	    return 60;

	case XK_BackSpace:
	    return 1;

	case XK_Escape:
	    return 63;

	default:
	    return 0;
	}
    }
}
