#include <hpstdlib.h>
#include "hpgraphics.h"
#include "hpgpriv.h"

hpg_font_t *hpg_alloc_font(char *buffer, int count, int height, int advance)
{
    hpg_init();

    hpg_font_t *font = (hpg_font_t *) malloc(sizeof(hpg_font_t));
    if (font == NULL) return NULL;

    font->buffer = buffer;
    font->count = count;
    font->height = height;
    font->advance = advance;

    return font;
}

void hpg_free_font(hpg_font_t *font)
{
    hpg_init();
    free(font);
}



hpg_pattern_t *hpg_alloc_pattern(char *buffer, int height, int fixed)
{
    hpg_init();

    hpg_pattern_t *pat = (hpg_pattern_t *) malloc(sizeof(hpg_pattern_t));
    if (pat == NULL) return NULL;

    pat->buffer = buffer;
    pat->height = height;
    pat->fixed = fixed;

    return pat;
}

void hpg_free_pattern(hpg_pattern_t *pattern)
{
    hpg_init();
    free(pattern);
}


hpg_t *hpg_alloc_mono_image(int width, int height)
{
    hpg_init();

    int fbwidth = (width + 7) >> 3; /*  width of a line in bytes */
    char *fb = (char *) malloc(fbwidth * height);
    if (fb == NULL) return NULL;
    hpg_t *img = (hpg_t *) malloc(sizeof(hpg_t));
    if (img == NULL)
    {
        free(fb);
        return NULL;
    }

    img->fb = img->drawfb = fb;
    img->drawfn = _hpg_mono_draw;
    img->getfn = _hpg_mono_get;
    img->width = width;
    img->height = height;
    img->fbwidth = fbwidth;
    img->clipx1 = img->clipy1 = 0;
    img->clipx2 = width;
    img->clipy2 = height;
    img->mode = HPG_MODE_PAINT;
    img->color = HPG_COLOR_BLACK;
    img->pattern = NULL;
    img->font = NULL;

    return img;
}


hpg_t *hpg_alloc_gray4_image(int width, int height)
{
    hpg_init();

    int fbwidth = (width + 3) >> 2; /*  width of a line in bytes */
    char *fb = (char *) malloc(fbwidth * height);
    if (fb == NULL) return NULL;
    hpg_t *img = (hpg_t *) malloc(sizeof(hpg_t));
    if (img == NULL)
    {
        free(fb);
        return NULL;
    }

    img->fb = img->drawfb = fb;
    img->drawfn = _hpg_gray4_draw;
    img->getfn = _hpg_gray4_get;
    img->width = width;
    img->height = height;
    img->fbwidth = fbwidth;
    img->clipx1 = img->clipy1 = 0;
    img->clipx2 = width;
    img->clipy2 = height;
    img->mode = HPG_MODE_PAINT;
    img->color = HPG_COLOR_BLACK;
    img->pattern = NULL;
    img->font = NULL;

    return img;
}


hpg_t *hpg_alloc_gray16_image(int width, int height)
{
    hpg_init();

    int fbwidth = (width + 1) >> 1; /*  width of a line in bytes */
    char *fb = (char *) malloc(fbwidth * height);
    if (fb == NULL) return NULL;
    hpg_t *img = (hpg_t *) malloc(sizeof(hpg_t));
    if (img == NULL)
    {
        free(fb);
        return NULL;
    }

    img->fb = img->drawfb = fb;
    img->drawfn = _hpg_gray16_draw;
    img->getfn = _hpg_gray16_get;
    img->width = width;
    img->height = height;
    img->fbwidth = fbwidth;
    img->clipx1 = img->clipy1 = 0;
    img->clipx2 = width;
    img->clipy2 = height;
    img->mode = HPG_MODE_PAINT;
    img->color = HPG_COLOR_BLACK;
    img->pattern = NULL;
    img->font = NULL;

    return img;
}

void hpg_free_image(hpg_t *img)
{
    hpg_init();

    free((char *) img->fb);
    free(img);
}
