#include <stddef.h>
#include "hpgraphics.h"
#include "hpgpriv.h"



void hpg_draw_pixel(int x, int y)
{
    _hpg_draw(hpg_stdscreen, x, y, 1, -1, -1);
}


void hpg_draw_pixel_on(hpg_t *g, int x, int y)
{
    hpg_init();
    _hpg_draw(g, x, y, 1, -1, -1);
}

void hpg_draw_line(int x1, int y1, int x2, int y2)
{
    hpg_draw_line_on(hpg_stdscreen, x1, y1, x2, y2);
}

void hpg_draw_line_on(hpg_t *g, int x1, int y1, int x2, int y2)
{
    hpg_init();

    if (y1 == y2)
    {
        int sx, len;
        if (x1 > x2)
        {
            sx = x2;
            len = x1 - x2 + 1;
        }
        else
        {
            sx = x1;
            len = x2 - x1 + 1;
        }

        _hpg_draw(g, sx, y1, len, -1, -1);
    }
    else
    {
        int dx, dy;
        int numpixels;
        int dinc1, dinc2, xinc1, xinc2, yinc1, yinc2;

        int x, y, d;
        int px, py, sx;

        int i;

        dx = (x1 > x2) ? (x1 - x2) : (x2 - x1);
        dy = (y1 > y2) ? (y1 - y2) : (y2 - y1);

        if (dx > dy)
        {
            numpixels = dx + 1;
            d = (2 * dy) - dx;
            dinc1 = 2 * dy;
            dinc2 = (dy - dx) * 2;
            xinc1 = 1;
            xinc2 = 1;
            yinc1 = 0;
            yinc2 = 1;
        }
        else
        {
            numpixels = dy + 1;
            d = (2 * dx) - dy;
            dinc1 = 2 * dx;
            dinc2 = (dx - dy) * 2;
            xinc1 = 0;
            xinc2 = 1;
            yinc1 = 1;
            yinc2 = 1;
        }

        if (x1 > x2)
        {
            xinc1 = -xinc1;
            xinc2 = -xinc2;
        }

        if (y1 > y2)
        {
            yinc1 = -yinc1;
            yinc2 = -yinc2;
        }

        x = x1;
        y = y1;

        sx = x;

        for (i = 0; i < numpixels; i++)
        {
            px = x;
            py = y;

            if (d < 0)
            {
                d += dinc1;
                x += xinc1;
                y += yinc1;
            }
            else
            {
                d += dinc2;
                x += xinc2;
                y += yinc2;
            }

            if (y != py)
            {
                if (xinc2 < 0) _hpg_draw(g, px, py, sx - px + 1, -1, -1);
                else _hpg_draw(g, sx, py, px - sx + 1, -1, -1);
                sx = x;
            }
        }

        if (x != sx)
        {
            if (xinc2 < 0) _hpg_draw(g, x + 1, y, sx - x, -1, -1);
            else _hpg_draw(g, sx, y, x - sx, -1, -1);
        }
    }
}

void hpg_draw_rect(int x1, int y1, int x2, int y2)
{
    hpg_draw_rect_on(hpg_stdscreen, x1, y1, x2, y2);
}

void hpg_draw_rect_on(hpg_t *g, int x1, int y1, int x2, int y2)
{
    hpg_init();

    if (x2 < x1)
    {
        int temp = x1;
        x1 = x2;
        x2 = temp;
    }

    if (y2 < y1)
    {
        int temp = y1;
        y1 = y2;
        y2 = temp;
    }

    _hpg_draw(g, x1, y1, x2 - x1 + 1, -1, -1);
    if (y2 - y1 == 0) return;

    _hpg_draw(g, x1, y2, x2 - x1 + 1, -1, -1);
    if (y2 - y1 == 1) return;

    int y;
    for (y = y1 + 1; y < y2; y++)
    {
        _hpg_draw(g, x1, y, 1, -1, -1);
        if (x2 != x1) _hpg_draw(g, x2, y, 1, -1, -1);
    }
}

void hpg_draw_circle(int cx, int cy, int r)
{
    hpg_draw_circle_on(hpg_stdscreen, cx, cy, r);
}


void hpg_draw_circle_on(hpg_t *g, int cx, int cy, int r)
{
    hpg_init();

    int x, y, d;

    d = 3 - (2 * r);
    x = 0;
    y = r;

    while (x <= y)
    {
        _hpg_draw(g, cx + x, cy + y, 1, -1, -1);
        _hpg_draw(g, cx + x, cy - y, 1, -1, -1);

        if (x != 0)
        {
            _hpg_draw(g, cx - x, cy + y, 1, -1, -1);
            _hpg_draw(g, cx - x, cy - y, 1, -1, -1);
        }

        if (x != y)
        {
            _hpg_draw(g, cx + y, cy + x, 1, -1, -1);
            _hpg_draw(g, cx - y, cy + x, 1, -1, -1);

            if (x != 0)
            {
                _hpg_draw(g, cx + y, cy - x, 1, -1, -1);
                _hpg_draw(g, cx - y, cy - x, 1, -1, -1);
            }
        }

        if (d < 0)
        {
            d += 4 * x + 6;
        }
        else
        {
            d += 4 * (x - y) + 10;
            y--;
        }

        x++;
    }
}

void hpg_draw_polygon(int vx[], int vy[], int len)
{
    hpg_draw_polygon_on(hpg_stdscreen, vx, vy, len);
}

void hpg_draw_polygon_on(hpg_t *g, int vx[], int vy[], int len)
{
    hpg_init();
    _hpg_poly(g, vx, vy, len, 0);
}

void hpg_draw_letter(char a, int x, int y)
{
    hpg_draw_letter_on(hpg_stdscreen, a, x, y);
}

void hpg_draw_letter_on(hpg_t *g, char a, int x, int y)
{
    hpg_init();

    if (g->font == NULL) g->font = hpg_get_minifont();

    if (a >= g->font->count) return;

    struct hpg_pattern charpat = {
        NULL, g->font->height, 0
    };

    struct hpg_pattern *oldpat = g->pattern;
    g->pattern = &charpat;

    charpat.buffer = g->font->buffer + (g->font->height * a);
    hpg_fill_rect_on(g, x, y, x + g->font->advance - 1, y + g->font->height - 1);

    g->pattern = oldpat;
}

void hpg_draw_text(char *s, int x, int y)
{
    hpg_draw_text_on(hpg_stdscreen, s, x, y);
}

void hpg_draw_text_on(hpg_t *g, char *s, int x, int y)
{
    hpg_init();

    if (g->font == NULL) g->font = hpg_get_minifont();

    struct hpg_pattern charpat = {
        NULL, g->font->height, 0
    };

    struct hpg_pattern *oldpat = g->pattern;
    g->pattern = &charpat;

    int startx = x;
    while (*s != '\0')
    {
        if (*s == '\n')
        {
            x = startx;
            y += g->font->height;
        }
        else
        {
            if (*s < g->font->count)
            {
                charpat.buffer = g->font->buffer + (g->font->height * (*s));
                hpg_fill_rect_on(g, x, y,
                    x + g->font->advance - 1, y + g->font->height - 1);

                x += g->font->advance;
            }
        }

        s++;
    }

    g->pattern = oldpat;
}
