#include <hpsys.h>
#include "hpgraphics.h"
#include "hpgpriv.h"

void hpg_set_color(hpg_t *g, unsigned char color)
{
    hpg_init();
    g->color = color;
}

void hpg_set_font(hpg_t *g, hpg_font_t *font)
{
    hpg_init();
    g->font = font;
}

void hpg_set_indicator(unsigned char indicator, unsigned char color)
{
    hpg_init();

    /*
     * A special graphics device to be used for turning the screen indicators
     * on or off.  This device is identical to the main graphics device except
     * for the modified clipping rectangle, which allows writing to the
     * off-screen segments corresponding to the indicators.
     *
     * The locations of the indicator pixels are:
     *
     * (131, 0) - Remote
     * (131, 1) - Left Shift
     * (131, 2) - Right Shift
     * (131, 3) - Alpha
     * (131, 4) - Low Battery
     * (131, 5) - Wait
     */
    static hpg_t indicator_hpg;
    indicator_hpg = *hpg_stdscreen;
    indicator_hpg.clipx1 = 0;
    indicator_hpg.clipx2 = 160;
    indicator_hpg.clipy1 = 0;
    indicator_hpg.clipy2 = 80;

    indicator_hpg.color = color;
    hpg_draw_pixel_on(&indicator_hpg, 131, indicator);
}

void hpg_set_mode(hpg_t *g, unsigned char mode)
{
    hpg_init();
    g->mode = mode;
}

void hpg_set_mode_mono(int dbuf)
{
    hpg_init();

    char *plane1;
    char *plane2;

    if (dbuf)
    {
        plane1 = (char *) sys_phys_malloc(1720 * 2);
        plane2 = plane1 + 1720;
    }
    else
    {
        plane2 = plane1 = DEFAULT_PLANE;
    }

    /*  PARAMETERS CHANGED BY CLAUDIO */
    _hpg_set_mode(MODE_MONO,plane1, plane2, _hpg_mono_draw, _hpg_mono_get);
}

void hpg_set_mode_gray4(int dbuf)
{
    hpg_init();

    char *plane1;
    char *plane2;

    if (dbuf)
    {
        plane1 = (char *) sys_phys_malloc(3420 * 2);
        plane2 = plane1 + 3420;
    }
    else
    {
        plane2 = plane1 = (char *) sys_phys_malloc(3420);
    }

    /*  PARAMETERS CHANGED BY CLAUDIO */
    _hpg_set_mode(MODE_4GRAY,plane1, plane2,
        _hpg_gray4_draw, _hpg_gray4_get);
}

void hpg_set_mode_gray16(int dbuf)
{
    hpg_init();

    char *plane1;
    char *plane2;

    if (dbuf)
    {
        plane1 = (char *) sys_phys_malloc(6820 * 2);
        plane2 = plane1 + 6820;
    }
    else
    {
        plane2 = plane1 = (char *) sys_phys_malloc(6820);
    }

    /*  PARAMETERS CHANGED BY CLAUDIO */
    _hpg_set_mode(MODE_16GRAY,plane1, plane2,
                _hpg_gray16_draw, _hpg_gray16_get);
}

void hpg_set_pattern(hpg_t *g, hpg_pattern_t *pattern)
{
    hpg_init();
    g->pattern = pattern;
}
