#ifndef _HPAPINE_GUI_H
#define _HPAPINE_GUI_H	1

/******************************************************************************
 * Here are the only global variables and routines you need to implement
 * the GUI.
 ******************************************************************************/

/*
 * A pointer to the graphic buffer.
 * You may not use the content of the buffer directly. Use the 
 * screen_getpixlum() routine instead (see below).
 *
 * You need to implement gui_change_screenbuf() (see below).
 */
extern unsigned char *screen_buf;


/* This routine is implemented in sys/screen.c
 * You'll have to use this to get the luminosity of a pixel.
 * - It returns an integer: 0 is black and 15 is white.
 * - x must range between 0 and 130
 * - y must range between 0 and 79
 */
extern int screen_getpixlum(int x, int y);



/******************************************************************************
 * Here are the variables and routines you have to implement:
 ******************************************************************************/

/*
 * The name of your module, e.g. "GTK+" or "Xlib"
 */
extern const char *gui_name;


/* Init and launch the gui if not already inited.
 * This routine may be called *each time* a HPGCC graphic routine is
 * called.
 * For example, it could open a window and launch a thread that
 * refreshes continuously the content of that window.
 */
extern void gui_run(void);


/* This set a new memory area as the screen buffer, i.e. something like:
 *
 * void gui_change_screenbuf(unsigned char *new)
 * {
 *   get_mutex_on_screen_buf();
 *   screen_buf = new;
 *   release_mutex_on_screen_buf();
 * }
 *
 *
 * This is needed to avoid possible race conditions when using threads.
 */
extern void gui_change_screenbuf(unsigned char *new);


/* Beep */
extern void gui_bell(int tone, int duration);


/* This must return an unsigned long long integer that contains
 * the state of the keyboard. This long long integer follow the same
 * conventions that the keymatrix struct of HPGCC's API (see the
 * include/hpkeyb49.h file).
 */
extern void gui_getmatrix(unsigned long long *km);


#endif
