#define DONT_USE_HPGCC_LIBC
#include <stdarg.h>
#include <math.h>

#include "hpstdio.h"
#include "hpconio.h"
#include "hpscreen.h"
#include "hpstring.h"
#include "hpmath.h"

#include "../include/hpapine_gui.h"

static char *out_copy(char *t, char *s)
{
    int slen;
    char *r = t;

    strncpy(t, s, slen = strlen(s));
    return r + slen;
}


int hpgcc_vsprintf(char *out, const char *fmt, va_list argp)
{
    const char *p;

    char fmtbuf[256];
    char padbuf[256];
    char hexbuf[16];
    char *s = NULL;

    int lead_zero, pad, minus, entry, fpdigits, fpscale;

    for (p = fmt; *p != '\0'; p++) {
	if (*p != '%') {
	    *out++ = *p;
	    continue;
	}

	lead_zero = pad = minus = fpdigits = fpscale = 0;
	entry = 1;

	p++;

	if ((minus = *p == '-'))
	    p++;

	while (isdigit(*p)) {
	    if (entry) {
		lead_zero = (*p == '0');
		entry = 0;
	    }
	    pad = 10 * pad + (*p & 0xF);
	    p++;
	}

	if ((fpscale = *p == '.')) {
	    p++;
	    while (isdigit(*p)) {
		fpdigits = 10 * fpdigits + (*p & 0xF);
		p++;
	    }
	}

	if (*p == 's') {
	    if (!(s = va_arg(argp, char *)))
		 s = "<NULL>";
	    out =
		out_copy(out, pad ? xpad(padbuf, s, pad, minus, ' ') : s);
	    goto t_break;
	}

	if (*p == 'd' || *p == 'i') {

	    s = itoa(va_arg(argp, int), fmtbuf, 10);
	    goto numeric_puts;
	}

	if (*p == 'c') {
	    *out++ = (char) va_arg(argp, int);
	    goto t_break;
	}

	if (*p == 'f' || *p == 'g' || *p == 'e' || *p == 'E') {
	    int old_fpdigits = get_decimal_digits();

	    if (fpscale)
		set_decimal_digits(fpdigits);

	    s = __dtoa(	dround(	va_arg(argp, double),
				fpscale ? fpdigits : old_fpdigits),
			fmtbuf,
			*p);

	    if (fpscale)
		set_decimal_digits(old_fpdigits);
	    goto numeric_puts;
	}

	if (*p == 'x' || *p == 'X') {
	    s = utoa(va_arg(argp, unsigned), fmtbuf, 16);
	    if (*p == 'X')
		s = upper(hexbuf, fmtbuf);
	    goto numeric_puts;
	}

	if (*p == 'p') {
	    s = utoa((unsigned) va_arg(argp, void *), fmtbuf, 16);
	    goto numeric_puts;
	}

	if (*p == 'u' || *p == 'b') {
	    s = utoa(va_arg(argp, unsigned), fmtbuf, *p == 'b' ? 2 : 10);
	    /*
	    puts(pad ? xpad(padbuf,s,pad,!minus,lead_zero ? '0' : ' ') : s);
	    */
	    goto numeric_puts;
	}

	if (*p == 'U') {

	    s = ulltoa(va_arg(argp, unsigned long long), fmtbuf, 10);
	    goto numeric_puts;
	}

	if (*p == 'L') {
	    s = lltoa(va_arg(argp, long long), fmtbuf, 10);
	    goto numeric_puts;
	}

	if (*p == '%') {
	    *out++ = '%';
	    goto t_break;
	}

      numeric_puts:
	out =
	    out_copy(out,
		     pad ? xpad(padbuf, s, pad, !minus,
				lead_zero ? '0' : ' ') : s);
	goto t_break;

      t_break:
        ;
    }

    *out = '\0';
    return strlen(out);

}

int hpgcc_sprintf(char *out, const char *fmt, ...)
{
    int ret;
    va_list argp;
    va_start(argp, fmt);

    ret = hpgcc_vsprintf(out,fmt,argp);
    
    va_end(argp);
    return ret;
}



int hpgcc_fprintf(FILE * f, const char *fmt, ...)
{
    va_list argp;
    char *out;
    int ret, ret1;

    if (!(out = (char *) malloc(4096)))
	sys_error("fatal:can't malloc in fprintf", -1);

    va_start(argp, fmt);

    ret = hpgcc_vsprintf(out, fmt, argp);
    ret1 = hpgcc_fputs(out, f);

    free(out);
    va_end(argp);
    return ret1 == 0 ? ret : EOF;
}

int hpgcc_printf(const char *fmt, ...)
{
    va_list argp;
    int ret;
    char *out;

    if (!(out = (char *) malloc(4096)))
	sys_error("fatal:can't malloc in printf", -1);

    va_start(argp, fmt);

    ret = hpgcc_vsprintf(out, fmt, argp);

    const unsigned char *p;
    p = (unsigned char *) out;
    while (*p) {
	hpgcc_putchar(*p);
	p++;
    }

    free(out);
    va_end(argp);
    return ret;
}



int hpgcc_fputs(const char *s, FILE * f)
{
    while (*s) {
	if (*s != fputc((int) *s++, f))
	    return EOF;
    }
    return 0;
}

int hpgcc_fputc(int c, FILE * f)
{
    if (f == stdout || f == stderr) {
	hpgcc_putc(c);
	return (int) c;
    }

    int ret;
    char b = (char) c;

    ret = kos_fwrite(f, (P_BYTE) & b, 1);
    return (ret == 0) ? EOF : c;
}

static void _scroll(void)
{
    char *ptr = getLCDPointer(0, 0);

    /* 20 bytes per line in BW mode */
    memcpy(ptr, ptr + __ch_h * 20, 12 * __ch_h * 20);

    /* the last character line is the 12th */
    memset(ptr + 12 * __ch_h * 20, 0, (80 - 12 * __ch_h) * 20);
}


static void putcon_stdscreen(int c)
{
    gui_run();
    drawBlock4bit(get_minifont() + c*3, 6, 
       __pos_x * __ch_w, __pos_y * __ch_h);
}

static void putcon(int c)
{
    putcon_stdscreen(c);
}


void hpgcc_putc(int c)
{
    putcon(c);
    ++__pos_x;
    if (__pos_x >= __scr_w) {
	__pos_x = 0;
	++__pos_y;
	if (__pos_y >= __scr_h) {
	    _scroll();
	    --__pos_y;
	}
    }
}

static int putchar_console(int c)
{
    if (c < 5 || c >= 17)
	hpgcc_putc(c);

    switch (c) {
    case 5:
	clreol();
	return c;
    case 7:
	beep();
	return c;
    case 8:
	--__pos_x;
	if (__pos_x < 0) {
	    __pos_x = __scr_w;
	    if (__pos_y > 0)
		__pos_y--;
	}
	putcon(c);
	return c;
    case 9:
	/* TRAP: works for TABSIZE being a power of 2 only! */
	{
	    int t = __pos_x & (_TAB_SIZE - 1);
	    if (!t)
		t = _TAB_SIZE;
	    /* No NULL tab steps! */
	    while (t--)
		hpgcc_putc(32);
	    return c;
	}
    case 10:
	++__pos_y;
	if (__auto_cr)
	    __pos_x = 0;
	if (__pos_y >= __scr_h) {
	    _scroll();
	    --__pos_y;
	}
	return c;
    case 11:
	hpgcc_putc(c);
	return c;
    case 12:
	_clear_screen();
	__pos_y = __pos_x = 0;
	return c;
    case 13:
	__pos_x = 0;
	return c;
    }
    return c;
}

int hpgcc_putchar(int c)
{
    return putchar_console(c);
}



int hpgcc_puts(const char *str)
{
    const char *p = str;

    while (*p) {
	hpgcc_putchar(*p);
	p++;
    }
    hpgcc_putchar('\n');

    return 1;
}
