#include "hpstring.h"
#include "hpmath.h"
#include "hpconio.h"
#include "hpstdio.h"

/*
 * dtoa()
 */

#define MAXDIGITS 15
#define TINY_EXP 1.0e-15
#define BIG_EXP 1.0e15

#define DEFAULT_EXPO_SYM 'E'
#define NORMALIZED_MANTISSA 0

char *__dtoa(double x, char *buf, char expo_sym)
{
    int digits, max_digits;
    int minus, expo, write_expo;
    char fraction[MAXDIGITS + 1];
    char integral[MAXDIGITS + 1];
    char exps[8];

    digits = minus = expo = write_expo = 0;
    max_digits = min(get_decimal_digits(), MAXDIGITS);
    memset(fraction, '0', max_digits);
    fraction[max_digits] = '\0';
    strcpy(integral, "0");


    if (x != 0.0) {
	int expo2;
	double ip;
	LONGLONG lip;
	LONGLONG nlp, lpow;
	int k;
	LONGLONG lfrac;

	if ((minus = x < 0.0))
	    x = -x;

	frexp(x, &expo2);
	expo = (int) (expo2 * M_LN2 / M_LN10);
	write_expo = (expo_sym == 'e' || expo_sym == 'E'
		      || (expo_sym == 'g'
			  && (x >= BIG_EXP || x <= TINY_EXP
			      || ((abs(expo)) > max_digits))));
	if (write_expo)
	    x *= i10powx(-expo);


	modf(x, &ip);
	lip = (LONGLONG) ip;
	lltoa(lip, integral, 10);

	nlp = 1;
	for (k = 0; k < max_digits - 1; k++)
	    nlp *= 10;
	lpow = 10 * nlp;

	lfrac = (LONGLONG) dround(x * lpow, 0) - lip * lpow;

	if (lfrac != 0LL) {
#if NORMALIZED_MANTISSA
/*
	    // Force a normalized exponential representation;
	    // x.ddd Eeee , where x > 0

	    // This is expensive in terms of space.
	    // It adds an 2+ K overhead for the LONGLONG division
*/
	    if (write_expo && lip == 0LL) {
		LONGLONG nip;

		nip = lfrac / nlp;
		lfrac = 10 * (lfrac - nip * nlp);
		lltoa(nip, integral, 10);
		expo--;
	    }
#endif
	    char fmt[8];
	    hpgcc_sprintf(fmt, "%%0%dL", max_digits);
	    hpgcc_sprintf(fraction, fmt, lfrac);
	}

    }

    hpgcc_sprintf(buf, "%s%s%s%s", minus ? "-" : "", integral,
		  max_digits ? get_decimal_separator() : "", fraction);

    if (write_expo) {
	int m;
	if ((m = expo < 0))
	    expo = -expo;
	if (!(expo_sym == 'e' || expo_sym == 'E'))
	    expo_sym = DEFAULT_EXPO_SYM;
	hpgcc_sprintf(exps, "%c%c%03d", expo_sym, m ? '-' : '+', expo);
	strcat(buf, exps);
    }

    return buf;
}


/*
 * itoa()
 */

char *itoa(int z, char *buff, int base)
{
    char *ret = buff;
    if (z < 0) {
	*buff = '-';
	++buff;
	z = -z;
    }
    utoa(z, buff, base);
    return ret;
}


/*
 * ulltoa() and lltoa()
 */

char *ulltoa(ULONGLONG z, char *buff, int base)
{
    ULONGLONG powers[64], *ptr = powers;
    ULONGLONG pw = 1;
    char *ret = buff;
    int digit;
/* GENERATE LIST OF POWERS OF THE BASE */
    do {
	*ptr = pw;
	++ptr;
/* DETECT OVERFLOW! */
	if (((pw >> 32) * base) >> 32)
	    break;
	pw *= base;
    }
    while (pw <= z);

    for (--ptr; ptr >= powers; ptr--) {
	digit = 0;
	while (z >= *ptr) {
	    ++digit;
	    z -= *ptr;
	}
	*buff = (digit > 9) ? (digit + 87) : (digit + 48);
	++buff;
    }

    *buff = 0;

    return ret;
}


char *lltoa(LONGLONG z, char *buff, int base)
{
    char *ret = buff;
    if (z < 0) {
	*buff = '-';
	++buff;
	z = -z;
    }
    ulltoa(z, buff, base);
    return ret;
}

/*
 * utoa()
 */

char *utoa(unsigned int z, char *buff, int base)
{
    unsigned powers[32], *ptr = powers;
    unsigned pw = 1;
    char *ret = buff;
    int digit;
/* GENERATE LIST OF POWERS OF THE BASE */
    do {
	*ptr = pw;
	++ptr;
/* DETECT OVERFLOW! */
	if (((pw >> 16) * base) >= 0x10000)
	    break;
	pw *= base;
    }
    while (pw <= z);

    for (--ptr; ptr >= powers; ptr--) {
	digit = 0;
	while (z >= *ptr) {
	    ++digit;
	    z -= *ptr;
	}
	*buff = (digit > 9) ? (digit + 87) : (digit + 48);
	++buff;
    }

    *buff = 0;
    return ret;
}
