#define DONT_USE_HPGCC_LIBC
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "rplapi.h"
#include "saturn.h"
#include "../include/_hpapine_version.h"	/* hpapine_print_version() */

rpl_stack_t rpl_stack = { 0, 0, NULL };



static void _print_atexit(void)
{
    static int init = 0;
    if (init)
	return;
    atexit(rpl_stack_print);
    init = 1;
}

static void _stack_init(void)
{
    static int init = 0;
    if (init)
	return;

    hpapine_print_version();

    fprintf(stderr, "--- Waiting for stack input on stdin...\n");

    rpl_t_OBJ *pobj;
    while (1) {
	pobj = rpl_fread_obj(stdin);
	if (!pobj) {
	    break;
	}
	rpl_stack_push(pobj);
    }

    fprintf(stderr, "--- Stack reading: done.\n\n");

    init = 1;
    rpl_stack_print();
    _print_atexit();
}


int rpl_stack_depth(void)
{
    _stack_init();
    return rpl_stack.depth;
}


void rpl_stack_push(rpl_t_OBJ * obj)
{
    _print_atexit();

    if (rpl_stack.depth == rpl_stack.allocated) {
	rpl_stack.allocated += 16;
	rpl_stack.data =
	    (rpl_t_OBJ *) realloc(rpl_stack.data,
				  rpl_stack.allocated *
				  sizeof(rpl_t_OBJ *));
	if (!rpl_stack.data) {
	    perror(__FUNCTION__);
	    exit(1);
	}
    }

    rpl_stack.data[rpl_stack.depth] = *obj;
    rpl_stack.depth++;
}

int rpl_stack_dropn(int n)
{
    int i = n;
    while (i && rpl_stack_drop())
	i--;
    return n - i;
}

int rpl_stack_drop(void)
{
    _stack_init();
    if (rpl_stack.depth == 0)
	return 0;

    rpl_stack.depth--;
    return 1;
}

rpl_t_OBJ *rpl_stack_pop(void)
{
    _stack_init();
    if (rpl_stack.depth == 0)
	return NULL;

    rpl_t_OBJ *pobj = &rpl_stack.data[rpl_stack.depth - 1];
    rpl_stack_drop();
    return pobj;
}

rpl_t_OBJ *rpl_stack_pick(unsigned int level)
{
    _stack_init();
    if (rpl_stack.depth < level || level == 0)
	return NULL;

    rpl_t_OBJ *pobj = &(rpl_stack.data[rpl_stack.depth - level]);
    return pobj;
}


/*********************************************
 * RPL object printing
 *********************************************/

extern void rpl_fprint_obj(FILE *f, rpl_t_OBJ * obj)
{
    if (!obj) {
	fprintf(f, "<NULL>");
	return;
    }

    switch (obj->prolog) {
    case SAT_DOREAL:
	fprintf(f, "%g", *((rpl_t_REAL *) obj->ptr));
	break;
    case SAT_DOCMP:
	fprintf(f, "(%f %f)", ((rpl_t_CMPLX *) obj->ptr)->re,
	       ((rpl_t_CMPLX *) obj->ptr)->im);
	break;
    case SAT_DOLIST:
	{
	    rpl_t_LIST *cur = (rpl_t_LIST *) obj->ptr;
	    fprintf(f, "{");
	    while (cur) {
		fprintf(f, " ");
		if (cur->obj)
		    rpl_fprint_obj(f, cur->obj);
		/* else assert(cur->next == NULL); */
		cur = cur->next;
	    }
	    fprintf(f, " }");
	}
	break;
    case SAT_DORRP:
	{
	    rpl_t_DIR *cur = (rpl_t_DIR *) obj->ptr;
	    fprintf(f, "DIR \n");
	    while (cur) {
		/* Print the name of the variable */
		fprintf(f, "     ");
		if (cur->obj)
		    rpl_fprint_obj(f, cur->obj);
		cur = cur->next;	/* assert(cur); */

		/* Print the content of the variable */
		fprintf(f, " ");
		if (cur->obj)
		    rpl_fprint_obj(f, cur->obj);
		cur = cur->next;

		fprintf(f, "\n");
	    }
	    fprintf(f, "    END");
	}
	break;
    case SAT_DOHXS:
	{
	    char *p = *((rpl_t_HXS *) obj->ptr);
	    fprintf(f, "# ");
	    while (*p) {
		char c = *p++;
		fprintf(f, "%c", isdigit(c) ? c : c - 1 - '9' + 'a');
	    }
	}
	break;
    case SAT_DOINT:
	{
	    char *p = *((rpl_t_ZINT *) obj->ptr);
	    fprintf(f, "= ");
	    while (*p)
		fprintf(f, "%c", *p++);
	}
	break;
    case SAT_DOCSTR:
	fprintf(f, "\"%s\"", *((rpl_t_CSTR *) obj->ptr));
	break;
    case SAT_DOIDNT:
	fprintf(f, "'%s'", *((rpl_t_IDNT *) obj->ptr));
	break;
    default:
	fprintf(f, "<Unknown>");
	break;
    }
}

void rpl_stack_print(void)
{
    int i;
    printf("--- RPL Stack:\n\n");
    for (i = 0; i < rpl_stack.depth; i++) {
	rpl_t_OBJ obj = rpl_stack.data[i];
	printf("%2i: ", rpl_stack.depth - i);
	rpl_fprint_obj(stdout, &obj);
	printf("\n");
    }
    printf("\n\n");
}


/******************************************************************************
 * RPL object creating and destroying
 ******************************************************************************/

rpl_t_OBJ *rpl_new_real(double x)
{
    rpl_t_OBJ *pobj = (rpl_t_OBJ *) malloc(sizeof(rpl_t_OBJ));
    if (!pobj) {
	perror(__FUNCTION__);
	exit(1);
    }

    pobj->ptr = (rpl_t_REAL *) malloc(sizeof(rpl_t_REAL));
    if (!pobj->ptr) {
	perror(__FUNCTION__);
	exit(1);
    }

    pobj->prolog = SAT_DOREAL;
    *((double *) pobj->ptr) = x;
    return pobj;
}

rpl_t_OBJ *rpl_new_cmplx(double re, double im)
{
    rpl_t_OBJ *pobj = (rpl_t_OBJ *) malloc(sizeof(rpl_t_OBJ));
    if (!pobj) {
	perror(__FUNCTION__);
	exit(1);
    }

    pobj->ptr = (rpl_t_CMPLX *) malloc(sizeof(rpl_t_CMPLX));
    if (!pobj->ptr) {
	perror(__FUNCTION__);
	exit(1);
    }

    pobj->prolog = SAT_DOCMP;
    ((rpl_t_CMPLX *) pobj->ptr)->re = re;
    ((rpl_t_CMPLX *) pobj->ptr)->im = im;
    return pobj;
}

rpl_t_OBJ *rpl_new_cstr(const char *str)
{
    rpl_t_OBJ *pobj = (rpl_t_OBJ *) malloc(sizeof(rpl_t_OBJ));
    if (!pobj) {
	perror(__FUNCTION__);
	exit(1);
    }

    pobj->ptr = (rpl_t_CSTR *) malloc(sizeof(rpl_t_CSTR));
    if (!pobj->ptr) {
	perror(__FUNCTION__);
	exit(1);
    }

    pobj->prolog = SAT_DOCSTR;
    *((char **) pobj->ptr) = malloc(strlen(str) + 1);
    strcpy(*((char **) pobj->ptr), str);
    return pobj;
}

rpl_t_OBJ *rpl_new_idnt(const char *str)
{
    rpl_t_OBJ *pobj = rpl_new_cstr(str);
    pobj->prolog = SAT_DOIDNT;
    return pobj;
}

rpl_t_OBJ *rpl_new_hxs(const char *str)
{
    rpl_t_OBJ *pobj = rpl_new_cstr(str);
    pobj->prolog = SAT_DOHXS;
    return pobj;
}

rpl_t_OBJ *rpl_new_zint(const char *str)
{
    rpl_t_OBJ *pobj = rpl_new_cstr(str);
    pobj->prolog = SAT_DOINT;
    return pobj;
}

rpl_t_OBJ *rpl_new_list(void)
{
    rpl_t_OBJ *pobj = (rpl_t_OBJ *) malloc(sizeof(rpl_t_OBJ));
    if (!pobj) {
	perror(__FUNCTION__);
	exit(1);
    }

    pobj->ptr = (rpl_t_LIST *) malloc(sizeof(rpl_t_LIST));
    if (!pobj->ptr) {
	perror(__FUNCTION__);
	exit(1);
    }

    ((rpl_t_LIST *) pobj->ptr)->obj = NULL;
    ((rpl_t_LIST *) pobj->ptr)->next = NULL;
    pobj->prolog = SAT_DOLIST;
    return pobj;
}

rpl_t_OBJ *rpl_new_dir(void)
{
    rpl_t_OBJ *pobj = rpl_new_list();
    pobj->prolog = SAT_DORRP;
    return pobj;
}

void rpl_destroy_obj(rpl_t_OBJ * pobj)
{
    free(pobj->ptr);
    free(pobj);
    return;
}

/* List */

rpl_t_LIST *rpl_list_append(rpl_t_LIST * plist, rpl_t_OBJ * pobj)
{
    if (plist->obj == NULL) {
	plist->obj = pobj;
	/* assert(plist->next == NULL); */
	return plist;
    }

    rpl_t_LIST *p = plist;
    while (p->next)
	p = p->next;

    rpl_t_LIST *newelt = malloc(sizeof(rpl_t_LIST));
    if (!newelt) {
	perror(__FUNCTION__);
	exit(1);
    }
    newelt->obj = pobj;
    newelt->next = NULL;

    p->next = newelt;

    return plist;
}



/******************************************************************************
 * RPL object reading
 ******************************************************************************/

static rpl_t_OBJ *_rpl_fread_obj(FILE * f, int quiet)
{
    int c;
    while (1) {
	c = getc(f);
	if (c == EOF)
	    return NULL;

	if (c == '.' || c == '-' || isdigit(c)) {	/* REAL */
	    ungetc(c, f);
	    return rpl_fread_real(f);
	} else if (c == '(') {	/* CMPLX */
	    ungetc(c, f);
	    return rpl_fread_cmplx(f);
	} else if (c == '"') {	/* STR */
	    ungetc(c, f);
	    return rpl_fread_cstr(f);
	} else if (c == '\'') {	/* IDNT */
	    ungetc(c, f);
	    return rpl_fread_idnt(f);
	} else if (c == '#') {	/* HXS */
	    ungetc(c, f);
	    return rpl_fread_hxs(f);
	} else if (c == '=') {	/* ZINT */
	    ungetc(c, f);
	    return rpl_fread_zint(f);
	} else if (c == '{') {	/* LIST */
	    ungetc(c, f);
	    return rpl_fread_list(f);
	} else if (c == 'D') {	/* DIR */
	    ungetc(c, f);
	    return rpl_fread_dir(f);
	} else {
	    if (!isspace(c)) {
		if (!quiet)
		    fprintf(stderr, " -- Got unexpected '%c'.\n", c);
		ungetc(c, f);
		return NULL;
	    }
	}
    }

    return NULL;
}

rpl_t_OBJ *rpl_fread_obj(FILE * f)
{
    return _rpl_fread_obj(f, 0);
}

rpl_t_OBJ *rpl_fread_real(FILE * f)
{
    double x;
    fscanf(f, "%lf", &x);
    return rpl_new_real(x);
}

rpl_t_OBJ *rpl_fread_cmplx(FILE * f)
{
    double re, im;
    fscanf(f, "(%lf %lf)", &im, &re);
    return rpl_new_cmplx(re, im);
}

static rpl_t_OBJ *_rpl_fread_string(FILE * f, char delim, char end)
{
    int c = getc(f);
    if (c != delim) {
	ungetc(c, f);
	return NULL;
    }

    char *str = NULL;
    int i = 0;
    int len = 0;

    while ((c = getc(f)) != EOF) {
	if (c == end)
	    break;

	if (i >= len) {
	    len += 128;
	    str = realloc(str, len);
	    if (!str) {
		perror(__FUNCTION__);
		exit(1);
	    }
	}

	if (c == '\\')
	    if ((c = getc(f)) == EOF)
		break;
	str[i++] = c;
    }

    str = realloc(str, i + 1);
    if (!str) {
	perror(__FUNCTION__);
	exit(1);
    }

    str[i] = '\0';
    return rpl_new_cstr(str);
}

rpl_t_OBJ *rpl_fread_cstr(FILE * f)
{
    rpl_t_OBJ *pobj = _rpl_fread_string(f, '"', '"');
    /* pobj->prolog = SAT_DOCSTR; */
    return pobj;
}

rpl_t_OBJ *rpl_fread_idnt(FILE * f)
{
    rpl_t_OBJ *pobj = _rpl_fread_string(f, '\'', '\'');
    if (strlen(*((rpl_t_IDNT *) pobj->ptr)) > 127)
	fprintf(stderr, "Warning: IDNT longer than 127 chars.\n");
    pobj->prolog = SAT_DOIDNT;
    return pobj;
}

rpl_t_OBJ *rpl_fread_hxs(FILE * f)
{
    int c = getc(f);
    if (c != '#') {
	ungetc(c, f);
	return NULL;
    }

    char *str = NULL;
    int i = 0;
    int len = 0;

    while ((c = getc(f)) != EOF) {
	if (i == 0 && (c == ' ' || c == '\t'))
	    continue;
	if (!isxdigit(c)) {
	    ungetc(c, f);
	    break;
	}

	if (i >= len) {
	    len += 128;
	    str = realloc(str, len);
	    if (!str) {
		perror(__FUNCTION__);
		exit(1);
	    }
	}

	if (isdigit(c))
	    str[i] = c;
	else
	    str[i] = c - (isupper(c) ? 'A' : 'a') + 10 + '0';
	i++;
    }

    str = realloc(str, i + 1);
    if (!str) {
	perror(__FUNCTION__);
	exit(1);
    }

    str[i] = '\0';
    return rpl_new_hxs(str);
}

rpl_t_OBJ *rpl_fread_zint(FILE * f)
{
    int c = getc(f);
    if (c != '=') {
	ungetc(c, f);
	return NULL;
    }

    int i = 0;
    int len = 0;
    char *str = NULL;

    while ((c = getc(f)) != EOF) {
	if (i == 0 && (c == ' ' || c == '\t'))
	    continue;
	if (!isdigit(c) && !(i == 0 && (c == '+' || c == '-'))) {
	    ungetc(c, f);
	    break;
	}

	if (i >= len) {
	    len += 128;
	    str = realloc(str, len);
	    if (!str) {
		perror(__FUNCTION__);
		exit(1);
	    }
	}
	if (i == 0 && isdigit(c))
	    str[i++] = '+';
	str[i++] = c;
    }

    if (i == 0) {
	free(str);
	fprintf(stderr, " -- Got unexpected '%c'.\n", c);
	return NULL;
    }
    if (i == 1)
	str[i++] = '0';

    str = realloc(str, i + 1);
    if (!str) {
	perror(__FUNCTION__);
	exit(1);
    }

    str[i] = '\0';
    return rpl_new_zint(str);
}

rpl_t_OBJ *rpl_fread_list(FILE * f)
{
    int c = getc(f);
    if (c != '{') {
	ungetc(c, f);
	fprintf(stderr, " -- Got unexpected '%c'.\n", c);
	return NULL;
    }

    rpl_t_OBJ *pobj = rpl_new_list();
    rpl_t_LIST *lastelt = (rpl_t_LIST *) pobj->ptr;
    rpl_t_OBJ *tmp;

    while (1) {
	while (isspace(c = getc(f)));
	if (c == EOF)
	    break;
	ungetc(c, f);
	tmp = _rpl_fread_obj(f, 1);
	if (!tmp) {
	    if ((c = getc(f)) == '}')
		break;
	    else {
		/* FIXME: free memory */
		return NULL;
	    }
	} else
	    rpl_list_append(lastelt, tmp);
    }

    return pobj;
}


rpl_t_OBJ *rpl_fread_dir(FILE * f)
{
    int i, c;

    for (i = 0; i < 3; i++) {
	c = getc(f);
	if (c != "DIR"[i]) {
	    ungetc(c, f);
	    return NULL;
	}
    }

    rpl_t_OBJ *pobj = rpl_new_dir();
    rpl_t_LIST *lastelt = (rpl_t_DIR *) pobj->ptr;
    rpl_t_OBJ *tmp;

    i = 0;
    while (1) {
	if (i % 2 == 0) {
	    while (isspace(c = getc(f)));
	    if (c == EOF)
		break;
	    else if (c == '\'')
		ungetc(c, f);
	    else {
		for (i = 0; i < 3; i++) {
		    if (i)
			c = getc(f);
		    if (c != "END"[i]) {
			ungetc(c, f);
			fprintf(stderr, " -- Got unexpected '%c'.\n", c);
			goto returnnull;
		    }
		}

		break;
	    }
	}

	tmp = rpl_fread_obj(f);
	if (!tmp)
	    goto returnnull;

	rpl_list_append(lastelt, tmp);
	i++;
    }

    return pobj;

  returnnull:
    /* FIXME: free memory */
    return NULL;
}
