#ifndef _RPLAPI_H
#define _RPLAPI_H	1

#include <stdio.h>

typedef double rpl_t_REAL;

typedef struct {
    double re, im;
} rpl_t_CMPLX;

typedef char *rpl_t_CSTR, *rpl_t_IDNT;

/* Each nibble n is coded as the char ('0'+n). Ended with '\0'. */
typedef char *rpl_t_ZINT, *rpl_t_HXS;


typedef struct {
    int prolog;
    void *ptr;
} rpl_t_OBJ;

typedef struct _rpl_t_LIST {
  rpl_t_OBJ *obj;
  struct _rpl_t_LIST *next;
} rpl_t_LIST, rpl_t_DIR;


typedef struct {
    int depth;
    int allocated;
    rpl_t_OBJ *data;
} rpl_stack_t;

extern rpl_stack_t rpl_stack;



extern int rpl_stack_depth(void);
extern rpl_t_OBJ *rpl_stack_pick(unsigned int level);

extern void rpl_stack_push(rpl_t_OBJ * obj);

extern int rpl_stack_dropn(int n);
extern int rpl_stack_drop(void);
extern rpl_t_OBJ *rpl_stack_pop(void);


extern void rpl_fprint_obj(FILE *f, rpl_t_OBJ *obj);
extern void rpl_stack_print(void);

rpl_t_OBJ *rpl_new_real(double x);
rpl_t_OBJ *rpl_new_cmplx(double re, double im);
rpl_t_OBJ *rpl_new_cstr(const char *str);
rpl_t_OBJ *rpl_new_idnt(const char *str);
rpl_t_OBJ *rpl_new_hxs(const char *str);
rpl_t_OBJ *rpl_new_zint(const char *str);
rpl_t_OBJ *rpl_new_list(void);
rpl_t_OBJ *rpl_new_dir(void);

void rpl_destroy_obj(rpl_t_OBJ *pobj);

rpl_t_LIST *rpl_list_append(rpl_t_LIST *plist, rpl_t_OBJ *pobj);

rpl_t_OBJ *rpl_fread_obj(FILE * f);
rpl_t_OBJ *rpl_fread_real(FILE * f);
rpl_t_OBJ *rpl_fread_cmplx(FILE * f);
rpl_t_OBJ *rpl_fread_cstr(FILE * f);
rpl_t_OBJ *rpl_fread_idnt(FILE * f);
rpl_t_OBJ *rpl_fread_hxs(FILE * f);
rpl_t_OBJ *rpl_fread_zint(FILE * f);
rpl_t_OBJ *rpl_fread_list(FILE * f);
rpl_t_OBJ *rpl_fread_dir(FILE * f);

#endif
