#include <stdlib.h>
#include <string.h>
#include <hpsys.h>
#include <saturn.h>
#include <satdir.h>

#include "dirapi.h"

int
sat_objhome()
{
    _home_init();
    return (int) __home;
}


int sat_objcurdir()
{
    _home_init();
    return (int) __context;
}

int sat_objgetdir(int objaddr)
{
    int home = sat_peek(SAT_USEROB, 5) + 5;

    objaddr += 5;

    /*  SKIP LIBRARIES ATTACHED TO HOME */
    if (objaddr == home)
	objaddr += 13 * sat_peek(objaddr, 3);

    return objaddr + 3;

}


int sat_itemgetdir(int itemaddr)
{
    int offset;
    itemaddr -= 5;

    do {
	offset = sat_peek(itemaddr, 5);
	itemaddr -= offset + 5;
    } while (offset != 0);

    return itemaddr;

}




int sat_dir_chdir(char *path)
{
    SAT_DIR_NODE *p;

    if ((p = _sat_find_path(path)))
	__sat_cwd = p;

    return p ? 0 : -1;
}


void _sat_dir_close(SAT_DIR_NODE * p)
{
    while (p) {

	SAT_DIR_ENTRY *q = p->object;

	while (q) {
	    SAT_DIR_ENTRY *u = q->next;
	    free(q->sat_obj->name);
	    free(q->sat_obj);
	    free(q);
	    q = u;
	}

	SAT_DIR_NODE *v = p->sibling;

	free(p->name);
	_sat_dir_close(p->child);
	free(p);

	p = v;
    }
}

void sat_dir_close()
{
    if (__sat_root) {
	_sat_dir_close(__sat_root);
	__sat_root = __sat_cwd = NULL;
    }
}


char
*sat_dir_curdir()
{
    return _sat_dir_path(_sat_dir_getcwd());

}


int sat_dircurrent()
{
    return sat_objgetdir(sat_peek(SAT_CONTEXT, 5));
}


void _sat_dir_destroy_slist(SAT_DIR_SLIST * s)
{
    while (s) {
	SAT_DIR_SLIST *t;

	free(s->str);
	t = s->next;
	free(s);
	s = t;
    }
}


double _sat_dir_fetchreal(char *path, double defval, char *mode)
{
    double val = defval;

    SAT_OBJ_DSCR *obj =
	*mode == 'r' ? sat_dir_rclobject(path) : sat_dir_getobject(path);

    if (obj && sat_obj_is_real(obj))
	if (sat_decode_real(obj->addr, &val))
	    return defval;

    return val;
}


int sat_dirfindfirst(int diraddr)
{
    int offset = sat_peek(diraddr, 5);

    /*  RETURN -1 IF EMPTY DIR */
    if (offset == 0)
	return -1;

    return diraddr + offset;
}

int sat_dirfindnext(int diritem)
{
    int offset;
    diritem -= 5;
    offset = sat_peek(diritem, 5);

    /*  RET -1 IF NO MORE OBJECTS */
    if (offset == 0)
	return -1;

    return diritem - offset;

}

SAT_DIR_NODE *_sat_dir_getcwd()
{
    if (!__sat_cwd) {
	sat_dir_open();
	if (!__sat_cwd)
	    __sat_cwd = __sat_root;
    }

    return __sat_cwd;
}


SAT_OBJ_DSCR *sat_dir_getobject(char *path)
{
    char *base, *dir;

    SAT_DIR_NODE *p = _sat_dir_getcwd();

    _sat_dir_split_path(path, &dir, &base);

    if (dir) {
	if (!(p = _sat_find_path(dir)))
	    return NULL;
    } else if (*path == '/')	/*  absolute path, start at root */
	p = _sat_dir_getroot();

    SAT_DIR_ENTRY *q = p->object;
    while (q && strcmp(q->sat_obj->name, base))
	q = q->next;

    free(base);
    free(dir);

    return q ? q->sat_obj : NULL;
}


int _sat_dir_getprolog(SAT_OBJ_DSCR * obj)
{
    return sat_peek(obj->addr, 5);
}


SAT_DIR_NODE *_sat_dir_getroot()
{
    if (!__sat_root)
	sat_dir_open();

    return __sat_root;
}


int sat_dirhome()
{
    return sat_objgetdir(sat_peek(SAT_USEROB, 5));
}


SAT_DIR_NODE *__sat_cwd, *__sat_root;


static void _sat_dir_scan(SAT_DIR_NODE * root, int dirobj)
{
    int h = sat_objgetdir(dirobj);
    int item = sat_dirfindfirst(h), obj, prolog;
    char name[256];

    if (h == sat_dirhome())
	strcpy(name, "HOME");
    else
	sat_itemgetname(sat_objgetitem(dirobj), name);


    while (item != -1) {
	sat_itemgetname(item, name);
	obj = sat_sknameup(item);
	prolog = sat_peek(obj, 5);

	if (prolog == SAT_DORRP) {	/*  sub-directory */
	    SAT_DIR_NODE *p;

	    sys_chkptr(p =
		       (SAT_DIR_NODE *) calloc(1, sizeof(SAT_DIR_NODE)));
	    p->name = strdup(name);
	    p->sibling = root->child;
	    root->child = p;
	    p->parent = root;
	    if (sat_objgetdir(obj) == sat_dircurrent())
		__sat_cwd = p;
	    _sat_dir_scan(p, obj);

	} else {		/*  simple object */
	    SAT_DIR_ENTRY *p;
	    SAT_OBJ_DSCR *d;

	    sys_chkptr(p =
		       (SAT_DIR_ENTRY *) calloc(1, sizeof(SAT_DIR_ENTRY)));
	    sys_chkptr(d =
		       (SAT_OBJ_DSCR *) calloc(1, sizeof(SAT_OBJ_DSCR)));

	    d->name = strdup(name);
	    d->addr = obj;

	    p->sat_obj = d;
	    p->next = root->object;
	    root->object = p;
	}

	item = sat_dirfindnext(item);
    }

}

void sat_dir_open()
{
    if (__sat_root)
	return;

    /*  create root node */
    SAT_DIR_NODE *root = (SAT_DIR_NODE *) calloc(1, sizeof(SAT_DIR_NODE));
    sys_chkptr(root);
    root->name = "";

    _sat_dir_scan(root, sat_objhome());
    __sat_root = root;
}


char
*_sat_dir_path(SAT_DIR_NODE * P)
{
#define _PATHLEN 256

    SAT_DIR_NODE *p = _sat_dir_getroot(), *q = P;
    char *path, *s, *t;
    int slen;

    sys_chkptr(path = calloc(_PATHLEN, sizeof(char)));
    s = (char *) ((unsigned long) path + _PATHLEN - 1);
    while (p != q) {
	slen = strlen(q->name);
	s = (char *) ((unsigned long) s - slen - 1);
	*s = '/';
	strncpy((char *) ((unsigned long) s + 1), q->name, slen);
	q = q->parent;
    }

    t = "HOME";
    slen = strlen(t);
    s = (char *) ((unsigned long) s - slen);
    strncpy(s, t, slen);
    t = strdup(s);
    free(path);

    return t;

}



SAT_OBJ_DSCR *sat_dir_rclobject(char *path)
{
    char *base, *dir;
    int dif = 1;

    /*  no absolute pathes allowed for RCL */
    if (*path == '/' || !strncmp(path, "HOME", 4))
	return sat_dir_getobject(path);

    SAT_DIR_NODE *p = _sat_dir_getcwd();

    _sat_dir_split_path(path, &dir, &base);

    if (dir)
	if (!(p = _sat_find_path(dir)))
	    return NULL;

    SAT_DIR_ENTRY *q = NULL;

    while (p) {

	q = p->object;

	while (q && (dif = strcmp(q->sat_obj->name, base)))
	    q = q->next;

	if (dif)
	    p = p->parent;
	else
	    break;
    }

    free(base);
    free(dir);

    return dif ? NULL : q->sat_obj;
}


int
_sat_dir_searchobject(char *name, SAT_DIR_NODE * p, SAT_DIR_SLIST ** res)
{

    int count = 0;

    while (p) {

	SAT_DIR_ENTRY *q;
	int dif;

	dif = 1;
	q = p->object;

	while (q && (dif = strcmp(q->sat_obj->name, name)))
	    q = q->next;

	if (!dif) {		/*  HIT */
	    SAT_DIR_SLIST *s;
	    char *t, *r;

	    sys_chkptr(s =
		       (SAT_DIR_SLIST *) malloc(sizeof(SAT_DIR_SLIST)));
	    r = _sat_dir_path(p);
	    sys_chkptr(t = (char *) malloc(strlen(r) + strlen(name) + 2));
	    strcpy(t, r);
	    strcat(t, "/");
	    s->str = strcat(t, name);
	    s->next = *res;
	    *res = s;
	    count++;
	    free(r);
	}

	count += _sat_dir_searchobject(name, p->child, res);

	p = p->sibling;
    }

    return count;
}

int sat_dir_searchobject(char *name, SAT_DIR_SLIST ** res)
{
    *res = NULL;
    return _sat_dir_searchobject(name, _sat_dir_getroot(), res);
}


void _sat_dir_split_path(char *path, char **dir, char **base)
{
    char *p = path;
    *dir = *base = NULL;
    while (*p)
	if (*p++ == '/')
	    goto _l1;

    *base = strdup(path);
    return;

  _l1:
    p = (char *) ((unsigned long) path + strlen(path) - 1);

    while (p >= path && *p != '/')
	p--;
    if (*p == '/')
	*base = strdup((char *) ((unsigned long) p + 1));
    if (p > path) {
	*p = '\0';
	sys_chkptr(*dir = malloc(strlen(path) + 1));
	strcpy(*dir, path);
	*p = '/';
    }

}



/*  RETURN POINTER TO THE ITEM THAT CONTAINS THE OBJECT */
/*  IF THE OBJECT IS IN A DIRECTORY */
/*  RETURNS -1 IF NOT IN USEROB */
/*  IF EMBEDDED WITHIN AN OBJECT, RETURNS THE ITEM */
/*  OF THE PARENT OBJECT, BUT NEGATIVE TO INDICATE IT */
/*  IS NOT THE ADDRESS OF THE REQUESTED OBJECT */
int 
sat_objgetitem(int objptr)
{
    int home=sat_peek(SAT_USEROB,5);
    int diritem=sat_dirfindfirst(sat_dirhome()),objstart,objend;
    
    /*  HOME is empty */
    if(diritem<0) return -1;
    
    int endhome=sat_skipob(sat_sknameup(diritem));
    
    /*  CHECK IF OBJECT IS IN USEROB */
    if(objptr<=home || objptr >=endhome) return -1;
    
    do {
        if(diritem<0) return -1;
        
        objstart=sat_sknameup(diritem);
        objend=sat_skipob(objstart);
        if(objptr==objstart) return diritem;
        
        if(objptr>objstart && objptr<objend) {
            
            /*  OBJECT IS EMBEDDED WITHIN THE OBJECT */
            if(sat_peek(objstart,5)==SAT_DORRP) {
                /*  SEARCH IN THE INNER DIRECTORY */
                diritem=sat_dirfindfirst(sat_objgetdir(objstart));
                continue;
            }
            else 
                /*  EMBEDDED IN AN OBJECT OTHER THAN AN RRP */
                return -diritem;
    
        }
        diritem=sat_dirfindnext(diritem);
        
    } while(diritem>=0);
    
    /*  IMPOSSIBLE TO REACH THIS POINT UNLESS MEMORY IS CORRUPTED */
    
    return -1;
}


/*  GET THE NAME CORRESPONDING TO AN ITEM */
/*  RETURNS THE LENGTH OF THE STRING */
/*  AND FILLS THE name */
/*  name SHOULD BE A BIG ENOUGH BUFFER */
int 
sat_itemgetname(int diritem,char *name)
{
    int len=sat_peek(diritem,2);
    
    sat_peek_sat_bytes(name,diritem+2,len);
    
    /*  ADD A TRAILING ZERO */
    name[len]=0;
    
    return len;
}



/*!
    \fn        SAT_DIR_NODE *_sat_find_path(char *path)
    \brief    Find the internal directory node for path \b "path"
            
            DON'T CALL EXPLICITLY
*/

SAT_DIR_NODE 
*_sat_find_path(char *path)
{
    char *s,*t,subdir[64];
    int  climb = 0;
    SAT_DIR_NODE *p;
    
    if (! strcmp("HOME",path))
        return _sat_dir_getroot();

    s = path;
    
    if(*s == '/') {
        s++;
        p = _sat_dir_getroot()->child;
    }
    else if(! strncmp("HOME/",s,5)) {
        s = (char *)((unsigned long) s + 5);
        p = _sat_dir_getroot()->child;
    }
    else
        p = _sat_dir_getcwd()->child;
    
    
    while (*s && p) {
        
        if (climb) 
            p = p->child;
        
        if (!p)
            break;
        
        t = subdir;
        while(*s && *s != '/') 
            *t++ = *s++;
        *t = '\0';
        
        
        if (*s == '/') s++; 
            
        if (! strcmp(subdir,".")) continue;
        if (! strcmp(subdir,"..")) {
            if(!(p = p->parent)) 
                break;
            climb = 0; 
            continue;
        }
        
        while(p && strcmp(subdir,p->name)) 
            p = p->sibling;
                    
        climb = 1;
    }
    
    return p;
}


