#include <string.h>
#include <math.h>
#include "saturn.h"

SAT_LIST sat_list_create(unsigned int maxSize)
{
    unsigned char *data = (unsigned char *) malloc(maxSize);
    if (maxSize == 0) {
	return 0;
    }

    SAT_LIST list = (SAT_LIST) malloc(sizeof(SAT_LIST));

    if (list == 0) {
	free(data);		/* return to the heap */
	return 0;		/* not enough RAM so quit */
    }

    list->size = 5;		/* initally 5 nibbles, for prologue */
    list->data = data;

    /* setup the prologue (0x02A74) */
    data[0] = 0x74;
    data[1] = 0x2A;
    data[2] = 0x00;
    return list;
}

void sat_list_destroy(SAT_LIST list)
{
    /* frees memory allocated for an RPL List */
    if (list != 0) {
	free(list->data);
	free(list);
    }
}


void sat_list_add_real(SAT_LIST list, double num)
{
    unsigned char realStr[20];
    sat_convert_double_real(num, realStr);

    sat_concatnib(list->data, list->size, realStr, 21);
    list->size = list->size + 21;
    return;
}


void sat_list_add_zint_array(SAT_LIST list, unsigned int digits,
			     unsigned int negative, unsigned char *source)
{
    char zint[digits / 2 + 25];
    sat_convert_zint_array(digits, negative, source, zint);

    unsigned int zintSize = digits + 5 + 5 + 1;
    sat_concatnib(list->data, list->size, zint, zintSize);
    list->size = list->size + zintSize;
    return;
}


void sat_list_add_zint(SAT_LIST list, LONGLONG num)
{
    char buffer[300];
    int negative = (num < 0);
    num = abs(num);

    lltoa(num, buffer, 10);
    int digits = strlen(buffer);

    int i;
    for (i = 0; i < digits; i++) {
	buffer[i] = buffer[i] - '0';	/* remove ASCII offset. */
    }

    sat_list_add_zint_array(list, digits, negative, buffer);
    return;
}
