#include <stdio.h>
#include <stdlib.h>
#include "hpsys.h"
#include "saturn.h"
#include "hpstring.h"
#include "rplapi.h"


int sat_push_bint(int bint)
{
    char s[16];
    snprintf(s, 16, "<%05x>", bint);

    rpl_stack_push(rpl_new_cstr(s));
    return 1;
}

int sat_push_char(char byte)
{
    char s[4];
    snprintf(s, 4, "'%c'", byte);

    rpl_stack_push(rpl_new_cstr(s));
    return 1;
}


int sat_push_comp(ULONGLONG real, ULONGLONG imag)
{
    char s[32];
    snprintf(s, 32, "(%08llx %08llx)", real, imag);

    rpl_stack_push(rpl_new_cstr(s));
    return 1;
}

void sat_pushptr(int sataddr)
{
    char s[16];
    snprintf(s, 16, "PTR: %05x", sataddr);

    rpl_stack_push(rpl_new_cstr(s));
}

int sat_push_realarray(int rows, int columns, ULONGLONG * arry)
{
    char s[64];
    snprintf(s, 64, "Array %ux%u (data:%p)", rows, columns, arry);

    rpl_stack_push(rpl_new_cstr(s));
    return 1;
}

int sat_push_real(double number)
{
    char s[24];
    snprintf(s, 24, "%17g", number);

    rpl_stack_push(rpl_new_cstr(s));
    return 1;
}



int sat_push_realvector(int elements, ULONGLONG * arry)
{
    char s[64];
    snprintf(s, 64, "Vector %ux1 (data: %p)", elements, arry);

    rpl_stack_push(rpl_new_cstr(s));
    return 1;
}


int sat_push_string(int type_prolog, int size_nibbles, const char *data)
{
    char *s = malloc(7 + size_nibbles / 2 + 1);
    if (!s) {
	perror(__FUNCTION__);
	exit(1);
    }
    snprintf(s, 7 + size_nibbles / 2 + 1, "%05x: %s", type_prolog, data);

    rpl_stack_push(rpl_new_cstr(s));
    return 1;
}

void sat_pushtruefalse(BOOL a)
{
    if (a)
        rpl_stack_push(rpl_new_cstr("PTR: TRUE"));
    else
        rpl_stack_push(rpl_new_cstr("PTR: FALSE"));
}


int sat_push_zint_array(unsigned int digits, unsigned int negative,
			unsigned char *source)
{
    char *p = sys_chkptr(malloc(digits+negative?1:0));
    int i;

    if (negative)
      *p++ = '-';

    for (i=0; i<digits; i++)
      *p++ = *source++ + '0';

    rpl_stack_push(rpl_new_cstr(p));

    free(p);
    return 1;
}

int sat_push_zint_llong(unsigned long long num)
{
    char *str = sys_chkptr(malloc(sizeof(unsigned long long)));
    sprintf(str, "%llu", num);
    rpl_stack_push(rpl_new_cstr(str));
    free(str);
    return 1;
}


int sat_push_list(SAT_LIST list)
{
    if (list == 0)
	return 0;

    char s[32];
    snprintf(s, 32, "List (data: %p)", list);

    rpl_stack_push(rpl_new_cstr(s));

    return 1;
}
