#include "hpsys.h"


int nlz(unsigned int x)
{
    int n;

    if (x == 0)
	return (32);

    n = 0;
    if (x <= 0x0000FFFF) {
	n = n + 16;
	x = x << 16;
    }
    if (x <= 0x00FFFFFF) {
	n = n + 8;
	x = x << 8;
    }
    if (x <= 0x0FFFFFFF) {
	n = n + 4;
	x = x << 4;
    }
    if (x <= 0x3FFFFFFF) {
	n = n + 2;
	x = x << 2;
    }
    if (x <= 0x7FFFFFFF) {
	n = n + 1;
    }
    return n;
}



unsigned int divlu2(unsigned int u1, unsigned int u0, unsigned int v,
		    unsigned int *r)
{
    const unsigned int b = 65536;	/*  Number base (16 bits). */
    unsigned int un1, un0,	/*  Norm. dividend LSD's. */
     vn1, vn0,			/*  Norm. divisor digits. */
     q1, q0,			/*  Quotient digits. */
     un32, un21, un10,		/*  Dividend digit pairs. */
     rhat;			/*  A remainder. */
    int s;			/*  Shift amount for norm. */

    if (u1 >= v) {		/*  If overflow, set rem. */
	if (r != NULL)		/*  to an impossible value, */
	    *r = 0xFFFFFFFF;	/*  and return the largest */
	return 0xFFFFFFFF;
    }				/*  possible quotient. */

    s = nlz(v);			/*  0 <= s <= 31. */
    v = v << s;			/*  Normalize divisor. */
    vn1 = v >> 16;		/*  Break divisor up into */
    vn0 = v & 0xFFFF;		/*  two 16-bit digits. */

    un32 = ((u1 << s) | (u0 >> (32 - s))) & ((-s) >> 31);
    un10 = u0 << s;		/*  Shift dividend left. */

    un1 = un10 >> 16;		/*  Break right half of */
    un0 = un10 & 0xFFFF;	/*  dividend into two digits. */

    q1 = un32 / vn1;		/*  Compute the first */
    rhat = un32 - q1 * vn1;	/*  quotient digit, q1. */
  again1:
    if (q1 >= b || q1 * vn0 > b * rhat + un1) {
	q1 = q1 - 1;
	rhat = rhat + vn1;
	if (rhat < b)
	    goto again1;
    }

    un21 = un32 * b + un1 - q1 * v;	/*  Multiply and subtract. */

    q0 = un21 / vn1;		/*  Compute the second */
    rhat = un21 - q0 * vn1;	/*  quotient digit, q0. */
  again2:
    if (q0 >= b || q0 * vn0 > b * rhat + un0) {
	q0 = q0 - 1;
	rhat = rhat + vn1;
	if (rhat < b)
	    goto again2;
    }

    if (r != NULL)		/*  If remainder is wanted, */
	*r = (un21 * b + un0 - q0 * v) >> s;	/*  return it. */
    return q1 * b + q0;
}


ULONGLONG udiv64(ULONGLONG z, unsigned int den, unsigned int *r)
{
    unsigned int num_h, num_l, rem;
    unsigned int res_h, res_l;

    num_h = (unsigned int) (z >> 32);
    num_l = (unsigned int) (z & 0xFFFFFFFFUL);
    res_h = divlu2((unsigned int) 0, num_h, den, &rem);
    res_l = divlu2(rem, num_l, den, &rem);
    *r = rem;
    return (ULONGLONG) ((((ULONGLONG) res_h) << 32) | res_l);
}

LONGLONG sdiv64(LONGLONG nom, int den, int *rem)
{
    int minus = 0;
    LONGLONG r;

    if (nom < 0) {
	nom = -nom;
	minus = !minus;
    }

    if (den < 0) {
	den = -den;
	minus = !minus;
    }

    r = udiv64((ULONGLONG) nom, (unsigned int) den, (unsigned int *) rem);

    return minus ? -r : r;
}
