#include "kos.h"
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <time.h>

#include <fcntl.h>
#include <unistd.h>
#include <errno.h>


H_FILE kos_fopen(P_BYTE fname, U8 mode)
{
    H_FILE f;
    char *s;

    if (mode == FILE_READONLY)
	s = "r";
    else if (mode == FILE_READWRITE)
	s = "r+";		/* FIXME: which mode? */
    else if (mode == FILE_APPEND)
	s = "a";
    else {
	fprintf(stderr, "kos_fopen: %u: invalid mode", mode);
	exit(1);
    }
    f = fopen(fname, s);
    if (f) {
	return f;
    } else {
	/* FIXME: */
	return NULL;
    }

}

size_t kos_fread(H_FILE fd, P_BYTE buf, U32 length)
{
    return fread(buf, 1, length, fd);
}

size_t kos_fwrite(H_FILE fd, P_BYTE buf, U32 length)
{
    return fwrite(buf, 1, length, fd);
}

S32 kos_fseek(H_FILE fd, S32 offset, U32 mode)
{
    return fseek(fd, offset, mode);
}

S32 kos_fclose(H_FILE fd)
{
    return fclose(fd);
}

S32 kos_fdelete(P_BYTE fname)
{
    return unlink(fname);
}


S32 kos_frename(P_BYTE oldname, P_BYTE newname)
{
    return rename(oldname, newname);
}

S32 kos_chdir(P_BYTE pDir)
{
    return chdir(pDir);
}


S32 kos_GetSpecialFileInfo(P_BYTE fname, P_FILE_INFO pInfo)
{
    struct stat info;
    int ret;

    ret = stat(fname, &info);
    if (ret) {
	return -1;
    } else {
      struct tm *tm;

#define _KDATETIME_FROM_TIME_T(k, t) \
	tm = gmtime(&t);\
	(k).date.year = tm->tm_year + 1900; \
	(k).date.month = tm->tm_mon; \
	(k).date.day = tm->tm_mday; \
	(k).time.hours = tm->tm_hour; \
	(k).time.minutes = tm->tm_min; \
	(k).time.seconds = tm->tm_sec;
#define _KDATE_FROM_TIME_T(k, t) \
	tm = gmtime(&t);\
	(k).year = tm->tm_year + 1900; \
	(k).month = tm->tm_mon; \
	(k).day = tm->tm_mday;

    _KDATETIME_FROM_TIME_T(pInfo->last_modifier, info.st_mtime);
    _KDATETIME_FROM_TIME_T(pInfo->create_date, info.st_ctime);
    _KDATE_FROM_TIME_T(pInfo->last_access_date, info.st_atime);

    /* FIXME */
#ifdef  NEW_FILE_NAME
    pInfo->file_name[11] = '\0';
    strncpy(pInfo->file_name, fname, 12);
#else
    pInfo->extension[3] = '\0';
    pInfo->name[7] = '\0';
    strncpy(pInfo->extension, fname + strlen(fname) - 4, 3);
    strncpy(pInfo->name, fname, 8);
#endif

    /* FIXME: */
    pInfo->attribute = KOS_READONLY_ATTRIBUTE * (info.st_mode | S_IWUSR);
    pInfo->ext_attr = 0;
    pInfo->size = info.st_size;
	return 0;
    }
}

S32 kos_GetFirstDirInfo(P_DIR_INFO pInfo)
{
    return kos_GetNextDirInfo(pInfo, 0);
}

S32 kos_GetNextDirInfo(P_DIR_INFO pInfo, U32 index)
{
    DIR *dir;
    struct dirent *d;
    int i = 0;

    dir = opendir(".");
    if (!dir) {
	return -1;
    } else {

	while ((d = readdir(dir))) {
	    if (1) {		/* FIXME: tester si est rpertoire */
		if (++i == index + 1)
		    break;
	    }
	}
	closedir(dir);

	return kos_GetSpecialFileInfo(d->d_name, pInfo);
    }
}

S32 kos_GetPrevDirInfo(P_DIR_INFO pInfo, U32 index)
{
    DIR *dir;
    struct dirent *d;
    int i = 0;

    dir = opendir(".");
    if (!dir) {
	return -1;
    } else {

	while ((d = readdir(dir))) {
	    if (1) {		/* FIXME: tester si est rpertoire */
		if (++i == index - 1)
		    break;
	    }
	}
	closedir(dir);

	return kos_GetSpecialFileInfo(d->d_name, pInfo);
    }
}


S32 kos_GetFirstFileInfo(P_FILE_INFO pInfo)
{
    return kos_GetNextFileInfo(pInfo, 0);
}

S32 kos_GetNextFileInfo(P_FILE_INFO pInfo, U32 index)
{
    DIR *dir;
    struct dirent *d;
    int i = 0;

    dir = opendir(".");
    if (!dir) {
	return -1;
    } else {

	while ((d = readdir(dir))) {
	    if (1) {		/* FIXME: tester si est fichier */
		if (++i == index + 1)
		    break;
	    }
	}
	closedir(dir);

	return kos_GetSpecialFileInfo(d->d_name, pInfo);
    }
}

S32 kos_GetPrevFileInfo(P_FILE_INFO pInfo, U32 index)
{
    DIR *dir;
    struct dirent *d;
    int i = 0;

    dir = opendir(".");
    if (!dir) {
	return -1;
    } else {

	while ((d = readdir(dir))) {
	    if (1) {		/* FIXME: tester si est fichier */
		if (++i == index - 1)
		    break;
	    }
	}
	closedir(dir);

	return kos_GetSpecialFileInfo(d->d_name, pInfo);
    }
}

S32 kos_fattr(P_BYTE fname, U8 attr)
{
    return 0;
}


S32 kos_GetDiskFreeSize(void)
{
  return 0;	/* FIXME */
}

S32 kos_CheckDiskInsert(void)
{
    return 1;
}

S32 kos_CheckDiskProtect(void)
{
    return 0;
}

void kos_ResetSD(void)
{
    return;
}
