#include "hpscreen.h"
#include <stdio.h>

void drawBlockOR(unsigned char *sprite, int length, int x, int y)
{
    char *LCDPos = getLCDPointer(x, y);
    char left, right;
    int ali = x & 0x7;

    while (length > 0) {
	left = ((*sprite) << ali) & 0xff;
	right = ((*sprite) >> (8 - ali)) & 0xff;

	*LCDPos = (*LCDPos) | left;
	*(LCDPos + 1) = *(LCDPos + 1) | right;

	LCDPos = LCDPos + 0x14;
	sprite++;
	length--;
    }
}

void drawBlockXOR(unsigned char *sprite, int length, int x, int y)
{
    char *LCDPos = getLCDPointer(x, y);
    char left, right;
    int ali = x & 0x7;

    while (length > 0) {
	left = ((*sprite) << ali) & 0xff;
	right = ((*sprite) >> (8 - ali)) & 0xff;

	*LCDPos = (*LCDPos) ^ left;
	*(LCDPos + 1) = *(LCDPos + 1) ^ right;

	LCDPos = LCDPos + 0x14;
	sprite++;
	length--;
    }
}


/*
 * Block4bit functions
 */


void getBlock4bit(unsigned char *dest, int width, int height, int x, int y)
{
    fprintf(stderr, "%s: not yet implemented\n", __FUNCTION__);
    return;
}

void drawBlock4bit(unsigned char *sprite, int height, int x, int y)
{
    char *LCDPos = getLCDPointer(x, y);
    unsigned char left, right;
    unsigned char mask;
    unsigned char odd = 0;	/*  odd or even row ? */
    unsigned char buf;		/*  half byte to store current row */
    int ali = x & 0x7;		/*  x mod 8 */

    while (height-- > 0) {
	buf = ((*sprite) >> (odd << 2)) & 0xf;	/* first/second 4bit row */
	odd = !odd;

	left = (buf << ali) & 0xff;
	mask = 0xff << ali;

	*LCDPos = ((*LCDPos) & ~mask) | left;
	if (ali > 4) {
	    right = (buf >> (8 - ali)) & 0xff;
	    mask =  0xff >> (8 - ali);
	    *(LCDPos + 1) = (*(LCDPos + 1) & ~mask) | right;
	}

	LCDPos += 0x14;		/*  Y=Y+1 */
	if (!odd)
	    sprite++;		/*  look at the next byte */
    }
}


int drawBlockOR4bitC(unsigned char *sprite, int height, int x, int y)
{
    char *LCDPos = getLCDPointer(x, y);
    unsigned char left, right;
    unsigned char odd = 0;	/*  odd or even row ? */
    unsigned char buf;		/*  half byte to store current row */
    int ali = x & 0x7;		/*  x mod 8 */
    int collision = 0;

    while (height-- > 0) {
	buf = ((*sprite) >> (odd << 2)) & 0xf;	/* first/second 4bit row */
	odd = !odd;

	left = (buf << ali) & 0xff;
	if (*LCDPos & left)
	    collision = 1;

	*LCDPos = (*LCDPos) | left;
	if (ali > 4) {
	    right = (buf >> (8 - ali)) & 0xff;
	    if (*(LCDPos + 1) & right)
		collision = 1;
	    *(LCDPos + 1) = *(LCDPos + 1) | right;
	}

	LCDPos += 0x14;		/*  Y=Y+1 */
	if (!odd)
	    sprite++;		/*  look at the next byte */
    }

    return collision;
}


int drawBlockXOR4bitC(unsigned char *sprite, int height, int x, int y)
{
    char *LCDPos = getLCDPointer(x, y);
    unsigned char left, right;	/*  the split bytes */
    unsigned char odd = 0;	/* odd or even row ? */
    unsigned char buf;		/*  half byte to store current row */
    int collision = 0;

    int ali = x & 0x7;		/*  x mod 8 */

    while (height-- > 0) {
	buf = ((*sprite) >> (odd << 2)) & 0xf;	/*  first/second 4bit row */
	odd = !odd;

	left = (buf << ali);

	if (*LCDPos & left)
	    collision = 1;
	*LCDPos = (*LCDPos) ^ left;

	if (ali > 4) {
	    right = (buf >> (8 - ali));
	    if (*(LCDPos + 1) & right)
		collision = 1;
	    *(LCDPos + 1) = *(LCDPos + 1) ^ right;
	}

	LCDPos += 0x14;		/*  Y=Y+1 */
	if (!odd)
	    sprite++;		/*  look at the next byte */
    }

    return (collision);
}


int testBlock4bitC(unsigned char *sprite, int height, int x, int y)
{
    char *LCDPos = getLCDPointer(x, y);
    unsigned char left, right;	/*  the split bytes */
    unsigned char odd = 0;	/*  odd or even row ? */
    unsigned char buf;		/*  half byte to store current row */

    int ali = x & 0x7;		/*  x mod 8 */

    while (height-- > 0) {
	buf = ((*sprite) >> (odd << 2)) & 0xf;	/*  first/second 4bit row */
	odd = !odd;

	left = (buf << ali) & 0xff;

	if (*LCDPos & left)
	    return (1);

	if (ali > 4) {
	    right = (buf >> (8 - ali)) & 0xff;
	    if (*(LCDPos + 1) & right)
		return (1);
	}

	LCDPos += 0x14;		/*  Y=Y+1 */
	if (!odd)
	    sprite++;		/*  look at the next byte */
    }

    return (0);
}


/*
 * fillTiles()
 */

void fillTiles(unsigned char *sprite, int length, int x, int y, int right,
	       int down)
{
    int startX = x;
    int rightCount;

    while (down > 0) {

	rightCount = right;
	x = startX;

	while (rightCount > 0) {
	    drawBlockOR(sprite, length, x, y);
	    rightCount--;
	    x = x + 8;		/*  move accross one */
	}

	y = y + length;
	down--;

    }
    return;
}
