#include <hpgcc49.h>
#include <hpgbmp.h>
#include "hpgpriv.h"

unsigned char RGB2Gray (int R, int G, int B)
{
	return ((unsigned char) (0.299 * R + 0.587 * G + 0.114 * B));
}

hpg_t *hpg_load_bmp_gray16 (char *name, int *ret)
{
	int Ret = 0;
	hpg_t *img = NULL;
	FILE *file;

	// Opens the bitmap file
	if (file = fopen (name, "rb"))
	{
		HEADER Header;
		INFOHEADER Info;
		char Tmp[13];

		// Checks for HP-49 header
		if (fread ((void *) Tmp, 1, 13, file) == 13)
		{
			if (strncmp ("HPHP49", Tmp, 6))
			{
				(void) fseek (file, 0, FS_SEEK_SET);
			}
		}
		
		// Reads the header
		if (fread ((void *) &Header, 1, sizeof (HEADER), file) == sizeof (HEADER))
		{
			// Checks the format
			if (! strncmp ("BM", (char *) &(Header.type), 2))
			{
				// Reads the header information
				(void) fseek (file, 2, FS_SEEK_CUR);
				if (fread ((void *) &Info, 1, sizeof (INFOHEADER), file) == sizeof (INFOHEADER))
				{
					// Checks the compression level
					if (Info.compression == 0)
					{
						unsigned char Palette[256][4];// Palette
						
						// Allocates the target image
						if (img = hpg_alloc_gray16_image (Info.width, Info.height))
						{
							// Loads the palette
							if (Info.bits < 24)
							{
								int NbColors = 1 << Info.bits;
								int Ind;
												
								for (Ind = 0; Ind < NbColors; Ind++)
								{
									if (fread ((void *) (Palette[Ind]), 1, 4, file) != 4)
									{
										break;
									}
								}
							}
											
							// Reads the file
							int NbPixels;				// Number of used bytes within a line
							int Width;					// Total number of bytes within a line (aligned on 4 bytes)
							unsigned char *Buffer;	// Buffer into which to read the line
											
							NbPixels = Info.width;
							Width = (Info.width * Info.bits % 32) ?
										(int) (Info.width * Info.bits / 32 + 1) * 4 : Info.width * Info.bits / 8;
							
							// Allocates a buffer to store each line
							if (Buffer = (unsigned char *) malloc (NbPixels * Info.bits / 8))
							{						
								int y;
								
								// Loops over the lines
								for (y = Info.height - 1; y >= 0; y--)
								{
									// Loads the line						
									if (fread ((void *) Buffer, 1, NbPixels * Info.bits / 8, file) == NbPixels * Info.bits / 8)
									{
										unsigned char *Ptr;
										int Ind;
										unsigned char Color;
										unsigned char M = 0xFF;
										int Shift = 0;
										int x = 0;
										int n;
										
										// For each pixel of the line
										for (Ptr = (unsigned char *) Buffer, n = 0; n < NbPixels; n++)
										{
											// Converts the color into level of gray
											if (Info.bits < 8)
											{
												Ind = (*Ptr & (M >> (8 - Info.bits)) << (8 - (Shift + Info.bits))) >> (8 - (Shift + Info.bits));
												Color = 255 - RGB2Gray (Palette[Ind][0], Palette[Ind][1], Palette[Ind][2]);
												if (! ((Shift += Info.bits) % 8))
												{
													Shift = 0;
													Ptr++;
												}
											}
											else if (Info.bits == 8)
											{
												Ind = (int) *Ptr;
												Color = 255 - RGB2Gray (Palette[Ind][0], Palette[Ind][1], Palette[Ind][2]);																
												Ptr++;
											}
											else if (Info.bits == 24)
											{
												Color = 255 - RGB2Gray ((int) *(Ptr+2),
																				(int) *(Ptr+1),
																				(int) *Ptr);
												Ptr += 3;
											}
											else
											{
												break;
											}
											
											// Sets the color of the target pixel
											hpg_set_color (img, Color);
											
											// Draws this pixel
											hpg_draw_pixel_on (img, x, y);
											
											// Next pixel in the line
											x++;
										}
										
										// Aligns on the end of the line
										int Rest = Width - NbPixels * Info.bits / 8;
										if (Rest)
										{
											(void) fseek (file, Rest, FS_SEEK_CUR);
										}
									}
									else
									{
										Ret = C_ErrReadImage;
									}
								}
							}
							else
							{
								Ret = C_ErrMemory;
							}
						}
						else
						{
							Ret = C_ErrMemory;
						}
					}
					else
					{
						Ret = C_ErrNotImpl;
					}
				}
				else
				{
					Ret = C_ErrReadHeader;
				}
			}
			else
			{
				Ret = C_ErrBadFormat;
			}
		}
		else
		{
			Ret = C_ErrReadHeader;
		}
					
		(void) fclose (file);
	}
	else
	{
		Ret = C_ErrOpenFile;
	}				
	
	*ret = Ret;
	return (img);
}
