#include <hpgcc49.h>
#include <hpgbmp.h>
#include "hpgpriv.h"

int hpg_save_anybmp_gray16 (char *name, palette_t pal[], hpg_t *image, int width, int height)
{
	int Ret = 0;
	FILE *file;

	// Opens the bitmap file
	if ((file = fopen (name, "wb")) != NULL)
	{
		char *type = "BM";
		//int size = 5558;
		//int size = 278 + (width * height)/2;
		int rowsize, size;
		int dummy = 0;
		int headersize = 118;
		int headerinfo = 40;
		unsigned short planes = 1;
		unsigned short bits = 4;
		int resol = 5905;
		unsigned short shortdummy = 0;
		unsigned char shortchar = 0;
		int i;

		rowsize = 4 * (int)ceil(width/8.0);
		size = 118 + rowsize * height;
		
		// Writes the header
		(void) fwrite ((void *)type, 1, 2, file);
		(void) fwrite ((void *)&size, 1, 4, file);
		(void) fwrite ((void *)&dummy, 1, 4, file);
		(void) fwrite ((void *)&headersize, 1, 4, file);
		
		(void) fwrite ((void *)&headerinfo, 1, 4, file);
		(void) fwrite ((void *)&width, 1, 4, file);
		(void) fwrite ((void *)&height, 1, 4, file);
		(void) fwrite ((void *)&planes, 1, 2, file);
		(void) fwrite ((void *)&bits, 1, 2, file);
		(void) fwrite ((void *)&dummy, 1, 4, file);
		(void) fwrite ((void *)&dummy, 1, 4, file);
		(void) fwrite ((void *)&resol, 1, 4, file);
		(void) fwrite ((void *)&resol, 1, 4, file);
		(void) fwrite ((void *)&dummy, 1, 4, file);
		(void) fwrite ((void *)&dummy, 1, 4, file);

		// If there is no provided palette as parameter, build one (levels of gray)
		if (! pal)
		{
			pal = (palette_t *) malloc (16 * sizeof (palette_t));
			int Ind;
					
			for (Ind = 0; Ind < 16; Ind++)
			{
				pal[15 - Ind].rgb[0] = Ind * 16 + Ind;
				pal[15 - Ind].rgb[1] = Ind * 16 + Ind;
				pal[15 - Ind].rgb[2] = Ind * 16 + Ind;
				pal[15 - Ind].rgb[3] = 0;
			}
		}
		
		// Writes the palette
		if (fwrite ((void *) pal, 1, 16 * sizeof(palette_t), file) == 16 * sizeof(palette_t))
		{	
			int x, y;
			unsigned char color = 0;
			int shift = 0;

			// Writes the file line by line (starting from the end), column by column
			for (y = height - 1; y >= 0; y--)
			{
				for (x = 0; x < width; x++)
				{
					// Gets the pixel from the hpg_stdscreen
					unsigned char c = (hpg_get_pixel (image, x, y)) & 0x0F;
					// Gets the color
					color |= (shift ? c : (c << 4));
					// Writes the color (since we generate a 4 bits coded color, 2 pixels
					// must be stored at once, that's explain the 'shift' mechanism)
					if (shift)
					{
						(void) fwrite ((void *) &color, 1, 1, file);
						color = 0;
					}
					shift = (shift + 1) % 2;

					if(x == (width - 1) && width%2 == 1) {
						c=0;
						color |= (shift ? c : (c << 4));
						(void) fwrite ((void *) &color, 1, 1, file);
						width++;
					}
				}

				// Fills the end of line for alignment
				for(i = 0; i < (rowsize - width/2); i++)
					(void) fwrite ((void *)&shortchar, 1, 1, file);
			}
		}
		else
		{
			Ret = C_ErrWritePalette;
		}
					
		(void) fclose (file);
	}
	else
	{
		Ret = C_ErrOpenFile;
	}				
	
	return (Ret);
}

