#include <hpgcc49.h>
#include <hpparser.h>
#include "prspriv.h"

//
// CONSTANTS DEFINITION
//

#define C_MaxVariable	50				// Max number of variables

#define C_TypeStack		0

//
// LOCAL VARIABLES
//

static T_Variable *S_Variable[C_MaxVariable];	// Array of variables

//
// PROTOTYPES
//

//
// LOCAL PROCEDURES
//

//
// InitParser
// ----------
// Initializes the parser structures
//
static void InitParser (void)
{
	static int Init = 0;
	int Ind;

	if (! Init)
	{
		// Variables initialization
		for (Ind = 0; Ind < C_MaxVariable; Ind++)
		{
			S_Variable[Ind] = NULL;
		}

		Init = 1;
	}
}

//
// GetVariableEntry
// ---------------------
// Gets the pointer to an existing or new variable descriptor
//
// P_Ident		: I  : Variable name
//
// Return		: Pointer to the variable descriptor
//
static int GetVariableEntry (char *P_Ident, T_Variable **P_Var)
{
	int Ind, Mem = C_MaxVariable;
	int Ret = PRS_OK;

	if (P_Ident && P_Var)
	{
		// Searches for an existing descriptor
		for (Ind = 0; Ind < C_MaxVariable; Ind++)
		{
			// By the way, memorizes the last empty slot (in case ...)
			if (! S_Variable[Ind])
			{
				Mem = Ind;
			}
			else
			{
				if (! strcmp (S_Variable[Ind]->Name, P_Ident))
				{
					// Quits the loop if variable found
					break;
				}
			}
		}

		// If variable found, returns its descriptor
		if (Ind < C_MaxVariable)
		{
			*P_Var = S_Variable[Ind];
		}
		else if (Mem < C_MaxVariable)
		{
			// Otherwise, creates a new decriptor in an empty slot
			*P_Var = S_Variable[Mem] = (T_Variable *) malloc (sizeof(T_Variable));
			if (*P_Var)
			{
				// Fills it
				(*P_Var)->Name = (char *) malloc (strlen (P_Ident) + 1);
				if ((*P_Var)->Name)
				{
					(void) strcpy ((*P_Var)->Name, P_Ident);
					(*P_Var)->Init = 0;
					(*P_Var)->Value = 0;
				}
				else
				{
					(void) free ((*P_Var));
					(*P_Var) = NULL;
					Ret = PRS_ERROR_MEMORY;
				}
			}
			else
			{
				Ret = PRS_ERROR_MEMORY;
			}
		}
		else
		{
			Ret = PRS_ERROR_SLOT_VAR;
		}
	}
	else
	{
		Ret = PRS_ERROR_PAR;
	}

	return (Ret);
}


//
// EXPORTED PROCEDURES
//

int prs_alg_set_var (char *P_Var, double P_Value)
{
	T_Variable *Var;
	int Ret = PRS_OK;

	if ((Ret = GetVariableEntry (P_Var, &Var)) == PRS_OK)
	{
		Var->Init = 1;
		Var->Value = P_Value;
	}

	return (Ret);
}

int prs_alg_add_val (entry_t P_Stack[], int P_Level, double P_Val)
{
	int Ret = PRS_OK;
	int size = P_Stack[0].data.Size;

	if (! P_Level)
	{
		P_Level = size;
	}
	if (P_Level <= size++)
	{
		P_Stack[P_Level].Type = C_TypeVal;
		P_Stack[P_Level].data.Val = P_Val;
		P_Stack[0].data.Size = size;
	}
	else
	{
		Ret = PRS_ERROR_STACK;
	}

	return (Ret);
}

int prs_alg_add_ident (entry_t P_Stack[], int P_Level, char *P_Ident)
{
	int Ret = PRS_OK;
	int size = P_Stack[0].data.Size;

	if (! P_Level)
	{
		P_Level = size;
	}
	if (P_Level <= size++)
	{
		T_Variable *Var;

		if ((Ret = GetVariableEntry (P_Ident, &Var)) == PRS_OK)
		{
			P_Stack[P_Level].Type = C_TypeIdent;
			P_Stack[P_Level].data.Var = Var;
			P_Stack[0].data.Size = size;
		}
	}
	else
	{
		Ret = PRS_ERROR_STACK;
	}

	return (Ret);
}

int prs_alg_add_func (entry_t P_Stack[], int P_Level, int P_Code)
{
	int Ret = PRS_OK;
	int size = P_Stack[0].data.Size;

	if (! P_Level)
	{
		P_Level = size;
	}
	if (P_Level <= size++)
	{
		P_Stack[P_Level].Type = C_TypeFunc;
		P_Stack[P_Level].data.Func = P_Code;
		P_Stack[0].data.Size = size;
	}
	else
	{
		Ret = PRS_ERROR_STACK;
	}

	return (Ret);
}

int prs_alg_add_symb (entry_t P_Stack[], int P_Level, entry_t *P_Symb)
{
	int Ret = PRS_OK;
	int size = P_Stack[0].data.Size;

	if (! P_Level)
	{
		P_Level = size;
	}
	if (P_Level <= size++)
	{
		P_Stack[P_Level].Type = C_TypeSymb;
		P_Stack[P_Level].data.Symb = P_Symb;
		P_Stack[0].data.Size = size;
	}
	else
	{
		Ret = PRS_ERROR_STACK;
	}

	return (Ret);
}

int prs_alg_create (entry_t **stack, int size)
{
	int Ret = PRS_OK;

	InitParser ();
	
	if (stack)
	{
		*stack = (entry_t *) malloc (size * sizeof (entry_t));
		if (*stack)
		{
			(*stack)[0].Type = C_TypeStack;
			(*stack)[0].data.Size = 1;
		}
		else
		{
			Ret = PRS_ERROR_MEMORY;
		}
	}
	else
	{
		Ret = PRS_ERROR_PAR;
	}

	return (Ret);
}

int prs_alg_destroy (entry_t **stack)
{
	int Ret = PRS_OK;

	if (stack && *stack)
	{
		int level = 1;
		
		while (level < stack[0]->data.Size)
		{
		   switch (stack[level]->Type)
		   {
		      case C_TypeSymb :
		      {
		         if (stack[level]->data.Symb)
		         {
		            (void) prs_alg_destroy (&(stack[level]->data.Symb));
					}
		         break;
				}
			}
			
			level++;
		}
		
		(void) free (*stack);
		*stack= NULL;
	}
	else
	{
		Ret = PRS_ERROR_PAR;
	}

	return (Ret);
}

int prs_alg_get_size (entry_t stack[], int *size)
{
	int Ret = PRS_OK;
	
	if (stack && size)
	{
		*size = stack[0].data.Size;
	}
	else
	{
		Ret = PRS_ERROR_PAR;
	}
	
	return (Ret);
}

int prs_alg_set_size (entry_t stack[], int size)
{
	int Ret = PRS_OK;
	
	if (stack)
	{
		stack[0].data.Size = size;
	}
	else
	{
		Ret = PRS_ERROR_PAR;
	}
	
	return (Ret);
}

static int IndVar = 0;

int prs_alg_next_var (entry_t stack[], char **name)
{
	int Ret = PRS_OK;
	int Ind;
	
	// Search the next variable belonging to the stack
	// NB : they have been flagged as such by prs_alg_first_var
	for (Ind = IndVar+1; Ind < C_MaxVariable; Ind++)
	{
		if (S_Variable[Ind] && (S_Variable[Ind]->Flag == stack))
		{
			*name = S_Variable[Ind]->Name;
			IndVar = Ind;
			break;
		}
	}
	
	if (Ind == C_MaxVariable)
	{
		*name = NULL;
		Ret = PRS_ERROR_VARIABLE;
	}
	
	return (Ret);
}

static void prs_alg_flag_var (entry_t *stack, entry_t *flag)
{
	int Ind;

	if (stack)
	{		
		for (Ind = 1; Ind < stack[0].data.Size; Ind++)
		{
			if (stack[Ind].Type == C_TypeIdent)
			{
				stack[Ind].data.Var->Flag = flag;
			}
			else if (stack[Ind].Type == C_TypeSymb)
			{
				prs_alg_flag_var (stack[Ind].data.Symb, flag);
			}
		}
	}
	else
	{
		for (Ind = 0; Ind < C_MaxVariable; Ind++)
		{
			if (S_Variable[Ind])
			{
				S_Variable[Ind]->Flag = stack;
			}
		}
	}
}

int prs_alg_first_var (entry_t stack[], char **name)
{
	int Ret = PRS_OK;
	
	// Flag the variables that belong to this stack
	prs_alg_flag_var (stack, stack);
	
	// Search the first variable belonging to this stack
	IndVar = -1;
	Ret = prs_alg_next_var (stack, name);
	
	return (Ret);
}


