#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_decode (unsigned address, hpobj_t *obj, int *incr)
{
	int Ret = HPS_OK;
	unsigned prolog;

	if (obj)
	{
		if (obj->type)
		{
			prolog = obj->type;							// There is no prologue (used by DOARRY)
			*incr = 0;
		}
		else
		{
			prolog = sat_peek (address, 5);
			*incr = 5;
		}

		switch (hps_convert_type (prolog))
		{
			case DOFUNC :
			{
				obj->type = DOFUNC;
				obj->data.dofunc = prolog;
				break;
			}

			case DOINT :		// DOINT -> long long
			{
				Ret = hps_decode_int (address, obj, incr);
				break;
			}

			case DOHXS :		// DOHXS -> unsigned long long
			{
				Ret = hps_decode_uint (address, obj, incr);
				break;
			}

			case DOREAL :		// DOREAL -> double
			{
				Ret = hps_decode_real (address, obj, incr);
				break;
			}

			case DOCMP :		// DOCMP -> complex_t
			{
				Ret = hps_decode_complex (address, obj, incr);
				break;
			}

			case DOCSTR :		// DOCSTR -> char *
			{
				Ret = hps_decode_str (address, obj, incr);
				break;
			}

			case DOIDNT :
			{
				Ret = hps_decode_ident (address, obj, incr);
				break;
			}

			case DOLIST :		// DOLIST -> list_t *
			{
				Ret = hps_decode_list (address, obj, incr);
				break;
			}

			case DOMATRIX :	// DOMATRIX -> array_t *
			{
				Ret = hps_decode_array (address, obj, incr);
				break;
			}

			case DOSYMB :
			{
				Ret = hps_decode_symb (address, obj, incr);
				break;
			}

			default :
			{
				Ret = HPS_ERROR_NOT_IMPLEMENTED;
				break;
			}

		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}

	return (Ret);
}
