#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_decode_array (unsigned address, hpobj_t *obj, int *incr)
{
	int Ret = HPS_OK;
	unsigned prolog;
	
	if (obj && incr)
	{
		if (obj->type)
		{
			prolog = obj->type;							// There is no prologue (used by DOARRY)
			*incr = 0;
		}
		else
		{
			prolog = sat_peek (address, 5);
			address += 5;									// Skips the prologue
			*incr = 5;
		}
			
		switch (prolog)
		{
			case DOMATRIX :
			{
				unsigned addr = address;
				array_t *array;
				
				// Creates the array
				if ((Ret = hps_array_create (&array)) == HPS_OK)
				{
					unsigned prologue;
					
					// Loops over the array until the end
					while ((prologue = sat_peek (addr, 5)) != SEMI)
					{
						hpobj_t *obj;
							
						if ((Ret = hps_obj_create (&obj)) == HPS_OK)
						{
							int delta;
							
							obj->type = 0;
							if ((Ret = hps_decode (addr, obj, &delta)) == HPS_OK)
							{
								(void) hps_array_add (array, obj);
								addr += delta;
							}
							else
							{
								break;
							}
						}
					}
					
					obj->data.domatrix = array;
					obj->type = DOMATRIX;
					*incr += addr + 5 - address;
				}
				
				break;
			}
			
			case DOARRY :
			{
				unsigned addr = address;
				array_t *array;
				unsigned arraytype;
				int rows, columns;
				
				// Skips the size
				addr += 5;
				
				// Gets the type
				arraytype = sat_peek (addr, 5);
				addr += 5;
				
				// Skips the dim
				addr += 5;
				
				// Gets the number of rows
				rows = sat_peek (addr, 5);
				addr += 5;
				
				// Gets the number of columns
				columns = sat_peek (addr, 5);
				addr += 5;
	
				// Creates the array
				if ((Ret = hps_array_create (&array)) == HPS_OK)
				{
					int r, c;
					
					// Loops over the array
					for (r = 0; r < rows; r++)
					{
						hpobj_t *dim;
						array_t *row;
	
						(void) hps_array_create (&row);
						(void) hps_obj_create_type (DOMATRIX, (void *) &row, &dim);
						
						for (c = 0; c < columns; c++)
						{
							int delta;
							hpobj_t *elem;
							
							(void) hps_obj_create_type (arraytype, NULL, &elem);
							if ((Ret = hps_decode (addr, elem, &delta)) == HPS_OK)
							{
								(void) hps_array_add (row, elem);
								addr += delta;
							}
							else
							{
								break;
							}
						}
						
						if ((Ret = hps_array_add (array, dim)) != HPS_OK)
						{
							break;
						}
					}
					obj->data.domatrix = array;
					obj->type = DOMATRIX;
					*incr += addr - address;
				}
				
				break;
			}
			
			default :
			{
				Ret = HPS_ERROR_TYPE;
				break;
			}
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}
