#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_decode_symb (unsigned address, hpobj_t *obj, int *incr)
{
	int Ret = HPS_OK;
	unsigned prolog;
	
	if (obj && incr)
	{
		if (obj->type)
		{
			prolog = obj->type;							// There is no prologue (used by DOARRY)
			*incr = 0;
		}
		else
		{
			prolog = sat_peek (address, 5);
			address += 5;									// Skips the prologue
			*incr = 5;
		}
			
		switch (prolog)
		{
			case DOSYMB :
			{
				entry_t *expr;
				unsigned addr = address;
				unsigned prologue;

				// Creates the expression
				if ((Ret = prs_alg_create (&expr, 100)) == HPS_OK)
				{
					int level = 1;
					
					// Loops over the expression until the end
					while (((prologue = sat_peek (addr, 5)) != SEMI) && (Ret == HPS_OK))
					{
						hpobj_t elem;
						int delta;
						
						elem.type = 0;
						switch (hps_convert_type (prologue))
						{
							case DOINT :
							{
								hps_decode_int (addr, &elem, &delta);
								prs_alg_add_val (expr, level, (double) (elem.data.doint));
								break;
							}
							
							case DOREAL :
							{
								hps_decode_real (addr, &elem, &delta);
								prs_alg_add_val (expr, level, elem.data.doreal);
								break;
							}

							case DOIDNT :
							{
								hps_decode_ident (addr, &elem, &delta);
								prs_alg_add_ident (expr, level, elem.data.doidnt);
								break;
							}
									
							case DOFUNC :
							{
								delta = 5;
								prs_alg_add_func (expr, level, prologue);
								break;
							}
									
							case DOSYMB :
							{
								hps_decode_symb (addr, &elem, &delta);
								prs_alg_add_symb (expr, level, elem.data.dosymb);
								break;
							}

							default :
							{
								Ret = HPS_ERROR_NOT_IMPLEMENTED;
								break;
							}
						}

						addr += delta;
						level++;
					}
					
					if (Ret == HPS_OK)
					{
						prs_alg_set_size (expr, level);
						obj->data.dosymb = expr;
						obj->type = DOSYMB;
						*incr += addr + 5 - address;		// Address after SEMI - address after prologue
					}
				}				
				break;
			}
			
			default :
			{
				Ret = HPS_ERROR_TYPE;
				break;
			}
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}
