#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_list_create (list_t **list)
{
	int Ret = HPS_OK;
	list_t *elem;
	
	if (elem = (list_t *) malloc (sizeof (list_t)))
	{
		elem->prev = NULL;
		elem->next = NULL;
		elem->obj = NULL;
		*list = elem;
	}
	else
	{
		Ret = HPS_ERROR_ARM_MEMORY;
	}
	
	return (Ret);
}

int hps_list_destroy (list_t **list)
{
	int Ret = HPS_OK;
	
	if (list && *list)
	{
		list_t *elem = *list;
		
		while (elem->next)
		{
			elem = elem->next;
		}

		while (elem)
		{
			(void) hps_obj_destroy_all (&(elem->obj));
			elem = elem->prev;
		}
		
		(void) free (*list);
		*list = NULL;
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
		
	return (Ret);
}

int hps_list_add (list_t *list, hpobj_t *obj)
{
	int Ret = HPS_OK;
	
	if (list && obj)
	{
		list_t *elem = list;
		list_t *new;

		while (elem->next)
		{
			elem = elem->next;
		}

		if (elem->obj)
		{
			if (new = (list_t *) malloc (sizeof (list_t)))
			{
				new->prev = elem;
				new->next = NULL;
				new->obj = obj;
				elem->next = new;
			}
			else
			{
				Ret = HPS_ERROR_ARM_MEMORY;
			}
		}
		else
		{
			elem->obj = obj;
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}

int hps_list_get (list_t *list, int ind, hpobj_t **obj)
{
	int Ret = HPS_OK;

	if (list && (ind > 0) && obj)
	{
		int i;
		list_t *elem = list;
		
		// Searches for the ind-th element
		for (i = 1; i < ind; i++)
		{
			if (! elem)
			{
				break;
			}
			elem = elem->next;
		}
		
		// If found, returns the object
		if ((i == ind) && elem)
		{
			*obj = elem->obj;
		}
		else
		{
			Ret = HPS_ERROR_PAR;
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);	
}

