#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_pick (int level, hpobj_t *obj)
{
	int Ret = HPS_OK;
	
	if (obj)
	{
		int depth;
		
		// Gets the stack depth
		(void) hps_depth (&depth);
		
		// Checks that the asked level exists
		if ((level > 0) && (level <= depth))
		{
			int incr;
			
			// Gets the address of the object located at this level
			unsigned address = sat_peek (sat_peek (SAT_DSKTOP, 5) + (level-1) * 5, 5);
			
			// Tries to decode the object
			obj->type = 0;
			Ret = hps_decode (address, obj, &incr);
		}
		else
		{
			Ret = HPS_ERROR_STACK;
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}

int hps_pick_type (int level, int type, void *data)
{
	int Ret = HPS_OK;

	if (data)
	{
		int objtype;
		
		// Gets the type of the object at the indicated level
		if ((Ret = hps_get_type (level, &objtype)) == HPS_OK)
		{
			// Checks it
			if (objtype == type)
			{
				hpobj_t obj;
				
				// Picks the object
				if ((Ret = hps_pick (level, &obj)) == HPS_OK)
				{
					int nb = 0;
						
					// Copies the data into the target variable
					switch (obj.type)
					{
						case DOINT :
						{
							nb = sizeof (LONGLONG);
							break;
						}
						
						case DOHXS :
						{
							nb = sizeof (ULONGLONG);
							break;
						}
						
						case DOREAL :
						{
							nb = sizeof (double);
							break;
						}
						
						case DOCMP :
						{
							nb = sizeof (complex_t);
							break;
						}
						
						case DOCSTR :
						{
							nb = sizeof (char *);
							break;
						}
						
						case DOLIST :
						{
							nb = sizeof (list_t *);
							break;
						}
						
						case DOMATRIX :
						{
							nb = sizeof (array_t *);
							break;
						}
						
						case DOIDNT :
						{
							nb = sizeof (char *);
							break;
						}

						case DOSYMB :
						{
							nb = sizeof (entry_t *);
							break;
						}

						default :	// Not accessible
						{
							Ret = HPS_ERROR_TYPE;
							break;
						}
					}
	
					(void) memcpy (data, (void *) &(obj.data), nb);
				}
			}
			else
			{
				Ret = HPS_ERROR_TYPE;
			}
		}
		else
		{
			Ret = HPS_ERROR_STACK;
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}
	
	return (Ret);
}
