#include <hpgcc49.h>
#include <hpgraphics.h>
#include <hpgbmp.h>
#include "vdefs.h"

Site *nextone(void);
void readsites(FILE *, int);

int triangulate, nsites, siteidx;
PREC xmin, xmax, ymin, ymax;
Site *sites;
Freelist sfl;

extern PREC pxmin, pxmax, pymin, pymax;

int plinesn = 0;
polyLine *plines;
int pedgesn = 0;
polyEdge *pedges;
int snapshots = 0;

typedef struct tCell {
	Point site;
	Point cog;
	PREC *vx;
	PREC *vy;
	int vn;
	PREC area, perimeter;
	int n, s, e, w;
} Cell;

typedef struct tPolyPoint {
	PREC theta;
	Point v;
} PolyPoint;

int polyedgecomp(const void *, const void *);
int checkpoint(PolyPoint *, int, PREC, PREC, int *, int *, int *, int *);
PREC theta(Point *, Point *);
int thetacomp(const void *, const void *);
int interact(Cell *, int, PREC);
int draw(PREC, PREC, PREC, PREC, Cell *, int, int, int);
int drawdia(PREC, PREC, PREC, PREC, Cell *, int, int, float, float, int);
int info(Cell *, int);
int snapshot();
#ifndef HPAPINE
extern unsigned int __heap_ptr,_heap_base_addr;
int freemem();
#endif

#define DIA_W 400
#define DIA_H 400

int main(int argc, char *argv[])
{
	SAT_STACK_ELEMENT e;
	SAT_STACK_DATA d;
	Site *(*next) ();
	char *filename, msg[40];
	FILE *fh = NULL;
	int i,j;
	Cell *cells;
	PREC cx, cy, dd, ld = 0;
	int center = -1;
	int maxn;
#ifndef HPAPINE
	int mem;
#endif
	triangulate = 0;

	sat_get_stack_element(1,&e);
	sat_decode_stack_element(&d,&e);

	if (d.type == SAT_DATA_TYPE_STRING) {
		filename = str_unquote(d.sval,'\'');
		sat_stack_drop();
	}
	else {
		sat_stack_push_string("Missing Filename");
		sat_push_real(1);
		return(0);
	}

	if((fh = fopen(filename,"r")) == NULL) {
		sprintf(msg,"Cannot read: \"%s\"",filename);

		sat_stack_push_string(msg);
		sat_push_real(1);
		return(0);
	}

#ifndef HPAPINE
	mem = freemem();
	maxn = mem / 550;
#else
	maxn = 1000000;
#endif

	sys_slowOff(); //full speed ahead

	hpg_set_mode_gray4(1);
	hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
	hpg_set_font(hpg_stdscreen, hpg_get_bigfont());
	hpg_clear();
	hpg_flip();
	hpg_clear();
	hpg_flip();

	freeinit(&sfl, sizeof(Site));
	readsites(fh,maxn);

	if(nsites == maxn) {
		hpg_clear();
		sprintf(msg,"Read only %d points!",nsites);
		hpg_draw_text(msg,0,0);
		hpg_draw_text("Low on memory.",0,18);
		hpg_draw_text("Press ON to continue.",0,36);
		hpg_flip();
		beep();
		WAIT_CANCEL;
	}

	hpg_clear();
	hpg_draw_text("Computing Voronoi...",6,38);
	hpg_flip();

	plines = (polyLine *) myalloc(ALLOCCHUNK * sizeof(polyLine));
	pedges = (polyEdge *) myalloc(2 * ALLOCCHUNK * sizeof(polyEdge));
	cells = (Cell *) myalloc(nsites * sizeof(Cell));
	for(i=0;i<nsites;i++)
		cells[sites[i].sitenbr].site = sites[i].coord;

	next = nextone;
	siteidx = 0;
	geominit();
	plotinit();
	voronoi(next);

	qsort((void *) pedges, pedgesn, sizeof(polyEdge), polyedgecomp);

	cx = (pxmax + pxmin)/2.0;
	cy = (pymax + pymin)/2.0;
	j=0;
	for(i=0;i<nsites;i++) {
		int vn, ln, t, b, r, l;
		int k, n, firstline = j;
		PolyPoint *poly = NULL;
		PREC A, CHx, CHy, PM;

		n = vn = ln = t = b = l = r = A = CHx = CHy = PM = 0;

		while(i == pedges[j].in) {
			n = pedges[j].n;

			if(checkpoint(poly,vn,plines[n].u.x,plines[n].u.y,&t,&b,&l,&r) == 0) {
				poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
				poly[vn].v = plines[n].u;
				poly[vn++].theta = theta(&plines[n].u,&cells[i].site);
			}
			if(checkpoint(poly,vn,plines[n].v.x,plines[n].v.y,&t,&b,&l,&r) == 0) {
				poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
				poly[vn].v = plines[n].v;
				poly[vn++].theta = theta(&plines[n].v,&cells[i].site);
			}
			ln++;

			j++;
		}

		if(ln != vn) {
			if (t == 1 && r == 1) {
				poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
				poly[vn].v.x = pxmax;
				poly[vn].v.y = pymax;
				poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
				vn++;
			}
			if (b == 1 && r == 1) {
				poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
				poly[vn].v.x = pxmax;
				poly[vn].v.y = pymin;
				poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
				vn++;
			}
			if (b == 1 && l == 1) {
				poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
				poly[vn].v.x = pxmin;
				poly[vn].v.y = pymin;
				poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
				vn++;
			}
			if (t == 1 && l == 1) {
				poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
				poly[vn].v.x = pxmin;
				poly[vn].v.y = pymax;
				poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
				vn++;
			}

			if(t == 1 && b == 1) {
				PREC x = (plines[n].c - plines[n].b * cells[i].site.y)/plines[n].a;

				poly = realloc(poly, (vn+2) * sizeof(PolyPoint));

				if(cells[i].site.x > x) {
					poly[vn].v.x = pxmax;
					poly[vn].v.y = pymin;
					poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
					vn++;
					poly[vn].v.x = pxmax;
					poly[vn].v.y = pymax;
					poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
					vn++;
				}
				else {
					poly[vn].v.x = pxmin;
					poly[vn].v.y = pymin;
					poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
					vn++;
					poly[vn].v.x = pxmin;
					poly[vn].v.y = pymax;
					poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
					vn++;
				}
			}

			if(r == 1 && l == 1) {
				PREC y = (plines[n].c - plines[n].a * cells[i].site.x)/plines[n].b;

				poly = realloc(poly, (vn+2) * sizeof(PolyPoint));

				if(cells[i].site.y > y) {
					poly[vn].v.x = pxmin;
					poly[vn].v.y = pymax;
					poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
					vn++;
					poly[vn].v.x = pxmax;
					poly[vn].v.y = pymax;
					poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
					vn++;
				}
				else {
					poly[vn].v.x = pxmin;
					poly[vn].v.y = pymin;
					poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
					vn++;
					poly[vn].v.x = pxmax;
					poly[vn].v.y = pymin;
					poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
					vn++;
				}
			}
		}

		qsort((void *) poly, vn, sizeof(PolyPoint), thetacomp);

		cells[i].vn = vn;
		cells[i].vx = (PREC *) myalloc(vn * sizeof(PREC));
		cells[i].vy = (PREC *) myalloc(vn * sizeof(PREC));

		for(k=0;k<vn;k++) {
			cells[i].vx[k] = poly[k].v.x;
			cells[i].vy[k] = poly[k].v.y;
		}

		for (k=0;k<vn;k++) {
			PREC C, Px, Py;
			int l = (k + 1) % vn;

			A += poly[k].v.x * poly[l].v.y;
			A -= poly[k].v.y * poly[l].v.x;
			C = ((poly[k].v.x * poly[l].v.y) - (poly[l].v.x * poly[k].v.y));
			CHx += (poly[k].v.x + poly[l].v.x) * C;
			CHy += (poly[k].v.y + poly[l].v.y) * C;
			Px = (poly[l].v.x - poly[k].v.x);
			Py = (poly[l].v.y - poly[k].v.y);
			PM += sqrt(Px * Px + Py * Py);
		}
		A /= 2;
		CHx /= (6 * A);
		CHy /= (6 * A);

		dd = ((CHx - cx)*(CHx - cx) + (CHy - cy)*(CHy - cy));
		if(dd < ld || ld == 0) {
			center = i;
			ld = dd;
		}

		cells[i].cog.x = CHx;
		cells[i].cog.y = CHy;
		cells[i].area = A;
		cells[i].perimeter = PM;

		cells[i].n = -1;
		cells[i].s = -1;
		cells[i].e = -1;
		cells[i].w = -1;
		for(k=firstline;k<j;k++) {
			int out = pedges[k].out;
			int n = pedges[k].n;
			PREC x, y;

			x = CHx;
			if(plines[n].b != 0) {
				y = (plines[n].c - plines[n].a * CHx)/plines[n].b;

				if((x > plines[n].u.x && x < plines[n].v.x) || (x > plines[n].v.x && x < plines[n].u.x)) {
					if(y > CHy)
						cells[i].n = out;
					else
						cells[i].s = out;
				}
			}

			y = CHy;
			if(plines[n].a != 0) {
				x = (plines[n].c - plines[n].b * CHy)/plines[n].a;

				if((y > plines[n].u.y && y < plines[n].v.y) || (y > plines[n].v.y && y < plines[n].u.y)) {
					if(x > CHx)
						cells[i].e = out;
					else
						cells[i].w = out;
				}
			}
		}

		free(poly);
	}

	free(plines);
	free(pedges);
	free(sites);

	beep();
	interact(cells,nsites,center);
	sat_push_real(0);
	sys_slowOn();
	return(0);
}

/*
#define W 131.0
#define H 74.0
#define BARS 0
*/
#define W 127.0
#define H 68.0
#define BARS 1

int interact(Cell *cells, int nsites, PREC center) {
	int zoom = 1, site = center;
	PREC l, r, b, t;
	PREC dx, dy;
	PREC w = pxmax - pxmin;
	PREC h = pymax - pymin;
	PREC cx = (pxmax + pxmin)/2.0;
	PREC cy = (pymax + pymin)/2.0;
	PREC bb_l, bb_r, bb_b, bb_t;
	int doinfo = 0, dosnap = 0, autozoom = 0;

	if(w/h > W/H) {
		l = pxmin;
		r = pxmax;
		b = cy - w*((H/2)/W);
		t = cy + w*((H/2)/W);
	}
	else {
		b = pymin;
		t = pymax;
		l = cx - h*(W/(H*2));
		r = cx + h*(W/(H*2));
	}

	draw(l, r, b, t, cells, nsites, site, zoom);

	for(;;) {
		sys_slowOn();   //conserve battery while user thinks
		while(!keyb_isAnyKeyPressed())
			;
		sys_slowOff();  //ok, got key, burn battery

		if(keyb_isON())
			break;

		if(keyb_isKeyPressed(0,5)) { // +
			zoom++;
		}
		if(keyb_isKeyPressed(0,4)) { // -
			zoom--;
			if(zoom == 0)
				zoom = 1;
		}
		if(keyb_isKeyPressed(0,2)) { // divide
			zoom /= 2;
			if(zoom == 0)
				zoom = 1;
		}
		if(keyb_isKeyPressed(0,3)) { // *
			zoom *= 2;
		}
		if(keyb_isKeyPressed(1,6) || keyb_isKeyPressed(3,6)) { // SPC
			zoom = 1;
		}
		if(keyb_isKeyPressed(2,6)) { // .
			autozoom = 1;
		}
		if(keyb_isUp()) {
			if(cells[site].n != -1)
				site = cells[site].n;
		}
		if(keyb_isDown()) {
			if(cells[site].s != -1)
				site = cells[site].s;
		}
		if(keyb_isLeft()) {
			if(cells[site].w != -1)
				site = cells[site].w;
		}
		if(keyb_isRight()) {
			if(cells[site].e != -1)
				site = cells[site].e;
		}
		if(keyb_isKeyPressed(7,1)) { // NXT
			site++;
			if(site == nsites)
				site = nsites - 1;
			if(zoom > 1)
				autozoom = 1;
		}
		if(keyb_isKeyPressed(7,0)) { // STO
			site--;
			if(site < 0)
				site = 0;
			if(zoom > 1)
				autozoom = 1;
		}
		if(keyb_isKeyPressed(6,5) || keyb_isKeyPressed(0,6)) // I or ENTER
			doinfo = 1;
		if(keyb_isKeyPressed(2,1)) // SIN
			dosnap = 1;

		if(autozoom) {
			int i;
			PREC my_l, my_r, my_b, my_t;
			int zy, zx;

			my_l = cells[site].vx[0];
			my_r = cells[site].vx[0];
			my_b = cells[site].vy[0];
			my_t = cells[site].vy[0];

			for(i=1;i<cells[site].vn;i++) {
				if(cells[site].vx[i] < my_l)
					my_l = cells[site].vx[i];
				if(cells[site].vx[i] > my_r)
					my_r = cells[site].vx[i];
				if(cells[site].vy[i] < my_b)
					my_b = cells[site].vy[i];
				if(cells[site].vy[i] > my_t)
					my_t = cells[site].vy[i];
			}

			zx = w/(my_r-my_l);
			zy = h/(my_t-my_b);

			if(zx < zy)
				zoom=zx;
			else
				zoom=zy;

			autozoom = 0;
		}

		cx = cells[site].cog.x;
		cy = cells[site].cog.y;

		if(zoom == 1) {
			draw(l, r, b, t, cells, nsites, site, zoom);
		} else {
			dx = (r - l)/(zoom+1);
			dy = (t - b)/(zoom+1);

			if(cx-dx < pxmin) {
				bb_l = pxmin;
				bb_r = pxmin + 2.0*dx;
			}
			else if(cx+dx > pxmax) {
				bb_l = pxmax - 2.0*dx;
				bb_r = pxmax;
			}
			else {
				bb_l = cx-dx;
				bb_r = cx+dx;
			}

			if(w < dx*2.0) {
				bb_l = (pxmax + pxmin)/2.0 - dx;
				bb_r = (pxmax + pxmin)/2.0 + dx;
			}

			if(cy-dy < pymin) {
				bb_b = pymin;
				bb_t = pymin + 2.0*dy;
			}
			else if(cy+dy > pymax) {
				bb_b = pymax - 2.0*dy;
				bb_t = pymax;
			}
			else {
				bb_b = cy-dy;
				bb_t = cy+dy;
			}

			if(h < dy*2.0) {
				bb_b = (pymax + pymin)/2.0 - dy;
				bb_t = (pymax + pymin)/2.0 + dy;
			}

			draw(bb_l, bb_r, bb_b, bb_t, cells, nsites, site, zoom);
		}

		if(doinfo) {
			info(cells,site);
			doinfo = 0;
		}

		if(dosnap) {
			int ret;

			snapshot();
			beep();
			hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_BLACK);
			hpg_flip();
			if(zoom == 1)
				ret = drawdia(l, r, b, t, cells, nsites, site, DIA_W, DIA_H, snapshots);
			else
				ret = drawdia(bb_l, bb_r, bb_b, bb_t, cells, nsites, site, DIA_W, DIA_H, snapshots);
			if(ret == 0)
				beep();
			hpg_flip();
			hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_WHITE);
			hpg_flip();
			dosnap = 0;
		}

		while(keyb_isAnyKeyPressed());
		sys_waitRTCTicks(2); //fix key bounce
	}

	return(0);
}

int draw(PREC l, PREC r, PREC b, PREC t, Cell *cells, int nsites, int site, int zoom) {
	PREC w = r - l;
	PREC h = t - b;
	PREC scale;
	int i;
	hpg_t *image;
	char sitename[20];
	char pointname[20];

	image = hpg_alloc_gray4_image(W+1,H);

	hpg_clear();
	hpg_clear_on(image);
	hpg_clip(image,1,0,W+1,H);

	if(w/h > W/H)
		scale = (W-1)/w;
	else
		scale = (H-1)/h;

	for(i=0;i<nsites;i++) {
		//if(cells[i].cog.x > l && cells[i].cog.x < r &&
		//cells[i].cog.y > b && cells[i].cog.y < t) {
		if(1) {
			int vn = cells[i].vn;
			int *vx, *vy;
			PREC x, y;
			int sx, sy, j;
			int ok = 0;

			vx = (int *) myalloc(vn * sizeof(int));
			vy = (int *) myalloc(vn * sizeof(int));

			for(j=0;j<vn;j++) {
				x  = (cells[i].vx[j] - l) * scale;
				x += ((W-1) - w*scale)/2.0;
				y  = (cells[i].vy[j] - b) * scale;
				y += ((H-1) - h*scale)/2.0;
				y  = (H-1) - y;
				vx[j] = x + 1.5;
				vy[j] = y + 0.5;
			}

			x  = (cells[i].site.x - l) * scale;
			x += ((W-1) - w*scale)/2.0;
			y  = (cells[i].site.y - b) * scale;
			y += ((H-1) - h*scale)/2.0;
			y  = (H-1) - y;
			sx = x + 1.5;
			sy = y + 0.5;

			if(sx > -1 && sx < W && sy > -1 && sy < H)
				ok = 1;

			if(site == i) {
				sprintf(sitename,"Site: %d %dx",site,zoom);
				//hpg_set_color(hpg_stdscreen, HPG_COLOR_GRAY_8);
				hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
				hpg_fill_rect_on(hpg_stdscreen,0,0,130,6);
				hpg_set_font(hpg_stdscreen, hpg_get_minifont());
				hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
				hpg_draw_text_on(hpg_stdscreen,sitename,1,1);

				sprintf(pointname,"%.2f,%.2f",cells[i].site.x,cells[i].site.y);
				hpg_draw_text_on(hpg_stdscreen,pointname,131-(strlen(pointname)*4),1);

				hpg_fill_polygon_on(image,vx,vy,vn);
				hpg_set_color(image, HPG_COLOR_WHITE);
				if(ok)
					hpg_draw_pixel_on(image,sx,sy);
				hpg_set_color(image, HPG_COLOR_BLACK);
			}
			else if(cells[site].n == i || cells[site].s == i ||
					cells[site].e == i || cells[site].w == i) {
				hpg_set_color(image, HPG_COLOR_GRAY_4);
				hpg_fill_polygon_on(image,vx,vy,vn);
				hpg_set_color(image, HPG_COLOR_BLACK);
				hpg_draw_polygon_on(image,vx,vy,vn);
				if(ok)
					hpg_draw_pixel_on(image,sx,sy);
			}
			else {
				hpg_draw_polygon_on(image,vx,vy,vn);
				if(ok)
					hpg_draw_pixel_on(image,sx,sy);
			}

			free(vx);
			free(vy);
		}
	}

	if(BARS) {
		PREC px, py;
		PREC lx, ly;
		int rx1, rx2, ry1, ry2;

		px = w / (pxmax - pxmin);
		py = h / (pymax - pymin);
		lx = (l - pxmin) / (pxmax - pxmin);
		ly = (b - pymin) / (pymax - pymin);

		if(px >= 1) {
			px = 1;
			lx = 0;
		}
		if(py >= 1) {
			py = 1;
			ly = 0;
		}

		rx1 = 126*lx + 0.5;
		rx2 = 126*(px+lx) + 0.5;
		if(rx1 < 0)
			rx1 = 0;
		if(rx2 > 126)
			rx2 = 126;

		ry1 = 67 - (67*ly)         +8.5;
		ry2 = 67 - (67*ly) - 67*py +8.5;
		if(ry2 < 8)
			ry2 = 8;
		if(ry1 > 75)
			ry1 = 75;

		//hpg_set_color(hpg_stdscreen, HPG_COLOR_GRAY_8);
		hpg_set_color(hpg_stdscreen, HPG_COLOR_GRAY_4);
		hpg_fill_rect_on(hpg_stdscreen,rx1,77,rx2,79);
		hpg_fill_rect_on(hpg_stdscreen,128,ry2,130,ry1);

		hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
		hpg_draw_line_on(hpg_stdscreen,0,78,126,78);
		hpg_draw_line_on(hpg_stdscreen,129,8,129,75);
	}

	hpg_blit(image,1,0,W,H,hpg_stdscreen,0,8);
	hpg_flip();
	hpg_free_image(image);
	return(0);
}

int drawdia(PREC l, PREC r, PREC b, PREC t, Cell *cells, int nsites, int site, float width, float height, int n) {
	PREC w = pxmax - pxmin;
	PREC h = pymax - pymin;
	PREC scale;
	int i,l1,l2;
	hpg_t *image;
	char filename[12];
	PREC x, y;
	int x1, y1, x2, y2, ret;
	palette_t mycolors[16];

	sprintf(filename,"DIA%03d.BMP",n);
	if((image = hpg_alloc_gray16_image(width,height)) == NULL)
		return(1);

	hpg_clear();
	hpg_clear_on(image);

	if(w/h > width/height)
		scale = (width-1)/w;
	else
		scale = (height-1)/h;

	for(i=0;i<nsites;i++) {
		int vn = cells[i].vn;
		int *vx, *vy;
		int sx, sy, j;
		int ok = 0;

		vx = (int *) myalloc(vn * sizeof(int));
		vy = (int *) myalloc(vn * sizeof(int));

		for(j=0;j<vn;j++) {
			x  = (cells[i].vx[j] - pxmin) * scale;
			x += ((width-1) - w*scale)/2.0;
			y  = (cells[i].vy[j] - pymin) * scale;
			y += ((height-1) - h*scale)/2.0;
			y  = (height-1) - y;
			vx[j] = x + 0.5;
			vy[j] = y + 0.5;
		}

		x  = (cells[i].site.x - pxmin) * scale;
		x += ((width-1) - w*scale)/2.0;
		y  = (cells[i].site.y - pymin) * scale;
		y += ((height-1) - h*scale)/2.0;
		y  = (height-1) - y;
		sx = x + 0.5;
		sy = y + 0.5;

		if(sx > -1 && sx < width && sy > -1 && sy < height)
			ok = 1;

		if(site == i) {
			hpg_set_color(image, HPG_COLOR_BLACK);
			hpg_fill_polygon_on(image,vx,vy,vn);
			hpg_set_color(image, HPG_COLOR_WHITE);
			if(ok)
				hpg_fill_circle_on(image,sx,sy,1);
			hpg_set_color(image, HPG_COLOR_BLACK);
		}
		else if(cells[site].n == i || cells[site].s == i || 
				cells[site].e == i || cells[site].w == i) {
			hpg_set_color(image, HPG_COLOR_GRAY_4);
			hpg_fill_polygon_on(image,vx,vy,vn);
			hpg_set_color(image, HPG_COLOR_BLACK);
			hpg_draw_polygon_on(image,vx,vy,vn);
			if(ok)
				hpg_fill_circle_on(image,sx,sy,1);
		}
		else {

			hpg_draw_polygon_on(image,vx,vy,vn);
			if(ok)
				hpg_fill_circle_on(image,sx,sy,1);
		}

		free(vx);
		free(vy);
	}

	x  = (l - pxmin) * scale;
	x += ((width-1) - w*scale)/2.0;
	y  = (t - pymin) * scale;
	y += ((height-1) - h*scale)/2.0;
	y  = (height-1) - y;
	x1 = x + 0.5;
	y1 = y + 0.5;

	x  = (r - pxmin) * scale;
	x += ((width-1) - w*scale)/2.0;
	y  = (b - pymin) * scale;
	y += ((height-1) - h*scale)/2.0;
	y  = (height-1) - y;
	x2 = x + 0.5;
	y2 = y + 0.5;

	hpg_set_color(image, HPG_COLOR_GRAY_8);
	for(l1 = x1; l1 <= x2; l1++)
		for(l2 = y1; l2 <= y2; l2++) {
			int p;

			p = hpg_get_pixel(image,l1,l2);
			if(p == 0)
				if(l1 >=0 && l1 < width && l2 >=0 && l2 < height)
					hpg_draw_pixel_on(image,l1,l2);
		}

#define R 2
#define G 1
#define B 0

	for (i = 0; i < 16; i++)
	{
		mycolors[15 - i].rgb[0] = i * 16 + i;
		mycolors[15 - i].rgb[1] = i * 16 + i;
		mycolors[15 - i].rgb[2] = i * 16 + i;
		mycolors[15 - i].rgb[3] = 0;
	}

	mycolors[8].rgb[R] = 0xd9;
	mycolors[8].rgb[G] = 0xea;
	mycolors[8].rgb[B] = 0xdb;

	ret = hpg_save_anybmp_gray16(filename,mycolors,image,width,height);
	hpg_free_image(image);
	return(ret);
}

#define INFO_L 2
#define INFO_R 123
#define INFO_T 10
#define INFO_B 72
#define LINESPACE 7
#define MARGIN 3 

int info(Cell *cells, int site)
{
	char line[30];
	int lc=0;

	hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
	hpg_draw_rect_on(hpg_stdscreen,INFO_L+1,INFO_T+1,INFO_R+1,INFO_B+1);
	hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
	hpg_fill_rect_on(hpg_stdscreen,INFO_L,INFO_T,INFO_R,INFO_B);
	hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
	hpg_draw_rect_on(hpg_stdscreen,INFO_L,INFO_T,INFO_R,INFO_B);
	hpg_set_font(hpg_stdscreen, hpg_get_minifont());

	if(cells[site].area < 10000 && cells[site].area > 0.1)
		sprintf(line,"Area:        %.6f",cells[site].area);
	else
		sprintf(line,"Area:        %.6E",cells[site].area);
	hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
	lc++;

	if(cells[site].perimeter < 10000 && cells[site].perimeter > 0.1)
		sprintf(line,"Perimeter:   %.6f",cells[site].perimeter);
	else
		sprintf(line,"Perimeter:   %.6E",cells[site].perimeter);
	hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
	lc++;

	if(cells[site].cog.x < 10000 && cells[site].cog.x > 0.1)
		sprintf(line,"Xcog:        %.6f",cells[site].cog.x);
	else
		sprintf(line,"Xcog:        %.6E",cells[site].cog.x);
	hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
	lc++;

	if(cells[site].cog.y < 10000 && cells[site].cog.y > 0.1)
		sprintf(line,"Ycog:        %.6f",cells[site].cog.y);
	else
		sprintf(line,"Ycog:        %.6E",cells[site].cog.y);
	hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
	lc++;

	sprintf(line,"Site North:  %d",cells[site].n);
	hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
	lc++;

	sprintf(line,"Site South:  %d",cells[site].s);
	hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
	lc++;

	sprintf(line,"Site East:   %d",cells[site].e);
	hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
	lc++;

	sprintf(line,"Site West:   %d",cells[site].w);
	hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
	lc++;

	hpg_flip();

	while(keyb_isAnyKeyPressed());
	sys_waitRTCTicks(2); //fix key bounce
	sys_slowOn();
	keyb_waitKeyPressed();
	sys_slowOff();

	hpg_flip();
	return(0);
}

int snapshot()
{
	char name[12];

	sprintf(name,"PIC%03d.BMP",++snapshots);
	hpg_save_bmp_gray16(name,NULL);

	return(0);
}

#ifndef HPAPINE
int freemem()
{
	register unsigned int stack_ptr asm ("sp");
	unsigned int base;

	base=(__heap_ptr == 0)? _heap_base_addr:__heap_ptr;

	return stack_ptr-base;
}
#endif

PREC theta(Point *p, Point *s)
{
	return(atan2(p->y-s->y,p->x-s->x) + 3.14159265358979);
}

int checkpoint(PolyPoint *p, int vn, PREC x, PREC y, int *t, int *b, int *l, int *r) {
	int i;

	for(i=0;i<vn;i++)
		if(p[i].v.x == x && p[i].v.y == y)
			return(1);

	if (x == pxmax)
		++*r;
	if (x == pxmin)
		++*l;
	if (y == pymax)
		++*t;
	if (y == pymin)
		++*b;

	return(0);
}

int thetacomp(const void *a0, const void *b0)
{
	PREC *a = (PREC *) a0;
	PREC *b = (PREC *) b0;

	if (*a < *b) {
		return (-1);
	}
	if (*a > *b) {
		return (1);
	}
	return (0);
}

int polyedgecomp(const void *a0, const void *b0)
{
	polyEdge *a = (polyEdge *) a0;
	polyEdge *b = (polyEdge *) b0;

	if (a->in < b->in) {
		return (-1);
	}
	if (a->in > b->in) {
		return (1);
	}
	return (0);
}

// sort sites on y, then x, coord
int scomp(const void *vs1, const void *vs2)
{
	Point *s1 = (Point *) vs1;
	Point *s2 = (Point *) vs2;

	if (s1->y < s2->y) {
		return (-1);
	}
	if (s1->y > s2->y) {
		return (1);
	}
	if (s1->x < s2->x) {
		return (-1);
	}
	if (s1->x > s2->x) {
		return (1);
	}
	return (0);
}

// return a single in-storage site
Site *nextone(void)
{
	Site *s;

	if (siteidx < nsites) {
		s = &sites[siteidx++];
		return (s);
	} else {
		return ((Site *) NULL);
	}
}

// read all sites, sort, and compute xmin, xmax, ymin, ymax
void readsites(FILE *fh, int maxn)
{
	int i, n=0;
	char line[40];
	double x, y;

	nsites = 0;
	sites = (Site *) myalloc(ALLOCCHUNK * sizeof(Site));

	while(fgets(line,sizeof(line),fh) != NULL && n++ < maxn) {
		if(sscanf(line,"%lg %lg", &x, &y) != 2)
			continue;
		sites[nsites].coord.x = x;
		sites[nsites].coord.y = y;
		sites[nsites].sitenbr = nsites;
		sites[nsites++].refcnt = 0;
		if (nsites % ALLOCCHUNK == 0)
			sites = (Site *) realloc(sites, (nsites + ALLOCCHUNK) * sizeof(Site));
	}

	qsort((void *) sites, nsites, sizeof(Site), scomp);
	xmin = sites[0].coord.x;
	xmax = sites[0].coord.x;
	for (i = 1; i < nsites; ++i) {
		if (sites[i].coord.x < xmin) {
			xmin = sites[i].coord.x;
		}
		if (sites[i].coord.x > xmax) {
			xmax = sites[i].coord.x;
		}
	}
	ymin = sites[0].coord.y;
	ymax = sites[nsites - 1].coord.y;
}

