1     	#include <hpgcc49.h>
2     	#include <hpgraphics.h>
3     	#include <hpgbmp.h>
4     	#include "vdefs.h"
5     	
6     	Site *nextone(void);
7     	void readsites(FILE *, int);
8     	
9     	int triangulate, nsites, siteidx;
10    	PREC xmin, xmax, ymin, ymax;
11    	Site *sites;
12    	Freelist sfl;
13    	
14    	extern PREC pxmin, pxmax, pymin, pymax;
15    	
16    	int plinesn = 0;
17    	polyLine *plines;
18    	int pedgesn = 0;
19    	polyEdge *pedges;
20    	int snapshots = 0;
21    	
22    	typedef struct tCell {
23    	    Point site;
24    	    Point cog;
25    	    PREC *vx;
26    	    PREC *vy;
27    	    int vn;
28    	    PREC area, perimeter;
29    	    int n, s, e, w;
30    	} Cell;
31    	
32    	typedef struct tPolyPoint {
33    	    PREC theta;
34    	    Point v;
35    	} PolyPoint;
36    	
37    	int polyedgecomp(const void *, const void *);
38    	int checkpoint(PolyPoint *, int, PREC, PREC, int *, int *, int *, int *);
39    	PREC theta(Point *, Point *);
40    	int thetacomp(const void *, const void *);
41    	int interact(Cell *, int, PREC);
42    	int draw(PREC, PREC, PREC, PREC, Cell *, int, int, int);
43    	int drawdia(PREC, PREC, PREC, PREC, Cell *, int, int, float, float, int);
44    	int info(Cell *, int);
45    	int snapshot();
46    	#ifndef HPAPINE
47    	extern unsigned int __heap_ptr,_heap_base_addr;
48    	int freemem();
49    	#endif
50    	
51    	#define DIA_W 500
52    	#define DIA_H 500
53    	
54    	int main(int argc, char *argv[])
55    	{
56    	    SAT_STACK_ELEMENT e;
57    	    SAT_STACK_DATA d;
58    	    Site *(*next) ();
59    	    char *filename, msg[40];
60    	    FILE *fh = NULL;
61    	    int i,j;
62    	    Cell *cells;
63    	    PREC cx, cy, dd, ld = 0;
64    	    int center = -1;
65    	    int maxn;
66    	#ifndef HPAPINE
67    	    int mem;
68    	#endif
69    	    triangulate = 0;
70    	
71    	    sat_get_stack_element(1,&e);
72    	    sat_decode_stack_element(&d,&e);
73    	
74    	    if (d.type == SAT_DATA_TYPE_STRING) {
75    	        filename = str_unquote(d.sval,'\'');
76    	        sat_stack_drop();
77    	    }
78    	    else {
79    	        sat_stack_push_string("Missing Filename");
80    	        sat_push_real(1);
81    	        return(0);
82    	    }
83    	
84    	    if((fh = fopen(filename,"r")) == NULL) {
85    	        sprintf(msg,"Cannot read: \"%s\"",filename);
86    	
87    	        sat_stack_push_string(msg);
88    	        sat_push_real(1);
89    	        return(0);
90    	    }
91    	
92    	#ifndef HPAPINE
93    	    mem = freemem();
94    	    maxn = mem / 550;
95    	#else
96    	    maxn = 1000000;
97    	#endif
98    	
99    	    sys_slowOff(); //full speed ahead
100   	
101   	    hpg_set_mode_gray4(1);
102   	    hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
103   	    hpg_set_font(hpg_stdscreen, hpg_get_bigfont());
104   	    hpg_clear();
105   	    hpg_flip();
106   	    hpg_clear();
107   	    hpg_flip();
108   	
109   	    freeinit(&sfl, sizeof(Site));
110   	    readsites(fh,maxn);
111   	
112   	    if(nsites == maxn) {
113   	        hpg_clear();
114   	        sprintf(msg,"Read only %d points!",nsites);
115   	        hpg_draw_text(msg,0,0);
116   	        hpg_draw_text("Low on memory.",0,18);
117   	        hpg_draw_text("Press ON to continue.",0,36);
118   	        hpg_flip();
119   	        beep();
120   	        WAIT_CANCEL;
121   	    }
122   	
123   	    hpg_clear();
124   	    hpg_draw_text("Computing Voronoi...",6,38);
125   	    hpg_flip();
126   	
127   	    plines = (polyLine *) myalloc(ALLOCCHUNK * sizeof(polyLine));
128   	    pedges = (polyEdge *) myalloc(2 * ALLOCCHUNK * sizeof(polyEdge));
129   	    cells = (Cell *) myalloc(nsites * sizeof(Cell));
130   	    for(i=0;i<nsites;i++)
131   	        cells[sites[i].sitenbr].site = sites[i].coord;
132   	
133   	    next = nextone;
134   	    siteidx = 0;
135   	    geominit();
136   	    plotinit();
137   	    voronoi(next);
138   	
139   	    qsort((void *) pedges, pedgesn, sizeof(polyEdge), polyedgecomp);
140   	
141   	    cx = (pxmax + pxmin)/2.0;
142   	    cy = (pymax + pymin)/2.0;
143   	    j=0;
144   	    for(i=0;i<nsites;i++) {
145   	        int vn, ln, t, b, r, l;
146   	        int k, n, firstline = j;
147   	        PolyPoint *poly = NULL;
148   	        PREC A, CHx, CHy, PM;
149   	
150   	        n = vn = ln = t = b = l = r = A = CHx = CHy = PM = 0;
151   	
152   	        while(i == pedges[j].in) {
153   	            n = pedges[j].n;
154   	
155   	            if(checkpoint(poly,vn,plines[n].u.x,plines[n].u.y,&t,&b,&l,&r) == 0) {
156   	                poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
157   	                poly[vn].v = plines[n].u;
158   	                poly[vn++].theta = theta(&plines[n].u,&cells[i].site);
159   	            }
160   	            if(checkpoint(poly,vn,plines[n].v.x,plines[n].v.y,&t,&b,&l,&r) == 0) {
161   	                poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
162   	                poly[vn].v = plines[n].v;
163   	                poly[vn++].theta = theta(&plines[n].v,&cells[i].site);
164   	            }
165   	            ln++;
166   	
167   	            j++;
168   	        }
169   	
170   	        if(ln != vn) {
171   	            if (t == 1 && r == 1) {
172   	                poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
173   	                poly[vn].v.x = pxmax;
174   	                poly[vn].v.y = pymax;
175   	                poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
176   	                vn++;
177   	            }
178   	            if (b == 1 && r == 1) {
179   	                poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
180   	                poly[vn].v.x = pxmax;
181   	                poly[vn].v.y = pymin;
182   	                poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
183   	                vn++;
184   	            }
185   	            if (b == 1 && l == 1) {
186   	                poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
187   	                poly[vn].v.x = pxmin;
188   	                poly[vn].v.y = pymin;
189   	                poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
190   	                vn++;
191   	            }
192   	            if (t == 1 && l == 1) {
193   	                poly = realloc(poly, (vn+1) * sizeof(PolyPoint));
194   	                poly[vn].v.x = pxmin;
195   	                poly[vn].v.y = pymax;
196   	                poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
197   	                vn++;
198   	            }
199   	
200   	            if(t == 1 && b == 1) {
201   	                PREC x = (plines[n].c - plines[n].b * cells[i].site.y)/plines[n].a;
202   	
203   	                poly = realloc(poly, (vn+2) * sizeof(PolyPoint));
204   	
205   	                if(cells[i].site.x > x) {
206   	                    poly[vn].v.x = pxmax;
207   	                    poly[vn].v.y = pymin;
208   	                    poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
209   	                    vn++;
210   	                    poly[vn].v.x = pxmax;
211   	                    poly[vn].v.y = pymax;
212   	                    poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
213   	                    vn++;
214   	                }
215   	                else {
216   	                    poly[vn].v.x = pxmin;
217   	                    poly[vn].v.y = pymin;
218   	                    poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
219   	                    vn++;
220   	                    poly[vn].v.x = pxmin;
221   	                    poly[vn].v.y = pymax;
222   	                    poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
223   	                    vn++;
224   	                }
225   	            }
226   	
227   	            if(r == 1 && l == 1) {
228   	                PREC y = (plines[n].c - plines[n].a * cells[i].site.x)/plines[n].b;
229   	
230   	                poly = realloc(poly, (vn+2) * sizeof(PolyPoint));
231   	
232   	                if(cells[i].site.y > y) {
233   	                    poly[vn].v.x = pxmin;
234   	                    poly[vn].v.y = pymax;
235   	                    poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
236   	                    vn++;
237   	                    poly[vn].v.x = pxmax;
238   	                    poly[vn].v.y = pymax;
239   	                    poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
240   	                    vn++;
241   	                }
242   	                else {
243   	                    poly[vn].v.x = pxmin;
244   	                    poly[vn].v.y = pymin;
245   	                    poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
246   	                    vn++;
247   	                    poly[vn].v.x = pxmax;
248   	                    poly[vn].v.y = pymin;
249   	                    poly[vn].theta = theta(&poly[vn].v,&cells[i].site);
250   	                    vn++;
251   	                }
252   	            }
253   	        }
254   	
255   	        qsort((void *) poly, vn, sizeof(PolyPoint), thetacomp);
256   	
257   	        cells[i].vn = vn;
258   	        cells[i].vx = (PREC *) myalloc(vn * sizeof(PREC));
259   	        cells[i].vy = (PREC *) myalloc(vn * sizeof(PREC));
260   	
261   	        for(k=0;k<vn;k++) {
262   	            cells[i].vx[k] = poly[k].v.x;
263   	            cells[i].vy[k] = poly[k].v.y;
264   	        }
265   	
266   	        for (k=0;k<vn;k++) {
267   	            PREC C, Px, Py;
268   	            int l = (k + 1) % vn;
269   	
270   	            A += poly[k].v.x * poly[l].v.y;
271   	            A -= poly[k].v.y * poly[l].v.x;
272   	            C = ((poly[k].v.x * poly[l].v.y) - (poly[l].v.x * poly[k].v.y));
273   	            CHx += (poly[k].v.x + poly[l].v.x) * C;
274   	            CHy += (poly[k].v.y + poly[l].v.y) * C;
275   	            Px = (poly[l].v.x - poly[k].v.x);
276   	            Py = (poly[l].v.y - poly[k].v.y);
277   	            PM += sqrt(Px * Px + Py * Py);
278   	        }
279   	        A /= 2;
280   	        CHx /= (6 * A);
281   	        CHy /= (6 * A);
282   	
283   	        dd = ((CHx - cx)*(CHx - cx) + (CHy - cy)*(CHy - cy));
284   	        if(dd < ld || ld == 0) {
285   	            center = i;
286   	            ld = dd;
287   	        }
288   	
289   	        cells[i].cog.x = CHx;
290   	        cells[i].cog.y = CHy;
291   	        cells[i].area = A;
292   	        cells[i].perimeter = PM;
293   	
294   	        cells[i].n = -1;
295   	        cells[i].s = -1;
296   	        cells[i].e = -1;
297   	        cells[i].w = -1;
298   	        for(k=firstline;k<j;k++) {
299   	            int out = pedges[k].out;
300   	            int n = pedges[k].n;
301   	            PREC x, y;
302   	
303   	            x = CHx;
304   	            if(plines[n].b != 0) {
305   	                y = (plines[n].c - plines[n].a * CHx)/plines[n].b;
306   	
307   	                if((x > plines[n].u.x && x < plines[n].v.x) || (x > plines[n].v.x && x < plines[n].u.x)) {
308   	                    if(y > CHy)
309   	                        cells[i].n = out;
310   	                    else
311   	                        cells[i].s = out;
312   	                }
313   	            }
314   	
315   	            y = CHy;
316   	            if(plines[n].a != 0) {
317   	                x = (plines[n].c - plines[n].b * CHy)/plines[n].a;
318   	
319   	                if((y > plines[n].u.y && y < plines[n].v.y) || (y > plines[n].v.y && y < plines[n].u.y)) {
320   	                    if(x > CHx)
321   	                        cells[i].e = out;
322   	                    else
323   	                        cells[i].w = out;
324   	                }
325   	            }
326   	        }
327   	
328   	        free(poly);
329   	    }
330   	
331   	    free(plines);
332   	    free(pedges);
333   	    free(sites);
334   	
335   	    beep();
336   	    interact(cells,nsites,center);
337   	    sat_push_real(0);
338   	    sys_slowOn();
339   	    return(0);
340   	}
341   	
342   	/*
343   	#define W 131.0
344   	#define H 74.0
345   	#define BARS 0
346   	*/
347   	#define W 127.0
348   	#define H 68.0
349   	#define BARS 1
350   	
351   	int interact(Cell *cells, int nsites, PREC center) {
352   	    int zoom = 1, site = center;
353   	    PREC l, r, b, t;
354   	    PREC dx, dy;
355   	    PREC w = pxmax - pxmin;
356   	    PREC h = pymax - pymin;
357   	    PREC cx = (pxmax + pxmin)/2.0;
358   	    PREC cy = (pymax + pymin)/2.0;
359   	    PREC bb_l, bb_r, bb_b, bb_t;
360   	    int doinfo = 0, dosnap = 0, autozoom = 0;
361   	
362   	    if(w/h > W/H) {
363   	        l = pxmin;
364   	        r = pxmax;
365   	        b = cy - w*((H/2)/W);
366   	        t = cy + w*((H/2)/W);
367   	    }
368   	    else {
369   	        b = pymin;
370   	        t = pymax;
371   	        l = cx - h*(W/(H*2));
372   	        r = cx + h*(W/(H*2));
373   	    }
374   	
375   	    draw(l, r, b, t, cells, nsites, site, zoom);
376   	
377   	    for(;;) {
378   	        sys_slowOn();   //conserve battery while user thinks
379   	        while(!keyb_isAnyKeyPressed())
380   	            ;
381   	        sys_slowOff();  //ok, got key, burn battery
382   	
383   	        if(keyb_isON())
384   	            break;
385   	
386   	        if(keyb_isKeyPressed(0,5)) { // +
387   	            zoom++;
388   	        }
389   	        if(keyb_isKeyPressed(0,4)) { // -
390   	            zoom--;
391   	            if(zoom == 0)
392   	                zoom = 1;
393   	        }
394   	        if(keyb_isKeyPressed(0,2)) { // divide
395   	            zoom /= 2;
396   	            if(zoom == 0)
397   	                zoom = 1;
398   	        }
399   	        if(keyb_isKeyPressed(0,3)) { // *
400   	            zoom *= 2;
401   	        }
402   	        if(keyb_isKeyPressed(1,6) || keyb_isKeyPressed(3,6)) { // SPC
403   	            zoom = 1;
404   	        }
405   	        if(keyb_isKeyPressed(2,6)) { // .
406   	            autozoom = 1;
407   	        }
408   	        if(keyb_isUp()) {
409   	            if(cells[site].n != -1)
410   	                site = cells[site].n;
411   	        }
412   	        if(keyb_isDown()) {
413   	            if(cells[site].s != -1)
414   	                site = cells[site].s;
415   	        }
416   	        if(keyb_isLeft()) {
417   	            if(cells[site].w != -1)
418   	                site = cells[site].w;
419   	        }
420   	        if(keyb_isRight()) {
421   	            if(cells[site].e != -1)
422   	                site = cells[site].e;
423   	        }
424   	        if(keyb_isKeyPressed(7,1)) { // NXT
425   	            site++;
426   	            if(site == nsites)
427   	                site = nsites - 1;
428   	            if(zoom > 1)
429   	                autozoom = 1;
430   	        }
431   	        if(keyb_isKeyPressed(7,0)) { // STO
432   	            site--;
433   	            if(site < 0)
434   	                site = 0;
435   	            if(zoom > 1)
436   	                autozoom = 1;
437   	        }
438   	        if(keyb_isKeyPressed(6,5) || keyb_isKeyPressed(0,6)) // I or ENTER
439   	            doinfo = 1;
440   	        if(keyb_isKeyPressed(2,1)) // SIN
441   	            dosnap = 1;
442   	
443   	        if(autozoom) {
444   	            int i;
445   	            PREC my_l, my_r, my_b, my_t;
446   	            int zy, zx;
447   	
448   	            my_l = cells[site].vx[0];
449   	            my_r = cells[site].vx[0];
450   	            my_b = cells[site].vy[0];
451   	            my_t = cells[site].vy[0];
452   	
453   	            for(i=1;i<cells[site].vn;i++) {
454   	                if(cells[site].vx[i] < my_l)
455   	                    my_l = cells[site].vx[i];
456   	                if(cells[site].vx[i] > my_r)
457   	                    my_r = cells[site].vx[i];
458   	                if(cells[site].vy[i] < my_b)
459   	                    my_b = cells[site].vy[i];
460   	                if(cells[site].vy[i] > my_t)
461   	                    my_t = cells[site].vy[i];
462   	            }
463   	
464   	            zx = w/(my_r-my_l);
465   	            zy = h/(my_t-my_b);
466   	
467   	            if(zx < zy)
468   	                zoom=zx;
469   	            else
470   	                zoom=zy;
471   	
472   	            autozoom = 0;
473   	        }
474   	
475   	        cx = cells[site].cog.x;
476   	        cy = cells[site].cog.y;
477   	
478   	        if(zoom == 1) {
479   	            draw(l, r, b, t, cells, nsites, site, zoom);
480   	        } else {
481   	            dx = (r - l)/(zoom+1);
482   	            dy = (t - b)/(zoom+1);
483   	
484   	            if(cx-dx < pxmin) {
485   	                bb_l = pxmin;
486   	                bb_r = pxmin + 2.0*dx;
487   	            }
488   	            else if(cx+dx > pxmax) {
489   	                bb_l = pxmax - 2.0*dx;
490   	                bb_r = pxmax;
491   	            }
492   	            else {
493   	                bb_l = cx-dx;
494   	                bb_r = cx+dx;
495   	            }
496   	
497   	            if(w < dx*2.0) {
498   	                bb_l = (pxmax + pxmin)/2.0 - dx;
499   	                bb_r = (pxmax + pxmin)/2.0 + dx;
500   	            }
501   	
502   	            if(cy-dy < pymin) {
503   	                bb_b = pymin;
504   	                bb_t = pymin + 2.0*dy;
505   	            }
506   	            else if(cy+dy > pymax) {
507   	                bb_b = pymax - 2.0*dy;
508   	                bb_t = pymax;
509   	            }
510   	            else {
511   	                bb_b = cy-dy;
512   	                bb_t = cy+dy;
513   	            }
514   	
515   	            if(h < dy*2.0) {
516   	                bb_b = (pymax + pymin)/2.0 - dy;
517   	                bb_t = (pymax + pymin)/2.0 + dy;
518   	            }
519   	
520   	            draw(bb_l, bb_r, bb_b, bb_t, cells, nsites, site, zoom);
521   	        }
522   	
523   	        if(doinfo) {
524   	            info(cells,site);
525   	            doinfo = 0;
526   	        }
527   	
528   	        if(dosnap) {
529   	            int ret;
530   	
531   	            snapshot();
532   	            beep();
533   	            hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_BLACK);
534   	            hpg_flip();
535   	            if(zoom == 1)
536   	                ret = drawdia(l, r, b, t, cells, nsites, site, DIA_W, DIA_H, snapshots);
537   	            else
538   	                ret = drawdia(bb_l, bb_r, bb_b, bb_t, cells, nsites, site, DIA_W, DIA_H, snapshots);
539   	            if(ret == 0)
540   	                beep();
541   	            hpg_flip();
542   	            hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_WHITE);
543   	            hpg_flip();
544   	            dosnap = 0;
545   	        }
546   	
547   	        while(keyb_isAnyKeyPressed());
548   	        sys_waitRTCTicks(2); //fix key bounce
549   	    }
550   	
551   	    return(0);
552   	}
553   	
554   	int draw(PREC l, PREC r, PREC b, PREC t, Cell *cells, int nsites, int site, int zoom) {
555   	    PREC w = r - l;
556   	    PREC h = t - b;
557   	    PREC scale;
558   	    int i;
559   	    hpg_t *image;
560   	    char sitename[20];
561   	    char pointname[20];
562   	
563   	    image = hpg_alloc_gray4_image(W+1,H);
564   	
565   	    hpg_clear();
566   	    hpg_clear_on(image);
567   	    hpg_clip(image,1,0,W+1,H);
568   	
569   	    if(w/h > W/H)
570   	        scale = (W-1)/w;
571   	    else
572   	        scale = (H-1)/h;
573   	
574   	    for(i=0;i<nsites;i++) {
575   	        //if(cells[i].cog.x > l && cells[i].cog.x < r &&
576   	        //cells[i].cog.y > b && cells[i].cog.y < t) {
577   	        if(1) {
578   	            int vn = cells[i].vn;
579   	            int *vx, *vy;
580   	            PREC x, y;
581   	            int sx, sy, j;
582   	            int ok = 0;
583   	
584   	            vx = (int *) myalloc(vn * sizeof(int));
585   	            vy = (int *) myalloc(vn * sizeof(int));
586   	
587   	            for(j=0;j<vn;j++) {
588   	                x  = (cells[i].vx[j] - l) * scale;
589   	                x += ((W-1) - w*scale)/2.0;
590   	                y  = (cells[i].vy[j] - b) * scale;
591   	                y += ((H-1) - h*scale)/2.0;
592   	                y  = (H-1) - y;
593   	                vx[j] = x + 1.5;
594   	                vy[j] = y + 0.5;
595   	            }
596   	
597   	            x  = (cells[i].site.x - l) * scale;
598   	            x += ((W-1) - w*scale)/2.0;
599   	            y  = (cells[i].site.y - b) * scale;
600   	            y += ((H-1) - h*scale)/2.0;
601   	            y  = (H-1) - y;
602   	            sx = x + 1.5;
603   	            sy = y + 0.5;
604   	
605   	            if(sx > -1 && sx < W && sy > -1 && sy < H)
606   	                ok = 1;
607   	
608   	            if(site == i) {
609   	                sprintf(sitename,"Site: %d %dx",site,zoom);
610   	                //hpg_set_color(hpg_stdscreen, HPG_COLOR_GRAY_8);
611   	                hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
612   	                hpg_fill_rect_on(hpg_stdscreen,0,0,130,6);
613   	                hpg_set_font(hpg_stdscreen, hpg_get_minifont());
614   	                hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
615   	                hpg_draw_text_on(hpg_stdscreen,sitename,1,1);
616   	
617   	                sprintf(pointname,"%.2f,%.2f",cells[i].site.x,cells[i].site.y);
618   	                hpg_draw_text_on(hpg_stdscreen,pointname,131-(strlen(pointname)*4),1);
619   	
620   	                hpg_fill_polygon_on(image,vx,vy,vn);
621   	                hpg_set_color(image, HPG_COLOR_WHITE);
622   	                if(ok)
623   	                    hpg_draw_pixel_on(image,sx,sy);
624   	                hpg_set_color(image, HPG_COLOR_BLACK);
625   	            }
626   	            else if(cells[site].n == i || cells[site].s == i ||
627   	                    cells[site].e == i || cells[site].w == i) {
628   	                hpg_set_color(image, HPG_COLOR_GRAY_4);
629   	                hpg_fill_polygon_on(image,vx,vy,vn);
630   	                hpg_set_color(image, HPG_COLOR_BLACK);
631   	                hpg_draw_polygon_on(image,vx,vy,vn);
632   	                if(ok)
633   	                    hpg_draw_pixel_on(image,sx,sy);
634   	            }
635   	            else {
636   	                hpg_draw_polygon_on(image,vx,vy,vn);
637   	                if(ok)
638   	                    hpg_draw_pixel_on(image,sx,sy);
639   	            }
640   	
641   	            free(vx);
642   	            free(vy);
643   	        }
644   	    }
645   	
646   	    if(BARS) {
647   	        PREC px, py;
648   	        PREC lx, ly;
649   	        int rx1, rx2, ry1, ry2;
650   	
651   	        px = w / (pxmax - pxmin);
652   	        py = h / (pymax - pymin);
653   	        lx = (l - pxmin) / (pxmax - pxmin);
654   	        ly = (b - pymin) / (pymax - pymin);
655   	
656   	        if(px >= 1) {
657   	            px = 1;
658   	            lx = 0;
659   	        }
660   	        if(py >= 1) {
661   	            py = 1;
662   	            ly = 0;
663   	        }
664   	
665   	        rx1 = 126*lx + 0.5;
666   	        rx2 = 126*(px+lx) + 0.5;
667   	        if(rx1 < 0)
668   	            rx1 = 0;
669   	        if(rx2 > 126)
670   	            rx2 = 126;
671   	
672   	        ry1 = 67 - (67*ly)         +8.5;
673   	        ry2 = 67 - (67*ly) - 67*py +8.5;
674   	        if(ry2 < 8)
675   	            ry2 = 8;
676   	        if(ry1 > 75)
677   	            ry1 = 75;
678   	
679   	        //hpg_set_color(hpg_stdscreen, HPG_COLOR_GRAY_8);
680   	        hpg_set_color(hpg_stdscreen, HPG_COLOR_GRAY_4);
681   	        hpg_fill_rect_on(hpg_stdscreen,rx1,77,rx2,79);
682   	        hpg_fill_rect_on(hpg_stdscreen,128,ry2,130,ry1);
683   	
684   	        hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
685   	        hpg_draw_line_on(hpg_stdscreen,0,78,126,78);
686   	        hpg_draw_line_on(hpg_stdscreen,129,8,129,75);
687   	    }
688   	
689   	    hpg_blit(image,1,0,W,H,hpg_stdscreen,0,8);
690   	    hpg_flip();
691   	    hpg_free_image(image);
692   	    return(0);
693   	}
694   	
695   	int drawdia(PREC l, PREC r, PREC b, PREC t, Cell *cells, int nsites, int site, float width, float height, int n) {
696   	    PREC w = pxmax - pxmin;
697   	    PREC h = pymax - pymin;
698   	    PREC scale;
699   	    int i,l1,l2;
700   	    hpg_t *image;
701   	    char filename[12];
702   	    PREC x, y;
703   	    int x1, y1, x2, y2, ret;
704   	    palette_t mycolors[16];
705   	
706   	    sprintf(filename,"DIA%03d.BMP",n);
707   	    if((image = hpg_alloc_gray16_image(width,height)) == NULL)
708   	        return(1);
709   	
710   	    hpg_clear();
711   	    hpg_clear_on(image);
712   	
713   	    if(w/h > width/height)
714   	        scale = (width-1)/w;
715   	    else
716   	        scale = (height-1)/h;
717   	
718   	    for(i=0;i<nsites;i++) {
719   	        int vn = cells[i].vn;
720   	        int *vx, *vy;
721   	        int sx, sy, j;
722   	        int ok = 0;
723   	
724   	        vx = (int *) myalloc(vn * sizeof(int));
725   	        vy = (int *) myalloc(vn * sizeof(int));
726   	
727   	        for(j=0;j<vn;j++) {
728   	            x  = (cells[i].vx[j] - pxmin) * scale;
729   	            x += ((width-1) - w*scale)/2.0;
730   	            y  = (cells[i].vy[j] - pymin) * scale;
731   	            y += ((height-1) - h*scale)/2.0;
732   	            y  = (height-1) - y;
733   	            vx[j] = x + 0.5;
734   	            vy[j] = y + 0.5;
735   	        }
736   	
737   	        x  = (cells[i].site.x - pxmin) * scale;
738   	        x += ((width-1) - w*scale)/2.0;
739   	        y  = (cells[i].site.y - pymin) * scale;
740   	        y += ((height-1) - h*scale)/2.0;
741   	        y  = (height-1) - y;
742   	        sx = x + 0.5;
743   	        sy = y + 0.5;
744   	
745   	        if(sx > -1 && sx < width && sy > -1 && sy < height)
746   	            ok = 1;
747   	
748   	        if(site == i) {
749   	            hpg_set_color(image, HPG_COLOR_BLACK);
750   	            hpg_fill_polygon_on(image,vx,vy,vn);
751   	            hpg_set_color(image, HPG_COLOR_WHITE);
752   	            if(ok)
753   	                hpg_fill_circle_on(image,sx,sy,1);
754   	            hpg_set_color(image, HPG_COLOR_BLACK);
755   	        }
756   	        else if(cells[site].n == i || cells[site].s == i || 
757   	                cells[site].e == i || cells[site].w == i) {
758   	            hpg_set_color(image, HPG_COLOR_GRAY_4);
759   	            hpg_fill_polygon_on(image,vx,vy,vn);
760   	            hpg_set_color(image, HPG_COLOR_BLACK);
761   	            hpg_draw_polygon_on(image,vx,vy,vn);
762   	            if(ok)
763   	                hpg_fill_circle_on(image,sx,sy,1);
764   	        }
765   	        else {
766   	            hpg_draw_polygon_on(image,vx,vy,vn);
767   	            if(ok)
768   	                hpg_fill_circle_on(image,sx,sy,1);
769   	        }
770   	
771   	        free(vx);
772   	        free(vy);
773   	    }
774   	
775   	    x  = (l - pxmin) * scale;
776   	    x += ((width-1) - w*scale)/2.0;
777   	    y  = (t - pymin) * scale;
778   	    y += ((height-1) - h*scale)/2.0;
779   	    y  = (height-1) - y;
780   	    x1 = x + 0.5;
781   	    y1 = y + 0.5;
782   	
783   	    x  = (r - pxmin) * scale;
784   	    x += ((width-1) - w*scale)/2.0;
785   	    y  = (b - pymin) * scale;
786   	    y += ((height-1) - h*scale)/2.0;
787   	    y  = (height-1) - y;
788   	    x2 = x + 0.5;
789   	    y2 = y + 0.5;
790   	
791   	    hpg_set_color(image, HPG_COLOR_GRAY_8);
792   	    for(l1 = x1; l1 <= x2; l1++)
793   	        for(l2 = y1; l2 <= y2; l2++) {
794   	            int p;
795   	
796   	            p = hpg_get_pixel(image,l1,l2);
797   	            if(p == 0)
798   	                if(l1 >=0 && l1 < width && l2 >=0 && l2 < height)
799   	                    hpg_draw_pixel_on(image,l1,l2);
800   	        }
801   	
802   	#define R 2
803   	#define G 1
804   	#define B 0
805   	
806   	    for (i = 0; i < 16; i++)
807   	    {
808   	        mycolors[15 - i].rgb[0] = i * 16 + i;
809   	        mycolors[15 - i].rgb[1] = i * 16 + i;
810   	        mycolors[15 - i].rgb[2] = i * 16 + i;
811   	        mycolors[15 - i].rgb[3] = 0;
812   	    }
813   	
814   	    mycolors[8].rgb[R] = 0xd9;
815   	    mycolors[8].rgb[G] = 0xea;
816   	    mycolors[8].rgb[B] = 0xdb;
817   	
818   	    ret = hpg_save_anybmp_gray16(filename,mycolors,image,width,height);
819   	    hpg_free_image(image);
820   	    return(ret);
821   	}
822   	
823   	#define INFO_L 2
824   	#define INFO_R 123
825   	#define INFO_T 10
826   	#define INFO_B 72
827   	#define LINESPACE 7
828   	#define MARGIN 3 
829   	
830   	int info(Cell *cells, int site)
831   	{
832   	    char line[30];
833   	    int lc=0;
834   	
835   	    hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
836   	    hpg_draw_rect_on(hpg_stdscreen,INFO_L+1,INFO_T+1,INFO_R+1,INFO_B+1);
837   	    hpg_set_color(hpg_stdscreen, HPG_COLOR_WHITE);
838   	    hpg_fill_rect_on(hpg_stdscreen,INFO_L,INFO_T,INFO_R,INFO_B);
839   	    hpg_set_color(hpg_stdscreen, HPG_COLOR_BLACK);
840   	    hpg_draw_rect_on(hpg_stdscreen,INFO_L,INFO_T,INFO_R,INFO_B);
841   	    hpg_set_font(hpg_stdscreen, hpg_get_minifont());
842   	
843   	    if(cells[site].area < 10000 && cells[site].area > 0.1)
844   	        sprintf(line,"Area:        %.6f",cells[site].area);
845   	    else
846   	        sprintf(line,"Area:        %.6E",cells[site].area);
847   	    hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
848   	    lc++;
849   	
850   	    if(cells[site].perimeter < 10000 && cells[site].perimeter > 0.1)
851   	        sprintf(line,"Perimeter:   %.6f",cells[site].perimeter);
852   	    else
853   	        sprintf(line,"Perimeter:   %.6E",cells[site].perimeter);
854   	    hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
855   	    lc++;
856   	
857   	    if(cells[site].cog.x < 10000 && cells[site].cog.x > 0.1)
858   	        sprintf(line,"Xcog:        %.6f",cells[site].cog.x);
859   	    else
860   	        sprintf(line,"Xcog:        %.6E",cells[site].cog.x);
861   	    hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
862   	    lc++;
863   	
864   	    if(cells[site].cog.y < 10000 && cells[site].cog.y > 0.1)
865   	        sprintf(line,"Ycog:        %.6f",cells[site].cog.y);
866   	    else
867   	        sprintf(line,"Ycog:        %.6E",cells[site].cog.y);
868   	    hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
869   	    lc++;
870   	
871   	    sprintf(line,"Site North:  %d",cells[site].n);
872   	    hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
873   	    lc++;
874   	
875   	    sprintf(line,"Site South:  %d",cells[site].s);
876   	    hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
877   	    lc++;
878   	
879   	    sprintf(line,"Site East:   %d",cells[site].e);
880   	    hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
881   	    lc++;
882   	
883   	    sprintf(line,"Site West:   %d",cells[site].w);
884   	    hpg_draw_text_on(hpg_stdscreen,line,INFO_L+MARGIN,INFO_T+MARGIN+LINESPACE*lc);
885   	    lc++;
886   	
887   	    hpg_flip();
888   	
889   	    while(keyb_isAnyKeyPressed());
890   	    sys_waitRTCTicks(2); //fix key bounce
891   	    sys_slowOn();
892   	    keyb_waitKeyPressed();
893   	    sys_slowOff();
894   	
895   	    hpg_flip();
896   	    return(0);
897   	}
898   	
899   	int snapshot()
900   	{
901   	    char name[12];
902   	
903   	    sprintf(name,"PIC%03d.BMP",++snapshots);
904   	    hpg_save_bmp_gray16(name,NULL);
905   	
906   	    return(0);
907   	}
908   	
909   	#ifndef HPAPINE
910   	int freemem()
911   	{
912   	    register unsigned int stack_ptr asm ("sp");
913   	    unsigned int base;
914   	
915   	    base=(__heap_ptr == 0)? _heap_base_addr:__heap_ptr;
916   	
917   	    return stack_ptr-base;
918   	}
919   	#endif
920   	
921   	PREC theta(Point *p, Point *s)
922   	{
923   	    return(atan2(p->y-s->y,p->x-s->x) + 3.14159265358979);
924   	}
925   	
926   	int checkpoint(PolyPoint *p, int vn, PREC x, PREC y, int *t, int *b, int *l, int *r) {
927   	    int i;
928   	
929   	    for(i=0;i<vn;i++)
930   	        if(p[i].v.x == x && p[i].v.y == y)
931   	            return(1);
932   	
933   	    if (x == pxmax)
934   	        ++*r;
935   	    if (x == pxmin)
936   	        ++*l;
937   	    if (y == pymax)
938   	        ++*t;
939   	    if (y == pymin)
940   	        ++*b;
941   	
942   	    return(0);
943   	}
944   	
945   	int thetacomp(const void *a0, const void *b0)
946   	{
947   	    PREC *a = (PREC *) a0;
948   	    PREC *b = (PREC *) b0;
949   	
950   	    if (*a < *b) {
951   	        return (-1);
952   	    }
953   	    if (*a > *b) {
954   	        return (1);
955   	    }
956   	    return (0);
957   	}
958   	
959   	int polyedgecomp(const void *a0, const void *b0)
960   	{
961   	    polyEdge *a = (polyEdge *) a0;
962   	    polyEdge *b = (polyEdge *) b0;
963   	
964   	    if (a->in < b->in) {
965   	        return (-1);
966   	    }
967   	    if (a->in > b->in) {
968   	        return (1);
969   	    }
970   	    return (0);
971   	}
972   	
973   	// sort sites on y, then x, coord
974   	int scomp(const void *vs1, const void *vs2)
975   	{
976   	    Point *s1 = (Point *) vs1;
977   	    Point *s2 = (Point *) vs2;
978   	
979   	    if (s1->y < s2->y) {
980   	        return (-1);
981   	    }
982   	    if (s1->y > s2->y) {
983   	        return (1);
984   	    }
985   	    if (s1->x < s2->x) {
986   	        return (-1);
987   	    }
988   	    if (s1->x > s2->x) {
989   	        return (1);
990   	    }
991   	    return (0);
992   	}
993   	
994   	// return a single in-storage site
995   	Site *nextone(void)
996   	{
997   	    Site *s;
998   	
999   	    if (siteidx < nsites) {
1000  	        s = &sites[siteidx++];
1001  	        return (s);
1002  	    } else {
1003  	        return ((Site *) NULL);
1004  	    }
1005  	}
1006  	
1007  	// read all sites, sort, and compute xmin, xmax, ymin, ymax
1008  	void readsites(FILE *fh, int maxn)
1009  	{
1010  	    int i, n=0;
1011  	    char line[40];
1012  	    double x, y;
1013  	
1014  	    nsites = 0;
1015  	    sites = (Site *) myalloc(ALLOCCHUNK * sizeof(Site));
1016  	
1017  	    while(fgets(line,sizeof(line),fh) != NULL && n++ < maxn) {
1018  	        if(sscanf(line,"%lg %lg", &x, &y) != 2)
1019  	            continue;
1020  	        sites[nsites].coord.x = x;
1021  	        sites[nsites].coord.y = y;
1022  	        sites[nsites].sitenbr = nsites;
1023  	        sites[nsites++].refcnt = 0;
1024  	        if (nsites % ALLOCCHUNK == 0)
1025  	            sites = (Site *) realloc(sites, (nsites + ALLOCCHUNK) * sizeof(Site));
1026  	    }
1027  	
1028  	    qsort((void *) sites, nsites, sizeof(Site), scomp);
1029  	    xmin = sites[0].coord.x;
1030  	    xmax = sites[0].coord.x;
1031  	    for (i = 1; i < nsites; ++i) {
1032  	        if (sites[i].coord.x < xmin) {
1033  	            xmin = sites[i].coord.x;
1034  	        }
1035  	        if (sites[i].coord.x > xmax) {
1036  	            xmax = sites[i].coord.x;
1037  	        }
1038  	    }
1039  	    ymin = sites[0].coord.y;
1040  	    ymax = sites[nsites - 1].coord.y;
1041  	}
1042  	
