#define  DECNUMDIGITS 5010
#include <decNumber.h>
#include <hpgcc49.h>
#include <hpgraphics.h>

#ifndef HPAPINE
extern unsigned int __heap_ptr,_heap_base_addr;
int freemem();
#endif

int decNumberIsPrec(decNumber, decNumber, decNumber, decContext);
int decNumberIsLess(decNumber, decNumber, decContext);

int main(void) {
	decNumber a, b, A, B, s, k, x, y, t;
	decNumber temp1, temp2, temp3, one, two, four, ten, prec;
	decNumber z;
	decContext set;
	FILE *fp;
	char out[DECNUMDIGITS+1];
	int zz, kk=0;
	int start, end;
#ifndef HPAPINE
	int mem;

	mem = freemem();
#endif

	if((fp = fopen("piagm.log","w")) == NULL) {
		char errormsg[30];

		sprintf(errormsg,"Cannot open: %s","piagm.log");

		sat_stack_push_string(errormsg);
		sat_push_real(1);
		return(0);
	}

	sys_slowOff();
	clear_screen();

	printf("PI AGM - Hold down ENTER to exit\n\n");

	decContextDefault(&set, DEC_INIT_BASE);
  	set.traps = 0;
	set.digits = DECNUMDIGITS;

	decNumberFromString(&a, "1", &set);
	decNumberFromString(&A, "1", &set);
	decNumberFromString(&B, "0.5", &set);
	decNumberFromString(&s, "0.5", &set);
	decNumberFromString(&k, "0", &set);
	decNumberFromString(&x, "0", &set);
	decNumberFromString(&y, "1", &set);

	decNumberFromString(&one,   "1", &set);
	decNumberFromString(&two,   "2", &set);
	decNumberFromString(&four,  "4", &set);
	decNumberFromString(&ten,  "10", &set);

	decNumberFromString(&prec, "1E-5000", &set);

	start = sys_RTC_seconds();
	while(!decNumberIsPrec(x,y,prec,set)) {
		int start, end;

		start = sys_RTC_seconds();

		// k = k + 1
		decNumberAdd(&k, &k, &one, &set);

		// status iterations
		kk++;
		printf("Iteration: %2d ",kk);
		fprintf(fp,"Iteration: %2d ",kk);
		hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_BLACK);

		// x = y
		decNumberCopy(&x, &y);

		// t = (A+B)/4.0
		decNumberAdd(&temp1, &A, &B, &set);
		decNumberDivide(&t, &temp1, &four, &set);

		// b = sqrt(B)
		decNumberSquareRoot(&b, &B, &set);

		// a = (a+b)/2.0
		decNumberAdd(&temp1, &a, &b, &set);
		decNumberDivide(&a, &temp1, &two, &set);

		// A = a^2.0
		decNumberMultiply(&A, &a, &a, &set);

		// B = (A-t)*2.0
		decNumberSubtract(&temp1, &A, &t, &set);
		decNumberMultiply(&B, &temp1, &two, &set);

		// s = s + (B-A)*2^k
		decNumberSubtract(&temp1, &B, &A, &set);
		decNumberPower(&temp2, &two, &k, &set);
		decNumberMultiply(&temp3, &temp1, &temp2, &set);
		decNumberAdd(&s, &s, &temp3, &set);

		// y = (a+b)^2.0/(2.0 * s)
		decNumberAdd(&temp1, &a, &b, &set);
		decNumberMultiply(&temp2, &temp1, &temp1, &set);
		decNumberMultiply(&temp3, &two, &s, &set);
		decNumberDivide(&y, &temp2, &temp3, &set);

		end = sys_RTC_seconds();

		// status digits
		decNumberSubtract(&z, &y, &x, &set);
		decNumberAbs(&z, &z, &set);
		zz=0;
		while(decNumberIsLess(z,one,set)) {
			decNumberMultiply(&z, &z, &ten, &set);
			zz++;
		}
		printf("Digits: %4d ",zz);
		fprintf(fp,"Digits: %4d ",zz);

		printf("Tm: %2d",end-start);
		fprintf(fp,"Tm: %2d\n",end-start);

		if(keyb_isAnyKeyPressed())
			break;
	}
	end = sys_RTC_seconds();

	printf("\n");
	fprintf(fp,"\n");
	printf("   time(s): %d\n",end-start);
	fprintf(fp,"   time(s): %d\n",end-start);
#ifndef HPAPINE
	printf("bytes used: %d",mem-freemem());
	fprintf(fp,"bytes used: %d\n",mem-freemem());
#endif
	fclose(fp);

	decNumberToString(&x, out);
	if((fp = fopen("piagm.txt","w")) == NULL) {
		char errormsg[30];

		sprintf(errormsg,"Cannot open: %s","piagm.txt");

		sat_stack_push_string(errormsg);
		sat_push_real(1);
		return(0);
	}
	fprintf(fp,"%s\n",out);
	fclose(fp);

	hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_WHITE);
	sat_push_real(0);
	beep();
	sys_slowOn();
	WAIT_CANCEL;
	return(0);
}

int decNumberIsPrec(decNumber a, decNumber b, decNumber c, decContext ctx)
{
	decNumber temp1, temp2;
	decNumber cmp;

	decNumberSubtract(&temp1, &a, &b, &ctx);
	decNumberAbs(&temp2, &temp1, &ctx);
	decNumberCompare(&cmp, &temp2, &c, &ctx);
	return(decNumberIsNegative(&cmp));
}

int decNumberIsLess(decNumber a, decNumber b, decContext ctx)
{
	decNumber cmp;
    
	decNumberCompare(&cmp, &a, &b, &ctx);
	return(decNumberIsNegative(&cmp));
}

#ifndef HPAPINE
int freemem()
{
	register unsigned int stack_ptr asm ("sp");
	unsigned int base;

	base=(__heap_ptr == 0)? _heap_base_addr:__heap_ptr;

	return stack_ptr-base;
}
#endif
