// PI.C: calculates pi to MAXDIGITS places.  J.W. Stumpel, May 1991
// HPGCC 50g Version
// Any atan Formula Version

#include <hpgcc49.h>
#include <hpgraphics.h>

FILE *fp;
#ifndef HPAPINE
extern unsigned int __heap_ptr,_heap_base_addr;
#endif

typedef struct tAtans {
	signed int c;
	unsigned int a;
} Atans;

int MAXSIZE;
int MAXDIGITS;

/*
   MAXSIZE is the accuracy of the calculation in 32-bit words.
   MAXDIGITS is the number of decimals that will be printed.
   MAXSIZE should be a little more than MAXDIGITS divided by the
   base-10 logarithm of 2 to the 32th power, or a little bit more 
   than 0.1038 times MAXDIGITS. For an explanation of this fact,
   see, e.g., "The Mathematical Theory of Communication" by Claude Shannon
*/

typedef union {
	unsigned long long L;
	struct {
		unsigned int lo;
		unsigned int hi;
	} w;
} dword;

typedef unsigned int *bignum;

/*
   This program represents "big numbers" (numbers with many digits) as
   arrays of unsigned ints. The first element (with index zero)
   in such an array represents the integer part of the number, the other
   elements represent the part after the decimal point.
*/

// function prototypes
void copybig(bignum dest, bignum source);
void multbig(bignum number, unsigned int x);
int printbig(bignum number);
int divbig(bignum number, unsigned int x);
void addbig(bignum dest, bignum source);
void subbig(bignum dest, bignum source);
void setbig(bignum number, int intpart, unsigned int decpart);
int atanbig(bignum A, unsigned int x);
int make_pi(int ,Atans *);

// Copies a big number from source to dest
void copybig(bignum dest, bignum source)
{
	int j;

	for (j = 0; j < MAXSIZE; j++)
		dest[j] = source[j];
}

// multiplies a big number by an unsigned integer
void multbig(bignum number, unsigned int x)
{
	int j = MAXSIZE - 1;
	dword result;
	unsigned int remember = 0;

	while (number[j] == 0 && j >= 0)
		j--;
	while (j >= 0) {
		result.L = (long long) number[j] * x + remember;
		number[j] = result.w.lo;
		remember = result.w.hi;
		j--;
	}
}

/*
  prints a big number as a decimal float number, by repeatedly multiplying
  by 100000 and printing the integer part
*/
int printbig(bignum number)
{
	bignum num;
	int count = 0, ndigits = 0;

	if((num = malloc(MAXSIZE*sizeof(int))) == NULL)
		return(1);

	copybig(num, number);
	fprintf(fp,"%d.\n", num[0]);
	while (ndigits < MAXDIGITS) {
		num[0] = 0;
		multbig(num, 100000);
		fprintf(fp,"%05d", num[0]);
		ndigits += 5;
		count++;
		if (count == 6) {
			fprintf(fp,"\n");
			count = 0;
		}
	}

	return(0);
}

/*
  divides a big number by an unsigned integer, returns zero if result zero,
  otherwise returns 1
*/
int divbig(bignum number, unsigned int x)
{
	dword result;
	int j = 0;
	unsigned int rest = 0;

	while (number[j] == 0 && j < MAXSIZE)
		j++;
	if (j == MAXSIZE)
		return (0);
	while (j < MAXSIZE) {
		result.w.lo = number[j];
		result.w.hi = rest;
		number[j] = result.L / x;
		rest = result.L % x;
		j++;
	}
	return (1);
}

/*
  adds "source" to "dest". "addbig" and "subbig" assume that "dest" and
  "source" are both positive and that the result of a subtraction will also
  always be positive.
*/
void addbig(bignum dest, bignum source)
{
	int j = MAXSIZE - 1;
	dword result;
	unsigned int remember = 0;

	while (source[j] == 0 && j >= 0)
		j--;
	while (j >= 0) {
		result.L = (long long) dest[j] + source[j] + remember;
		dest[j] = result.w.lo;
		remember = result.w.hi;
		j--;
	}
}

// subtracts source from dest
void subbig(bignum dest, bignum source)
{
	int j = MAXSIZE - 1;
	unsigned int borrow = 0;
	dword result;

	while (source[j] == 0 && j >= 0)
		j--;
	while (j >= 0) {
		result.L = (long long) dest[j] - source[j] + (int) borrow;
		dest[j] = result.w.lo;
		borrow = result.w.hi;
		j--;
	}
}

// fill a big number with some numbers
void setbig(bignum number, int intpart, unsigned int decpart)
{
	int j;

	number[0] = intpart;
	for (j = 1; j < MAXSIZE; j++)
		number[j] = decpart;
}

/*
  Calculates the arc tangent of (1/x) by series summation.
  Result is stored in big number A
*/
int atanbig(bignum A, unsigned int x)
{
	bignum X, Y;
	unsigned int n = 1;
	int xx = 1;

	if((X = malloc(MAXSIZE*sizeof(int))) == NULL)
		return(1);
	if((Y = malloc(MAXSIZE*sizeof(int))) == NULL)
		return(1);

	setbig(X, 1, 0);
	divbig(X, x);
	copybig(A, X);
	if(x < 65536) {
		x *= x;
		xx = 0;
	}
	while (1) {
		n += 2;
		divbig(X, x);
		if(xx)
			divbig(X, x);
		copybig(Y, X);
		if (!divbig(Y, n))
			break;
		if (n & 2)
			subbig(A, Y);
		else
			addbig(A, Y);
#ifndef HPAPINE
		if(keyb_isAnyKeyPressed())
			beep();
#endif
	}

	free(X);
	free(Y);

	return(0);
}

// Calculates and prints pi using any atan formula
int make_pi(int i,Atans *atans)
{
	bignum A, B;
	int start, end;
	int j, k, p;
	signed int m;
	char buf[10];

	if((A = malloc(MAXSIZE*sizeof(int))) == NULL)
		return(1);
	if((B = malloc(MAXSIZE*sizeof(int))) == NULL)
		return(1);

	for(j=0;j<i;j++) {
		printf("computing atan(1/%d)",atans[j].a);
		itoa(atans[j].a,buf,9);
		for(k=0;k<7-strlen(buf);k++)
			printf(".");
		start = sys_RTC_seconds();
		if(atanbig(A, atans[j].a) != 0)
			return(1);
		end = sys_RTC_seconds();
		printf("%5d s\n",end-start);

		m = atans[j].c/abs(atans[j].c);
		p = abs(atans[j].c);

		multbig(A, p);

		if(j == 0) {
			copybig(B,A);
			continue;
		}

		if(m == -1)
			subbig(B, A);
		else
			addbig(B, A);
	}

	free(A);
	if(printbig(B) != 0)
		return(1);
	printf("\n");

	return(0);
}

#ifndef HPAPINE
int freemem()
{
	register unsigned int stack_ptr asm ("sp");
	unsigned int base;

	base=(__heap_ptr == 0)? _heap_base_addr:__heap_ptr;

	return stack_ptr-base;
}
#endif

int main(void)
{
	int start, end, speed;
	char *filename = "piatan.txt";
	char *s, c[100];
	int k=0, i=0, x, y;
	Atans *atans = NULL;
	double pi_index = 0;

#ifndef HPAPINE
	int mem;

	mem = freemem();
#endif
	speed = sat_pop_real();
	if(speed != 0 && speed != 1) {
		sat_push_zint_llong(speed);
		sat_stack_push_string("speed must be 0 or 1");
		sat_push_real(1);
		return(0);
	}

	MAXDIGITS = sat_pop_real();
	if(MAXDIGITS < 1) {
		sat_push_zint_llong(MAXDIGITS);
		sat_stack_push_string("PI digits must be > 0");
		sat_push_real(1);
		return(0);
	}
	MAXSIZE = MAXDIGITS * 0.1038 + 10;

	if((fp = fopen(filename,"w")) == NULL) {
		char errormsg[30];

		sprintf(errormsg,"Cannot open: %s",filename);

		sat_stack_push_string(errormsg);
		sat_push_real(1);
		return(0);
	}

	s = sat_stack_pop_string_alloc();

	while(*s++ != '\0') {
		if(*s == '{') {
			s++;
			k=0;
			continue;
		}
		if(*s == '}') {
			c[--k] = '\0';
			if(k > 0) {
				sscanf(c,"%d %d",&x,&y);
				atans = (Atans *) realloc(atans, (i+1) * sizeof(Atans));
				atans[i].c = x;
				atans[i].a = y;
				i++;
			}
			s+=3;
			k=0;
			continue;
		}
		c[k++] = *s;
	}

	if(speed == 0) 
		sys_slowOn();
	else
		sys_slowOff();

	clear_screen();

	printf("pi=");
	for(k=0;k<i;k++) {
		int m = atans[k].c/abs(atans[k].c);

		if(k > 0)
			if(m == 1)
				printf("+");
		printf("%d,%d",atans[k].c,atans[k].a);
		pi_index += 1/log10(atans[k].a);
	}
	printf("\n\n");

	hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_BLACK);
	start = sys_RTC_seconds();
	if(make_pi(i,atans) != 0) {
		sys_slowOn();
		sat_stack_push_string("Failed to allocate memory.");
		sat_push_real(1);
		return(0);
	}
	end = sys_RTC_seconds();

	hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_WHITE);

	fclose(fp);

	if(speed == 0)
		printf("      speed: normal\n");
	else
		printf("      speed: fast\n");
	printf("   pi index: %.2f\n",pi_index);
	printf("    time(s): %d\n",end-start);
	printf("bytes alloc: %d\n",4*MAXSIZE*(int)sizeof(int));
#ifndef HPAPINE
	printf(" bytes used: %d\n",mem-freemem());
#endif

	sat_push_real(0);
	beep();
	sys_slowOn();
	WAIT_CANCEL;
	return(0);
}
