1     	// PI.C: calculates pi to MAXDIGITS places.  J.W. Stumpel, May 1991
2     	// HPGCC 50g Version
3     	// Any atan Formula Version
4     	
5     	#include <hpgcc49.h>
6     	#include <hpgraphics.h>
7     	
8     	FILE *fp;
9     	#ifndef HPAPINE
10    	extern unsigned int __heap_ptr,_heap_base_addr;
11    	#endif
12    	
13    	typedef struct tAtans {
14    	    signed int c;
15    	    unsigned int a;
16    	} Atans;
17    	
18    	int MAXSIZE;
19    	int MAXDIGITS;
20    	
21    	/*
22    	   MAXSIZE is the accuracy of the calculation in 32-bit words.
23    	   MAXDIGITS is the number of decimals that will be printed.
24    	   MAXSIZE should be a little more than MAXDIGITS divided by the
25    	   base-10 logarithm of 2 to the 32th power, or a little bit more 
26    	   than 0.1038 times MAXDIGITS. For an explanation of this fact,
27    	   see, e.g., "The Mathematical Theory of Communication" by Claude Shannon
28    	*/
29    	
30    	typedef union {
31    	    unsigned long long L;
32    	    struct {
33    	        unsigned int lo;
34    	        unsigned int hi;
35    	    } w;
36    	} dword;
37    	
38    	typedef unsigned int *bignum;
39    	
40    	/*
41    	   This program represents "big numbers" (numbers with many digits) as
42    	   arrays of unsigned ints. The first element (with index zero)
43    	   in such an array represents the integer part of the number, the other
44    	   elements represent the part after the decimal point.
45    	*/
46    	
47    	// function prototypes
48    	void copybig(bignum dest, bignum source);
49    	void multbig(bignum number, unsigned int x);
50    	int printbig(bignum number);
51    	int divbig(bignum number, unsigned int x);
52    	void addbig(bignum dest, bignum source);
53    	void subbig(bignum dest, bignum source);
54    	void setbig(bignum number, int intpart, unsigned int decpart);
55    	int atanbig(bignum A, unsigned int x);
56    	int make_pi(int ,Atans *);
57    	
58    	// Copies a big number from source to dest
59    	void copybig(bignum dest, bignum source)
60    	{
61    	    int j;
62    	
63    	    for (j = 0; j < MAXSIZE; j++)
64    	        dest[j] = source[j];
65    	}
66    	
67    	// multiplies a big number by an unsigned integer
68    	void multbig(bignum number, unsigned int x)
69    	{
70    	    int j = MAXSIZE - 1;
71    	    dword result;
72    	    unsigned int remember = 0;
73    	
74    	    while (number[j] == 0 && j >= 0)
75    	        j--;
76    	    while (j >= 0) {
77    	        result.L = (long long) number[j] * x + remember;
78    	        number[j] = result.w.lo;
79    	        remember = result.w.hi;
80    	        j--;
81    	    }
82    	}
83    	
84    	/*
85    	  prints a big number as a decimal float number, by repeatedly multiplying
86    	  by 100000 and printing the integer part
87    	*/
88    	int printbig(bignum number)
89    	{
90    	    bignum num;
91    	    int count = 0, ndigits = 0;
92    	
93    	    if((num = malloc(MAXSIZE*sizeof(int))) == NULL)
94    	        return(1);
95    	
96    	    copybig(num, number);
97    	    fprintf(fp,"%d.\n", num[0]);
98    	    while (ndigits < MAXDIGITS) {
99    	        num[0] = 0;
100   	        multbig(num, 100000);
101   	        fprintf(fp,"%05d", num[0]);
102   	        ndigits += 5;
103   	        count++;
104   	        if (count == 6) {
105   	            fprintf(fp,"\n");
106   	            count = 0;
107   	        }
108   	    }
109   	
110   	    return(0);
111   	}
112   	
113   	/*
114   	  divides a big number by an unsigned integer, returns zero if result zero,
115   	  otherwise returns 1
116   	*/
117   	int divbig(bignum number, unsigned int x)
118   	{
119   	    dword result;
120   	    int j = 0;
121   	    unsigned int rest = 0;
122   	
123   	    while (number[j] == 0 && j < MAXSIZE)
124   	        j++;
125   	    if (j == MAXSIZE)
126   	        return (0);
127   	    while (j < MAXSIZE) {
128   	        result.w.lo = number[j];
129   	        result.w.hi = rest;
130   	        number[j] = result.L / x;
131   	        rest = result.L % x;
132   	        j++;
133   	    }
134   	    return (1);
135   	}
136   	
137   	/*
138   	  adds "source" to "dest". "addbig" and "subbig" assume that "dest" and
139   	  "source" are both positive and that the result of a subtraction will also
140   	  always be positive.
141   	*/
142   	void addbig(bignum dest, bignum source)
143   	{
144   	    int j = MAXSIZE - 1;
145   	    dword result;
146   	    unsigned int remember = 0;
147   	
148   	    while (source[j] == 0 && j >= 0)
149   	        j--;
150   	    while (j >= 0) {
151   	        result.L = (long long) dest[j] + source[j] + remember;
152   	        dest[j] = result.w.lo;
153   	        remember = result.w.hi;
154   	        j--;
155   	    }
156   	}
157   	
158   	// subtracts source from dest
159   	void subbig(bignum dest, bignum source)
160   	{
161   	    int j = MAXSIZE - 1;
162   	    unsigned int borrow = 0;
163   	    dword result;
164   	
165   	    while (source[j] == 0 && j >= 0)
166   	        j--;
167   	    while (j >= 0) {
168   	        result.L = (long long) dest[j] - source[j] + (int) borrow;
169   	        dest[j] = result.w.lo;
170   	        borrow = result.w.hi;
171   	        j--;
172   	    }
173   	}
174   	
175   	// fill a big number with some numbers
176   	void setbig(bignum number, int intpart, unsigned int decpart)
177   	{
178   	    int j;
179   	
180   	    number[0] = intpart;
181   	    for (j = 1; j < MAXSIZE; j++)
182   	        number[j] = decpart;
183   	}
184   	
185   	/*
186   	  Calculates the arc tangent of (1/x) by series summation.
187   	  Result is stored in big number A
188   	*/
189   	int atanbig(bignum A, unsigned int x)
190   	{
191   	    bignum X, Y;
192   	    unsigned int n = 1;
193   	    int xx = 1;
194   	
195   	    if((X = malloc(MAXSIZE*sizeof(int))) == NULL)
196   	        return(1);
197   	    if((Y = malloc(MAXSIZE*sizeof(int))) == NULL)
198   	        return(1);
199   	
200   	    setbig(X, 1, 0);
201   	    divbig(X, x);
202   	    copybig(A, X);
203   	    if(x < 65536) {
204   	        x *= x;
205   	        xx = 0;
206   	    }
207   	    while (1) {
208   	        n += 2;
209   	        divbig(X, x);
210   	        if(xx)
211   	            divbig(X, x);
212   	        copybig(Y, X);
213   	        if (!divbig(Y, n))
214   	            break;
215   	        if (n & 2)
216   	            subbig(A, Y);
217   	        else
218   	            addbig(A, Y);
219   	#ifndef HPAPINE
220   	        if(keyb_isAnyKeyPressed())
221   	            beep();
222   	#endif
223   	    }
224   	
225   	    free(X);
226   	    free(Y);
227   	
228   	    return(0);
229   	}
230   	
231   	// Calculates and prints pi using any atan formula
232   	int make_pi(int i,Atans *atans)
233   	{
234   	    bignum A, B;
235   	    int start, end;
236   	    int j, k, p;
237   	    signed int m;
238   	    char buf[10];
239   	
240   	    if((A = malloc(MAXSIZE*sizeof(int))) == NULL)
241   	        return(1);
242   	    if((B = malloc(MAXSIZE*sizeof(int))) == NULL)
243   	        return(1);
244   	
245   	    for(j=0;j<i;j++) {
246   	        printf("computing atan(1/%d)",atans[j].a);
247   	        itoa(atans[j].a,buf,9);
248   	        for(k=0;k<7-strlen(buf);k++)
249   	            printf(".");
250   	        start = sys_RTC_seconds();
251   	        if(atanbig(A, atans[j].a) != 0)
252   	            return(1);
253   	        end = sys_RTC_seconds();
254   	        printf("%5d s\n",end-start);
255   	
256   	        m = atans[j].c/abs(atans[j].c);
257   	        p = abs(atans[j].c);
258   	
259   	        multbig(A, p);
260   	
261   	        if(j == 0) {
262   	            copybig(B,A);
263   	            continue;
264   	        }
265   	
266   	        if(m == -1)
267   	            subbig(B, A);
268   	        else
269   	            addbig(B, A);
270   	    }
271   	
272   	    free(A);
273   	    if(printbig(B) != 0)
274   	        return(1);
275   	    printf("\n");
276   	
277   	    return(0);
278   	}
279   	
280   	#ifndef HPAPINE
281   	int freemem()
282   	{
283   	    register unsigned int stack_ptr asm ("sp");
284   	    unsigned int base;
285   	
286   	    base=(__heap_ptr == 0)? _heap_base_addr:__heap_ptr;
287   	
288   	    return stack_ptr-base;
289   	}
290   	#endif
291   	
292   	int main(void)
293   	{
294   	    int start, end, speed;
295   	    char *filename = "piatan.txt";
296   	    char *s, c[100];
297   	    int k=0, i=0, x, y;
298   	    Atans *atans = NULL;
299   	    double pi_index = 0;
300   	
301   	#ifndef HPAPINE
302   	    int mem;
303   	
304   	    mem = freemem();
305   	#endif
306   	    speed = sat_pop_real();
307   	    if(speed != 0 && speed != 1) {
308   	        sat_push_zint_llong(speed);
309   	        sat_stack_push_string("speed must be 0 or 1");
310   	        sat_push_real(1);
311   	        return(0);
312   	    }
313   	
314   	    MAXDIGITS = sat_pop_real();
315   	    if(MAXDIGITS < 1) {
316   	        sat_push_zint_llong(MAXDIGITS);
317   	        sat_stack_push_string("PI digits must be > 0");
318   	        sat_push_real(1);
319   	        return(0);
320   	    }
321   	    MAXSIZE = MAXDIGITS * 0.1038 + 10;
322   	
323   	    if((fp = fopen(filename,"w")) == NULL) {
324   	        char errormsg[30];
325   	
326   	        sprintf(errormsg,"Cannot open: %s",filename);
327   	
328   	        sat_stack_push_string(errormsg);
329   	        sat_push_real(1);
330   	        return(0);
331   	    }
332   	
333   	    s = sat_stack_pop_string_alloc();
334   	
335   	    while(*s++ != '\0') {
336   	        if(*s == '{') {
337   	            s++;
338   	            k=0;
339   	            continue;
340   	        }
341   	        if(*s == '}') {
342   	            c[--k] = '\0';
343   	            if(k > 0) {
344   	                sscanf(c,"%d %d",&x,&y);
345   	                atans = (Atans *) realloc(atans, (i+1) * sizeof(Atans));
346   	                atans[i].c = x;
347   	                atans[i].a = y;
348   	                i++;
349   	            }
350   	            s+=3;
351   	            k=0;
352   	            continue;
353   	        }
354   	        c[k++] = *s;
355   	    }
356   	
357   	    if(speed == 0) 
358   	        sys_slowOn();
359   	    else
360   	        sys_slowOff();
361   	
362   	    clear_screen();
363   	
364   	    printf("pi=");
365   	    for(k=0;k<i;k++) {
366   	        int m = atans[k].c/abs(atans[k].c);
367   	
368   	        if(k > 0)
369   	            if(m == 1)
370   	                printf("+");
371   	        printf("%d,%d",atans[k].c,atans[k].a);
372   	        pi_index += 1/log10(atans[k].a);
373   	    }
374   	    printf("\n\n");
375   	
376   	    hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_BLACK);
377   	    start = sys_RTC_seconds();
378   	    if(make_pi(i,atans) != 0) {
379   	        sys_slowOn();
380   	        sat_stack_push_string("Failed to allocate memory.");
381   	        sat_push_real(1);
382   	        return(0);
383   	    }
384   	    end = sys_RTC_seconds();
385   	    hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_WHITE);
386   	
387   	    fclose(fp);
388   	
389   	    if(speed == 0)
390   	        printf("      speed: normal\n");
391   	    else
392   	        printf("      speed: fast\n");
393   	    printf("   pi index: %.2f\n",pi_index);
394   	    printf("    time(s): %d\n",end-start);
395   	    printf("bytes alloc: %d\n",4*MAXSIZE*(int)sizeof(int));
396   	#ifndef HPAPINE
397   	    printf(" bytes used: %d\n",mem-freemem());
398   	#endif
399   	
400   	    sat_push_real(0);
401   	    beep();
402   	    sys_slowOn();
403   	    WAIT_CANCEL;
404   	    return(0);
405   	}
