1     	/* ------------------------------------------------------------------ */
2     	/* Decimal Number Library Demonstration program                       */
3     	/* ------------------------------------------------------------------ */
4     	/* Copyright (c) Ingo Blank 2006  All rights reserved.                */
5     	/* ----------------------------------------------------------------+- */
6     	/*                                                 right margin -->|  */
7     	
8     	/*
9     	 * chud_pi.c
10    	 * Arbitrary digits pi appoximation using Chudnvsky's formula. 
11    	 */
12    	
13    	
14    	// $Id: chud_pi.c 317 2006-10-28 15:20:48Z ingo $
15    	
16    	#define HP 1
17    	
18    	#define DECNUMDIGITS 5000
19    	
20    	#include "decNumber.h"            // base number library
21    	#include "decNumberLocal.h"
22    	
23    	#ifdef PC
24    	#include <stdio.h>                // for printf
25    	#include <time.h>
26    	#include <stdlib.h>
27    	#include <string.h>
28    	#include <math.h>
29    	#include <signal.h>                // signal handling
30    	#include <setjmp.h>                // setjmp/longjmp
31    	#else
32    	
33    	#define TINY_PRINTF 1
34    	#include <hpgcc49.h>
35    	#endif
36    	
37    	#ifndef max
38    	#define max(a,b) (a) > (b) ? (a) : (b)
39    	#endif
40    	
41    	#ifndef min
42    	#define min(a,b) (a) < (b) ? (a) : (b)
43    	#endif
44    	
45    	#define decNumberSize(digits) (2*sizeof(int32_t)+sizeof(uint8_t) + D2U((digits)) * sizeof(decNumberUnit))
46    	
47    	#ifdef PC
48    	jmp_buf preserve;                // stack snapshot
49    	
50    	void signalHandler(int sig)
51    	{
52    	    signal(SIGFPE, signalHandler);    // re-enable
53    	    longjmp(preserve, sig);        // branch to preserved point
54    	}
55    	
56    	#endif
57    	
58    	
59    	void free2(void *a, void *b)
60    	{
61    	    free(a);
62    	    free(b);
63    	}
64    	
65    	void free3(void *a, void *b, void *c)
66    	{
67    	    free2(a, b);
68    	    free(c);
69    	}
70    	
71    	void free4(void *a, void *b, void *c, void *d)
72    	{
73    	    free2(a, b);
74    	    free2(c, d);
75    	}
76    	
77    	void free5(void *a, void *b, void *c, void *d, void *e)
78    	{
79    	    free3(a, b, c);
80    	    free2(d, e);
81    	}
82    	
83    	void free6(void *a, void *b, void *c, void *d, void *e, void *f)
84    	{
85    	    free3(a, b, c);
86    	    free3(d, e, f);
87    	}
88    	
89    	int sq(int x)
90    	{
91    	    return x * x;
92    	}
93    	
94    	int ipow10(int n)
95    	{
96    	    if (n == 0)
97    	        return 1;
98    	    if (n & 1)
99    	        return 10 * ipow10(n - 1);
100   	
101   	    return sq(ipow10(n / 2));
102   	}
103   	
104   	
105   	
106   	int _ilog10(int n)
107   	{
108   	    int p = 10, t = 1;
109   	    if (n < 0)
110   	        return _ilog10(-n);
111   	    for (;;) {
112   	        if (n < p)
113   	            return t;
114   	        p *= 10;
115   	        t++;
116   	    }
117   	}
118   	
119   	
120   	
121   	int decNumberIsOdd(decNumber * a)
122   	{
123   	    return *((decNumberUnit *) (a->lsu)) & 1;
124   	}
125   	
126   	int decNumberIsEven(decNumber * a)
127   	{
128   	    return !decNumberIsOdd(a);
129   	}
130   	
131   	int decNumberIsLess(decNumber * a, decNumber * b, decContext * ctx)
132   	{
133   	    decNumber cmp;
134   	
135   	    decNumberCompare(&cmp, a, b, ctx);
136   	    return decNumberIsNegative(&cmp);
137   	}
138   	
139   	int decNumberIsGreater(decNumber * a, decNumber * b, decContext * ctx)
140   	{
141   	    decNumber cmp;
142   	
143   	    decNumberCompare(&cmp, a, b, ctx);
144   	    return !(decNumberIsNegative(&cmp) || decNumberIsZero(&cmp));
145   	}
146   	
147   	
148   	int decNumberIsEqual(decNumber * a, decNumber * b, decContext * ctx)
149   	{
150   	    decNumber cmp;
151   	
152   	    decNumberCompare(&cmp, a, b, ctx);
153   	    return decNumberIsZero(&cmp);
154   	}
155   	
156   	
157   	
158   	
159   	decNumber *decNumberAlloc(int digits)
160   	{
161   	    return (decNumber *) calloc(1, decNumberSize(digits));
162   	}
163   	
164   	decNumber *decNumberSaveCopy(decNumber * a, decNumber * b)
165   	{
166   	    decNumber *z;
167   	
168   	    if (b->digits > a->digits) {
169   	        z = decNumberAlloc(b->digits);
170   	        decNumberCopy(z, a);
171   	        free(a);
172   	        a = z;
173   	    }
174   	
175   	    return decNumberCopy(a, b);
176   	}
177   	
178   	
179   	decNumber *decNumberAllocAssign(decNumber * rhs)
180   	{
181   	    decNumber *lhs;
182   	
183   	    lhs = decNumberAlloc(rhs->digits);
184   	    return decNumberCopy(lhs, rhs);
185   	}
186   	
187   	decNumber *decNumberAllocAssignConst(char *rhs, decContext * ctx)
188   	{
189   	    decNumber *lhs;
190   	
191   	    lhs = decNumberAlloc(strlen(rhs));
192   	    decNumberFromString(lhs, rhs, ctx);
193   	
194   	    return lhs;
195   	}
196   	
197   	
198   	decNumber *decNumberFromInteger(int n, decContext * ctx)
199   	{
200   	    char zbuff[10];
201   	
202   	#ifdef PC
203   	    sprintf(zbuff, "%d", n);
204   	#else
205   	    itoa(n, zbuff, 10);
206   	#endif
207   	
208   	    return decNumberAllocAssignConst(zbuff, ctx);
209   	}
210   	
211   	
212   	
213   	decNumber *decNumberAddAlloc(decNumber * a, decNumber * b,
214   	                             decContext * ctx)
215   	{
216   	    decContext save = *ctx;
217   	    decNumber *z;
218   	
219   	    ctx->digits = max(a->digits, b->digits) + 1;
220   	    z = decNumberAlloc(ctx->digits);
221   	    decNumberAdd(z, a, b, ctx);
222   	
223   	    *ctx = save;
224   	
225   	    return z;
226   	}
227   	
228   	decNumber *decNumberSubAlloc(decNumber * a, decNumber * b,
229   	                             decContext * ctx)
230   	{
231   	    decContext save = *ctx;
232   	    decNumber *z;
233   	
234   	    ctx->digits = max(a->digits, b->digits) + 1;
235   	    z = decNumberAlloc(ctx->digits);
236   	    decNumberSubtract(z, a, b, ctx);
237   	
238   	    *ctx = save;
239   	
240   	    return z;
241   	}
242   	
243   	
244   	
245   	decNumber *decNumberMultiplyAlloc(decNumber * a, decNumber * b,
246   	                                  decContext * ctx)
247   	{
248   	    decContext save = *ctx;
249   	    decNumber *z;
250   	
251   	    ctx->digits = a->digits + b->digits;
252   	    z = decNumberAlloc(ctx->digits);
253   	    decNumberMultiply(z, a, b, ctx);
254   	
255   	    *ctx = save;
256   	
257   	    return z;
258   	}
259   	
260   	
261   	
262   	decNumber *decNumberModuloAlloc(decNumber * a, decNumber * b,
263   	                                decContext * ctx)
264   	{
265   	
266   	    decContext save = *ctx;
267   	    decNumber *z;
268   	
269   	    ctx->digits = a->digits;
270   	    z = decNumberAlloc(ctx->digits);
271   	    decNumberRemainder(z, a, b, ctx);
272   	
273   	    *ctx = save;
274   	
275   	    return z;
276   	}
277   	
278   	decNumber *decNumberDivideIntegerAlloc(decNumber * a, decNumber * b,
279   	                                       decContext * ctx)
280   	{
281   	
282   	    decContext save = *ctx;
283   	    decNumber *z;
284   	
285   	
286   	    ctx->digits = a->digits;
287   	    z = decNumberAlloc(ctx->digits);
288   	    decNumberDivideInteger(z, a, b, ctx);
289   	
290   	    *ctx = save;
291   	
292   	    return z;
293   	}
294   	
295   	
296   	
297   	
298   	decNumber *decNumberFactorial(int n, decContext * ctx)
299   	{
300   	
301   	    decNumber *z, f, *one;
302   	    decContext save;
303   	
304   	    one = decNumberAlloc(1);
305   	    decNumberFromString(one, "1", ctx);
306   	
307   	    if (n <= 1)
308   	        return one;
309   	
310   	    save = *ctx;
311   	    ctx->digits = (int) (log10((double) n) * n);
312   	    z = decNumberAlloc(ctx->digits);
313   	    decNumberCopy(z, one);
314   	    decNumberCopy(&f, one);
315   	
316   	    for (; n--;) {
317   	        decNumberMultiply(z, z, &f, ctx);
318   	        decNumberAdd(&f, &f, one, &save);
319   	    }
320   	
321   	    free(one);
322   	    *ctx = save;
323   	
324   	    return z;
325   	}
326   	
327   	
328   	decNumber *decNumberPowInteger(decNumber * x, decNumber * y,
329   	                               decContext * ctx)
330   	{
331   	    // TODO: check power of 2 bases
332   	    // ==> 2**3 gives 0 !!!
333   	
334   	    decContext save = *ctx;
335   	    decNumber *z, *s, *t, *u, *ONE, *TWO;
336   	    int minus;
337   	
338   	    if ((minus = decNumberIsNegative(y)))
339   	        decNumberMinus(y, y, ctx);
340   	
341   	    ONE = decNumberAllocAssignConst("1", ctx);
342   	    TWO = decNumberAllocAssignConst("2", ctx);
343   	
344   	
345   	    s = decNumberAllocAssign(ONE);
346   	    t = decNumberAllocAssign(x);
347   	    u = decNumberAllocAssign(y);
348   	
349   	
350   	    while (!decNumberIsZero(u)) {    // while (u != 0)
351   	
352   	        if (decNumberIsOdd(u)) {
353   	            z = decNumberMultiplyAlloc(s, t, ctx);
354   	            free(s);
355   	            s = z;
356   	        }
357   	
358   	        decNumberDivideInteger(u, u, TWO, ctx);
359   	        z = decNumberMultiplyAlloc(t, t, ctx);
360   	        free(t);
361   	        t = z;
362   	
363   	    }
364   	
365   	
366   	    if (minus) {
367   	        ctx->digits = min(s->digits, ctx->digits);
368   	        decNumberDivide(s, ONE, s, ctx);
369   	    }
370   	
371   	    free4(t, u, ONE, TWO);
372   	    *ctx = save;
373   	
374   	    return s;
375   	
376   	}
377   	
378   	
379   	decNumber *decNumberGCD(decNumber * _a, decNumber * _b, decContext * ctx)
380   	{
381   	
382   	    decNumber *a, *b, *c;
383   	
384   	    b = decNumberAllocAssign(_b);
385   	    a = decNumberAllocAssign(_a);
386   	
387   	    while (!decNumberIsZero(b)) {
388   	        c = decNumberModuloAlloc(a, b, ctx);
389   	        a = decNumberSaveCopy(a, b);
390   	        b = decNumberSaveCopy(b, c);
391   	        free(c);
392   	    }
393   	
394   	    free(b);
395   	    return a;
396   	}
397   	
398   	void decNumberPrint(decNumber * num)
399   	{
400   	    char *s = (char *) malloc(num->digits + 10);
401   	
402   	    decNumberToString(num, s);
403   	    printf("%s ", s);
404   	    free(s);
405   	}
406   	
407   	void decNumberPrintLn(decNumber * num)
408   	{
409   	    decNumberPrint(num);
410   	    printf("\n");
411   	}
412   	
413   	void decNumberLabelPrint(const char *txt, decNumber * num)
414   	{
415   	    printf("%s ", txt);
416   	    decNumberPrint(num);
417   	}
418   	
419   	void decNumberLabelPrintLn(const char *txt, decNumber * num)
420   	{
421   	    decNumberLabelPrint(txt, num);
422   	    printf("\n");
423   	}
424   	
425   	#ifndef PC
426   	void decNumberPushString(decNumber * num)
427   	{
428   	    char *s = (char *) sys_chkptr(malloc(num->digits + 10));
429   	
430   	    decNumberToString(num, s);
431   	    sat_stack_push_string(s);
432   	    free(s);
433   	}
434   	#endif
435   	
436   	
437   	
438   	// Globals
439   	
440   	decNumber *k1, *k2, *k3, *k4, *k5, *k6, *K;
441   	
442   	decNumber *P(int n, decContext * ctx)
443   	{
444   	    decNumber *f, *dn, *z, *r, *r1;
445   	
446   	    f = decNumberFactorial(6 * n, ctx);
447   	    dn = decNumberFromInteger(n, ctx);
448   	    z = decNumberMultiplyAlloc(dn, k1, ctx);
449   	    r1 = decNumberAddAlloc(k2, z, ctx);
450   	
451   	    r = decNumberMultiplyAlloc(r1, f, ctx);
452   	    free4(f, dn, z, r1);
453   	
454   	    return r;
455   	}
456   	
457   	
458   	decNumber *Q(int n, decContext * ctx)
459   	{
460   	    decNumber *f, *three, *fp3, *f3, *dn, *kn, *r;
461   	
462   	    f = decNumberFactorial(n, ctx);
463   	    three = decNumberAllocAssignConst("3", ctx);
464   	    if (n == 2)                    // CHEAT!
465   	        fp3 = decNumberAllocAssignConst("8", ctx);
466   	    else
467   	        fp3 = decNumberPowInteger(f, three, ctx);
468   	
469   	    // fp3 = fac(n)**3
470   	
471   	    f3 = decNumberFactorial(3 * n, ctx);
472   	    free(f);
473   	    f = decNumberMultiplyAlloc(fp3, f3, ctx);
474   	    // f = fac(n)**3 * fac(3*n)
475   	
476   	    dn = decNumberFromInteger(n, ctx);
477   	    kn = decNumberPowInteger(K, dn, ctx);
478   	
479   	    r = decNumberMultiplyAlloc(f, kn, ctx);
480   	
481   	    free6(f, three, fp3, f3, dn, kn);
482   	
483   	    return r;
484   	}
485   	
486   	
487   	int
488   	#ifdef PC
489   	main(int argc, char **argv)
490   	#else
491   	main()
492   	#endif
493   	{
494   	    int i, n;
495   	
496   	    decNumber *eight, *z, *p0, *p, *q0, *q, *g1, *t1, *t2, *pi;
497   	    decContext set;
498   	
499   	#ifdef PC
500   	
501   	    {
502   	        int value;
503   	
504   	        signal(SIGFPE, signalHandler);    // set up signal handler
505   	        value = setjmp(preserve);    // preserve and test environment
506   	
507   	        if (value) {            // (non-0 after longjmp)
508   	            set.status &= DEC_Errors;    // keep only errors
509   	            printf("\nSignal trapped [%s].\n",
510   	                   decContextStatusToString(&set));
511   	            return 1;
512   	        }
513   	    }
514   	
515   	#else
516   	    sys_slowOff();
517   	    clear_screen();
518   	
519   	#endif
520   	
521   	    decContextDefault(&set, DEC_INIT_BASE);    // initialize
522   	
523   	#ifndef PC
524   	    set.traps = 0;
525   	#endif
526   	
527   	    // set.traps=0; // no traps, thank you
528   	    set.digits = DECNUMDIGITS;
529   	
530   	    k1 = decNumberAllocAssignConst("545140134", &set);
531   	    k2 = decNumberAllocAssignConst("13591409", &set);
532   	    k3 = decNumberAllocAssignConst("640320", &set);
533   	    k4 = decNumberAllocAssignConst("100100025", &set);
534   	    k5 = decNumberAllocAssignConst("327843840", &set);
535   	    k6 = decNumberAllocAssignConst("53360", &set);
536   	
537   	    eight = decNumberAllocAssignConst("8", &set);
538   	    z = decNumberMultiplyAlloc(k4, k5, &set);
539   	
540   	    K = decNumberMultiplyAlloc(eight, z, &set);
541   	    free(z);
542   	
543   	#ifdef PC
544   	    n = argc > 1 ? atoi(*++argv) : 10;
545   	#else
546   	    n = sat_stack_depth()? (int) sat_pop_real() : 10;
547   	    printf("Chudnovsky pi iteration [%d]\n\n", n);
548   	#endif
549   	
550   	    p0 = P(0, &set);
551   	    q0 = Q(0, &set);
552   	
553   	
554   	
555   	    for (i = 1; i <= n; i++) {
556   	
557   	        printf(".");
558   	
559   	        p = P(i, &set);
560   	        q = Q(i, &set);
561   	
562   	        if (i & 1)
563   	            decNumberMinus(p, p, &set);    // p = -p
564   	
565   	
566   	        g1 = decNumberDivideIntegerAlloc(q, q0, &set);
567   	        t1 = decNumberMultiplyAlloc(p0, g1, &set);
568   	        z = decNumberAddAlloc(t1, p, &set);
569   	        free2(p0, t1);
570   	        p0 = z;
571   	
572   	
573   	        z = decNumberMultiplyAlloc(g1, q0, &set);
574   	        free(q0);
575   	
576   	        q0 = z;
577   	
578   	        free3(p, q, g1);
579   	    }
580   	
581   	
582   	
583   	    set.digits = p0->digits / 2;
584   	    t1 = decNumberAlloc(set.digits);
585   	    decNumberSquareRoot(t1, k3, &set);
586   	
587   	
588   	    t2 = decNumberMultiplyAlloc(k6, t1, &set);
589   	
590   	    free(t1);
591   	    t1 = decNumberMultiplyAlloc(t2, q0, &set);
592   	
593   	    free2(t2, q0);
594   	
595   	    pi = decNumberAlloc(set.digits);
596   	
597   	    decNumberDivide(pi, t1, p0, &set);
598   	
599   	    free2(t1, p0);
600   	
601   	#ifdef PC
602   	    decNumberPrintLn(pi);
603   	#else
604   	
605   	    decNumberPushString(pi);
606   	    //sys_slowOn();
607   	    //WAIT_CANCEL;
608   	#endif
609   	
610   	    return 0;
611   	}
