1     	// PI.C: calculates pi to MAXDIGITS places.  J.W. Stumpel, May 1991
2     	// HPGCC 50g Version
3     	
4     	#include <hpgcc49.h>
5     	#include <hpgraphics.h>
6     	
7     	FILE *fp;
8     	#ifndef HPAPINE
9     	extern unsigned int __heap_ptr,_heap_base_addr;
10    	#endif
11    	
12    	int MAXSIZE;
13    	int MAXDIGITS;
14    	
15    	/*
16    	   MAXSIZE is the accuracy of the calculation in 32-bit words.
17    	   MAXDIGITS is the number of decimals that will be printed.
18    	   MAXSIZE should be a little more than MAXDIGITS divided by the
19    	   base-10 logarithm of 2 to the 32th power, or a little bit more 
20    	   than 0.1038 times MAXDIGITS. For an explanation of this fact,
21    	   see, e.g., "The Mathematical Theory of Communication" by Claude Shannon
22    	*/
23    	
24    	typedef union {
25    	    unsigned long long L;
26    	    struct {
27    	        unsigned int lo;
28    	        unsigned int hi;
29    	    } w;
30    	} dword;
31    	
32    	typedef unsigned int *bignum;
33    	
34    	/*
35    	   This program represents "big numbers" (numbers with many digits) as
36    	   arrays of unsigned ints. The first element (with index zero)
37    	   in such an array represents the integer part of the number, the other
38    	   elements represent the part after the decimal point.
39    	*/
40    	
41    	// function prototypes
42    	void copybig(bignum dest, bignum source);
43    	void multbig(bignum number, unsigned int x);
44    	int printbig(bignum number);
45    	int divbig(bignum number, unsigned int x);
46    	void addbig(bignum dest, bignum source);
47    	void subbig(bignum dest, bignum source);
48    	void setbig(bignum number, int intpart, unsigned int decpart);
49    	int atanbig(bignum A, unsigned int x);
50    	int make_pi(void);
51    	
52    	// Copies a big number from source to dest
53    	void copybig(bignum dest, bignum source)
54    	{
55    	    int j;
56    	
57    	    for (j = 0; j < MAXSIZE; j++)
58    	        dest[j] = source[j];
59    	}
60    	
61    	// multiplies a big number by an unsigned integer
62    	void multbig(bignum number, unsigned int x)
63    	{
64    	    int j = MAXSIZE - 1;
65    	    dword result;
66    	    unsigned int remember = 0;
67    	
68    	    while (number[j] == 0 && j >= 0)
69    	        j--;
70    	    while (j >= 0) {
71    	        result.L = (long long) number[j] * x + remember;
72    	        number[j] = result.w.lo;
73    	        remember = result.w.hi;
74    	        j--;
75    	    }
76    	}
77    	
78    	/*
79    	  prints a big number as a decimal float number, by repeatedly multiplying
80    	  by 100000 and printing the integer part
81    	*/
82    	int printbig(bignum number)
83    	{
84    	    bignum num;
85    	    int count = 0, ndigits = 0;
86    	
87    	    if((num = malloc(MAXSIZE*sizeof(int))) == NULL)
88    	        return(1);
89    	
90    	    copybig(num, number);
91    	    fprintf(fp,"%d.\n", num[0]);
92    	    while (ndigits < MAXDIGITS) {
93    	        num[0] = 0;
94    	        multbig(num, 100000);
95    	        fprintf(fp,"%05d", num[0]);
96    	        ndigits += 5;
97    	        count++;
98    	        if (count == 6) {
99    	            fprintf(fp,"\n");
100   	            count = 0;
101   	        }
102   	    }
103   	
104   	    return(0);
105   	}
106   	
107   	/*
108   	  divides a big number by an unsigned integer, returns zero if result zero,
109   	  otherwise returns 1
110   	*/
111   	int divbig(bignum number, unsigned int x)
112   	{
113   	    dword result;
114   	    int j = 0;
115   	    unsigned int rest = 0;
116   	
117   	    while (number[j] == 0 && j < MAXSIZE)
118   	        j++;
119   	    if (j == MAXSIZE)
120   	        return (0);
121   	    while (j < MAXSIZE) {
122   	        result.w.lo = number[j];
123   	        result.w.hi = rest;
124   	        number[j] = result.L / x;
125   	        rest = result.L % x;
126   	        j++;
127   	    }
128   	    return (1);
129   	}
130   	
131   	/*
132   	  adds "source" to "dest". "addbig" and "subbig" assume that "dest" and
133   	  "source" are both positive and that the result of a subtraction will also
134   	  always be positive.
135   	*/
136   	void addbig(bignum dest, bignum source)
137   	{
138   	    int j = MAXSIZE - 1;
139   	    dword result;
140   	    unsigned int remember = 0;
141   	
142   	    while (source[j] == 0 && j >= 0)
143   	        j--;
144   	    while (j >= 0) {
145   	        result.L = (long long) dest[j] + source[j] + remember;
146   	        dest[j] = result.w.lo;
147   	        remember = result.w.hi;
148   	        j--;
149   	    }
150   	}
151   	
152   	// subtracts source from dest
153   	void subbig(bignum dest, bignum source)
154   	{
155   	    int j = MAXSIZE - 1;
156   	    unsigned int borrow = 0;
157   	    dword result;
158   	
159   	    while (source[j] == 0 && j >= 0)
160   	        j--;
161   	    while (j >= 0) {
162   	        result.L = (long long) dest[j] - source[j] + (int) borrow;
163   	        dest[j] = result.w.lo;
164   	        borrow = result.w.hi;
165   	        j--;
166   	    }
167   	}
168   	
169   	// fill a big number with some numbers
170   	void setbig(bignum number, int intpart, unsigned int decpart)
171   	{
172   	    int j;
173   	
174   	    number[0] = intpart;
175   	    for (j = 1; j < MAXSIZE; j++)
176   	        number[j] = decpart;
177   	}
178   	
179   	/*
180   	  Calculates the arc tangent of (1/x) by series summation.
181   	  Result is stored in big number A
182   	*/
183   	int atanbig(bignum A, unsigned int x)
184   	{
185   	    bignum X, Y;
186   	    unsigned int n = 1;
187   	
188   	    if((X = malloc(MAXSIZE*sizeof(int))) == NULL)
189   	        return(1);
190   	    if((Y = malloc(MAXSIZE*sizeof(int))) == NULL)
191   	        return(1);
192   	
193   	    setbig(X, 1, 0);
194   	    divbig(X, x);
195   	    copybig(A, X);
196   	    x *= x;
197   	    while (1) {
198   	        n += 2;
199   	        divbig(X, x);
200   	        copybig(Y, X);
201   	        if (!divbig(Y, n))
202   	            break;
203   	        if (n & 2)
204   	            subbig(A, Y);
205   	        else
206   	            addbig(A, Y);
207   	    }
208   	
209   	    free(X);
210   	    free(Y);
211   	
212   	    return(0);
213   	}
214   	
215   	/*
216   	  Calculates and prints pi using Machin's formula:
217   	  pi = 16 * atan(1/5)  - 4 * atan (1/239)
218   	*/
219   	int make_pi(void)
220   	{
221   	    bignum A, B;
222   	    int start, end;
223   	
224   	    if((A = malloc(MAXSIZE*sizeof(int))) == NULL)
225   	        return(1);
226   	    if((B = malloc(MAXSIZE*sizeof(int))) == NULL)
227   	        return(1);
228   	
229   	    printf("computing atan(1/5).....");
230   	    start = sys_RTC_seconds();
231   	    if(atanbig(A, 5) != 0)
232   	        return(1);
233   	    end = sys_RTC_seconds();
234   	    printf(" %3d s\n",end-start);
235   	
236   	    printf("computing atan(1/239)...");
237   	    start = sys_RTC_seconds();
238   	    if(atanbig(B, 239) != 0)
239   	        return(1);
240   	    end = sys_RTC_seconds();
241   	    printf(" %3d s\n",end-start);
242   	
243   	    multbig(A, 16);
244   	    multbig(B, 4);
245   	    subbig(A, B);
246   	    free(B);
247   	    if(printbig(A) != 0)
248   	        return(1);
249   	    printf("\n");
250   	
251   	    return(0);
252   	}
253   	
254   	#ifndef HPAPINE
255   	int freemem()
256   	{
257   	    register unsigned int stack_ptr asm ("sp");
258   	    unsigned int base;
259   	
260   	    base=(__heap_ptr == 0)? _heap_base_addr:__heap_ptr;
261   	
262   	    return stack_ptr-base;
263   	}
264   	#endif
265   	
266   	int main(void)
267   	{
268   	    int start, end, speed;
269   	    char *filename = "pimach.txt";
270   	#ifndef HPAPINE
271   	    int mem;
272   	
273   	    mem = freemem();
274   	#endif
275   	    speed = sat_pop_real();
276   	    if(speed != 0 && speed != 1) {
277   	        sat_push_zint_llong(speed);
278   	        sat_stack_push_string("speed must be 0 or 1");
279   	        sat_push_real(1);
280   	        return(0);
281   	    }
282   	
283   	    MAXDIGITS = sat_pop_real();
284   	    if(MAXDIGITS < 1) {
285   	        sat_push_zint_llong(MAXDIGITS);
286   	        sat_stack_push_string("PI digits must be > 0");
287   	        sat_push_real(1);
288   	        return(0);
289   	    }
290   	    MAXSIZE = MAXDIGITS * 0.1038 + 10;
291   	
292   	    if((fp = fopen(filename,"w")) == NULL) {
293   	        char errormsg[30];
294   	
295   	        sprintf(errormsg,"Cannot open: %s",filename);
296   	
297   	        sat_stack_push_string(errormsg);
298   	        sat_push_real(1);
299   	        return(0);
300   	    }
301   	
302   	    if(speed == 0) 
303   	        sys_slowOn();
304   	    else
305   	        sys_slowOff();
306   	
307   	    clear_screen();
308   	    printf("Machin's Pi\n\n"); 
309   	
310   	    hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_BLACK);
311   	    start = sys_RTC_seconds();
312   	    if(make_pi() != 0) {
313   	        sys_slowOn();
314   	        sat_stack_push_string("Failed to allocate memory.");
315   	        sat_push_real(1);
316   	        return(0);
317   	    }
318   	    end = sys_RTC_seconds();
319   	    hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_WHITE);
320   	
321   	    fclose(fp);
322   	
323   	    if(speed == 0)
324   	        printf("      speed: normal\n");
325   	    else
326   	        printf("      speed: fast\n");
327   	    printf("    time(s): %d\n",end-start);
328   	    printf("bytes alloc: %d\n",4*MAXSIZE*(int)sizeof(int));
329   	#ifndef HPAPINE
330   	    printf(" bytes used: %d\n",mem-freemem());
331   	#endif
332   	
333   	    sat_push_real(0);
334   	    beep();
335   	    sys_slowOn();
336   	    WAIT_CANCEL;
337   	    return(0);
338   	}
