#include <hpgcc49.h>  //the standard HP lib
#include <hpgraphics.h>

#ifndef HPAPINE
extern unsigned int __heap_ptr,_heap_base_addr;
int freemem();
#endif

int main(void){
	unsigned int *a, b, c, d = 0, e = 0, f = 10000, g, h = 0;
	int start, end, cc = 0, ndigits;
	FILE *fp;
	char *filename = "spigot.txt";
	char errormsg[40];
#ifndef HPAPINE
	int mem;

	mem = freemem();
#endif

	ndigits = sat_pop_real();  //get ndigits from stack
	if(ndigits < 1) {
		sat_push_zint_llong((int)ndigits);
		sat_stack_push_string("PI digits must be > 0");
		sat_push_real(1);
		return(0);
	}
	if(ndigits % 4 != 0)
		ndigits+=4;
	c=(ndigits/4+1)*14;

	if((fp = fopen(filename,"w")) == NULL) {
		sprintf(errormsg,"Cannot open: %s",filename);

		sat_stack_push_string(errormsg);
		sat_push_real(1);
		return(0);
	}

	sys_slowOff();    //max speed

	if((a=malloc(c*sizeof(*a))) == NULL) {
		sprintf(errormsg,"Cannot allocate %d bytes\n",(int)(c*sizeof(*a)));
		sat_stack_push_string(errormsg);
		sys_slowOn();
		sat_push_real(1);
		return(0);
	}

	clear_screen();   //clear the screen
	hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_BLACK);

	start = sys_RTC_seconds();  //get start time in seconds since 1/1/1970
	while ((b=c-=14) > 0) {     //logic, compute 4 digits of pi at a time then display
		while(--b > 0) {
			d *= b;
			if (h == 0)
				d += 2000 * f;
			else
				d += a[b] * f;
			g=b+b-1;
			a[b] = d % g;
			d /= g;
		}
		printf("%04d", e + d/f);
		fprintf(fp,"%04d", e + d/f);
		cc+=4;
		if(cc % 32 == 0) {
			printf("\n");
			fprintf(fp,"\n");
    		hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_BLACK);
		}
		if(keyb_isAnyKeyPressed())  //hold any key to stop, do not use ON
			break;
		d = e =	d % f;
		h = 4;
	}
	end = sys_RTC_seconds();  //get end time in seconds since 1/1/1970
	if(cc % 32 != 0) {
		printf("\n");
		fprintf(fp,"\n");
	}
	printf("\n");
	fprintf(fp,"\n");
	fclose(fp);

	printf("bytes alloc: %d\n",(ndigits/4+1)*14*(int)sizeof(*a));
	sat_push_zint_llong((ndigits/4+1)*14*(int)sizeof(*a));
	sat_stack_push_string("bytes alloc");
#ifndef HPAPINE
	printf(" bytes used: %d\n",mem-freemem());
	sat_push_zint_llong(mem-freemem());
	sat_stack_push_string("bytes used");
#endif
	printf("    time(s): %d\n",end-start);
	sat_push_zint_llong(end-start);
	sat_stack_push_string("time(s)");

	printf("     digits: %d\n",cc);
	sat_push_zint_llong(cc);
	sat_stack_push_string("digits");

	if(end-start == 0) {
		printf("   digits/s: NAN");
		sat_stack_push_string("NAN");
	}
	else {
		printf("   digits/s: %.2f",(float)cc/(float)(end-start));
		sat_push_real(((cc*100)/(end-start) + .5)/100.0);
	}
	sat_stack_push_string("digits/s");

	sat_push_real(0);
	hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_WHITE);
	beep();
	sys_slowOn(); //normal speed
	WAIT_CANCEL;  //loop until ON pressed
	return(0);
}

#ifndef HPAPINE
int freemem()
{
	register unsigned int stack_ptr asm ("sp");
	unsigned int base;

	base=(__heap_ptr == 0)? _heap_base_addr:__heap_ptr;

	return stack_ptr-base;
}
#endif
