#include <hpgcc49.h>
#include <hpgraphics.h>
#include <tommath.h>

#ifndef HPAPINE
extern unsigned int __heap_ptr,_heap_base_addr;
int freemem();
#endif

int main()
{
	mp_int q, r, t, k, n, l;
	mp_int r1, n1;
	mp_int temp1, temp2, rmd;
	int numdigits = 0, loops = 0, start, end;
	double maxdigits;
	char buf[4];
	FILE *fp;
	char *filename = "uspi.txt";
#ifndef HPAPINE
	int mem, lowmem = 0;;
#endif

	maxdigits = sat_pop_real();
	if(maxdigits < 1) {
		sat_push_zint_llong((int)maxdigits);
		sat_stack_push_string("PI digits must be > 0");
		sat_push_real(1);
		return(0);
	}

	if((fp = fopen(filename,"w")) == NULL) {
		char errormsg[30];

		sprintf(errormsg,"Cannot open: %s",filename);

		sat_stack_push_string(errormsg);
		sat_push_real(1);
		return(0);
	}

	sys_slowOff();
	clear_screen();

#ifndef HPAPINE
	mem = freemem();
#endif
	mp_init(&q);
	mp_init(&r);
	mp_init(&t);
	mp_init(&k);
	mp_init(&n);
	mp_init(&l);
	mp_init(&r1);
	mp_init(&n1);
	mp_init(&temp1);
	mp_init(&temp2);
	mp_init(&rmd);

	mp_set_int(&q,1);
	mp_zero(&r);
	mp_set_int(&t,1);
	mp_set_int(&k,1);
	mp_set_int(&n,3);
	mp_set_int(&l,3);

	hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_BLACK);
	start = sys_RTC_seconds();

	while(numdigits < maxdigits) {
		loops++;

		mp_mul_d(&q,4,&temp1);
		mp_add(&temp1,&r,&temp1);
		mp_sub(&temp1,&t,&temp1);
		mp_mul(&n,&t,&temp2);

		if(mp_cmp(&temp1,&temp2) == MP_LT) {
			mp_todecimal(&n, buf);
			printf("%d",atoi(buf));
			fprintf(fp,"%d",atoi(buf));
			numdigits++;

			if(numdigits % 33 == 0) {
				fprintf(fp,"\n");
				hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_BLACK);
#ifndef HPAPINE
				if(freemem() < 25000)
					hpg_set_indicator(HPG_INDICATOR_BATTERY,HPG_COLOR_BLACK);
#endif
			}

			mp_sub(&r,&temp2,&r1);
			mp_mul_d(&r1,10,&r1);

			mp_mul_d(&q,3,&temp1);
			mp_add(&temp1,&r,&temp1);
			mp_mul_d(&temp1,10,&temp1);
			mp_div(&temp1,&t,&temp1,&rmd);
			mp_mul_d(&n,10,&temp2);
			mp_sub(&temp1,&temp2,&n1);

			mp_mul_d(&q,10,&q);

			mp_copy(&r1,&r);
			mp_copy(&n1,&n);
		}
		else {
			mp_mul_d(&q,2,&r1);
			mp_add(&r1,&r,&r1);
			mp_mul(&r1,&l,&r1);

			mp_mul_d(&k,7,&temp1);
			mp_add_d(&temp1,2,&temp1);
			mp_mul(&temp1,&q,&temp1);
			mp_mul(&r,&l,&temp2);
			mp_add(&temp1,&temp2,&temp1);
			mp_mul(&t,&l,&temp2);
			mp_div(&temp1,&temp2,&n1,&rmd);

			mp_mul(&q,&k,&q);
			mp_add_d(&k,1,&k);
			mp_mul(&t,&l,&t);
			mp_add_d(&l,2,&l);

			mp_copy(&r1,&r);
			mp_copy(&n1,&n);
		}

		if(keyb_isAnyKeyPressed())
			break;

#ifndef HPAPINE
		if(freemem() < 25000 && lowmem == 0) {
			lowmem = numdigits;
			beep();
			hpg_set_indicator(HPG_INDICATOR_BATTERY,HPG_COLOR_BLACK);
		}

		if(freemem() < 5000)
			break;
#endif
	}

	end = sys_RTC_seconds();
	fclose(fp);

	printf("\n\n");
	printf("longest digit: %d digits\n",(int)(log10(256.0)*mp_unsigned_bin_size(&q)));
#ifdef HPAPINE
	printf("  bytes alloc: %d\n",
		mp_unsigned_bin_size(&q) +
		mp_unsigned_bin_size(&r) +
		mp_unsigned_bin_size(&t) +
		mp_unsigned_bin_size(&k) +
		mp_unsigned_bin_size(&n) +
		mp_unsigned_bin_size(&l) +
		mp_unsigned_bin_size(&r1) +
		mp_unsigned_bin_size(&n1) +
		mp_unsigned_bin_size(&temp1) +
		mp_unsigned_bin_size(&temp2) +
		mp_unsigned_bin_size(&rmd)
	);
#else
	printf("   bytes free: %d\n",mem);
	printf("   bytes used: %d\n",mem-freemem());
	if(lowmem)
		printf(" bytes low(d): %d\n",lowmem);
#endif
	printf("      time(s): %d\n",end-start);
	printf("       digits: %d\n",numdigits);
	if(end-start == 0)
		printf("     digits/s: NAN\n");
	else
		printf("     digits/s: %.2f\n",(float)numdigits/(float)(end-start));
	printf("   iterations: %d\n",loops);
	if(end-start == 0)
		printf("interations/s: NAN");
	else
		printf("interations/s: %.2f",(float)loops/(float)(end-start));

	hpg_set_indicator(HPG_INDICATOR_WAIT,HPG_COLOR_WHITE);
	hpg_set_indicator(HPG_INDICATOR_BATTERY,HPG_COLOR_WHITE);

	sat_push_real(0);
	beep();
	sys_slowOn();
	WAIT_CANCEL;
	return(0);
}

#ifndef HPAPINE
int freemem()
{
	register unsigned int stack_ptr asm ("sp");
	unsigned int base;

	base=(__heap_ptr == 0)? _heap_base_addr:__heap_ptr;

	return stack_ptr-base;
}
#endif
