#include <hpgcc49.h>
#include <cs.h>

double *load_vector(char *);
cs *load_matrix(char *);

int main (void)
{
	cs *A;
	double *b; 
	int ok = 0;
	char *s, *t, *c;
	int lusol, qrsol, cholsol, return_vector;

	lusol = qrsol = cholsol = return_vector = 0;

	c = sat_stack_pop_string_alloc();

	if(strcmp(c,"LUSOL") == 0)
		lusol = 1;
	else if(strcmp(c,"QRSOL") == 0)
		qrsol = 1;
	else if(strcmp(c,"CHOLSOL") == 0)
		cholsol = 1;
	else {
		char error[40];
		char opt[20];

		strncpy(opt,c,19);
		opt[19] = '\0';
		sprintf(error,"Unknown option: %s",opt);
		sat_stack_push_string(error);
		sat_push_real(1);
		return(0);
	}

	if(lusol || qrsol || cholsol) {
		int order;
		double tol = 1e-14;

		order = sat_pop_zint_llong();

		if(order < 0 || order > 3) {
			char error[40];
			sprintf(error,"Unknown order: %d",order);
			sat_stack_push_string(error);
			sat_push_real(1);
	    	return(0);
		}

		t = sat_stack_pop_string_alloc();
		if(t == NULL) {
			sat_stack_push_string("b is not a Vector or Out of Memory.");
			sat_push_real(1);
			return(0);
		}

		s = sat_stack_pop_string_alloc();
		if(t == NULL) {
			sat_stack_push_string("A is not a Matrix Triplet List or Out of Memory.");
			sat_push_real(1);
			return(0);
		}

		sys_slowOff();

		b = load_vector(t);
		free(t);
		A = cs_compress(load_matrix(s));
		free(s);

		if(lusol)
			ok = cs_lusol(order, A, b, tol);
		if(qrsol)
			ok = cs_qrsol(order, A, b);
		if(cholsol)
			ok = cs_cholsol(order, A, b);

		return_vector = 1;
	}

	if(ok == 1 && return_vector) {
		int i;
		char num[21];
		char *out;
		int n;

		n = A->n;
		cs_free(A);

		out = malloc((22*n + 4)*sizeof(char));
		out[0] = '\0';

		strcat(out,"[ ");
		for(i=0;i < n;i++) {
			sprintf(num,"%0.12E ",b[i]);
			strcat(out,num);
		}
		strcat(out,"]");

		sat_stack_push_string(out);
		sat_push_real(0);
	}
	else {
		if(cholsol)
			sat_stack_push_string("Matrix is not Positive Definite or Out of Memory.");
		else
			sat_stack_push_string("Bogus Inputs or Out of Memory.");
		sat_push_real(1);
	}

	free(b);

	sys_slowOn();

    return(0);
}

double *load_vector(char *s) {
	int i=0;
	double x;
	double *b = NULL;

	int k=0;
	char c[100];

	s++;
	while(*s++ != '\0') {
		if(*s == ' ') {
			c[k] = '\0';
			if(c > 0) {
				if(sscanf(c,"%lg",&x) == 1) {
					b = realloc(b,(i+1)*sizeof(double));
					b[i++] = x;
				}
			}
			k=0;
		}
		c[k++] = *s;
	}

	return(b);
}

cs *load_matrix(char *s) {
	int i,j;
	double x;
	cs *T;

	int k=0;
	char c[100];

	T = cs_spalloc(0,0,1,1,1);

	while(*s++ != '\0') {
		if(*s == '{') {
			s++;
			k=0;
			continue;
		}
		if(*s == '}') {
			c[--k] = '\0';
			if(k > 0) {
				sscanf(c,"%d %d %lg",&i,&j,&x);
				if(!cs_entry(T, i, j, x))
					return(cs_spfree(T));
			}
			s+=3;
			k=0;
			continue;
		}
		c[k++] = *s;
	}

	return(T);
}
