1     	#include <hpgcc49.h>
2     	#include <cs.h>
3     	
4     	double *load_vector(char *);
5     	cs *load_matrix(char *);
6     	
7     	int main (void)
8     	{
9     	    cs *A;
10    	    double *b; 
11    	    int ok = 0;
12    	    char *s, *t, *c;
13    	    int lusol, qrsol, cholsol, return_vector;
14    	
15    	    lusol = qrsol = cholsol = return_vector = 0;
16    	
17    	    c = sat_stack_pop_string_alloc();
18    	
19    	    if(strcmp(c,"LUSOL") == 0)
20    	        lusol = 1;
21    	    else if(strcmp(c,"QRSOL") == 0)
22    	        qrsol = 1;
23    	    else if(strcmp(c,"CHOLSOL") == 0)
24    	        cholsol = 1;
25    	    else {
26    	        char error[40];
27    	        char opt[20];
28    	
29    	        strncpy(opt,c,19);
30    	        opt[19] = '\0';
31    	        sprintf(error,"Unknown option: %s",opt);
32    	        sat_stack_push_string(error);
33    	        sat_push_real(1);
34    	        return(0);
35    	    }
36    	
37    	    if(lusol || qrsol || cholsol) {
38    	        int order;
39    	        double tol = 1e-14;
40    	
41    	        order = sat_pop_zint_llong();
42    	
43    	        if(order < 0 || order > 3) {
44    	            char error[40];
45    	            sprintf(error,"Unknown order: %d",order);
46    	            sat_stack_push_string(error);
47    	            sat_push_real(1);
48    	            return(0);
49    	        }
50    	
51    	        t = sat_stack_pop_string_alloc();
52    	        if(t == NULL) {
53    	            sat_stack_push_string("b is not a Vector or Out of Memory.");
54    	            sat_push_real(1);
55    	            return(0);
56    	        }
57    	
58    	        s = sat_stack_pop_string_alloc();
59    	        if(t == NULL) {
60    	            sat_stack_push_string("A is not a Matrix Triplet List or Out of Memory.");
61    	            sat_push_real(1);
62    	            return(0);
63    	        }
64    	
65    	        sys_slowOff();
66    	
67    	        b = load_vector(t);
68    	        free(t);
69    	        A = cs_compress(load_matrix(s));
70    	        free(s);
71    	
72    	        if(lusol)
73    	            ok = cs_lusol(order, A, b, tol);
74    	        if(qrsol)
75    	            ok = cs_qrsol(order, A, b);
76    	        if(cholsol)
77    	            ok = cs_cholsol(order, A, b);
78    	
79    	        return_vector = 1;
80    	    }
81    	
82    	    if(ok == 1 && return_vector) {
83    	        int i;
84    	        char num[21];
85    	        char *out;
86    	        int n;
87    	
88    	        n = A->n;
89    	        cs_free(A);
90    	
91    	        out = malloc((22*n + 4)*sizeof(char));
92    	        out[0] = '\0';
93    	
94    	        strcat(out,"[ ");
95    	        for(i=0;i < n;i++) {
96    	            sprintf(num,"%0.12E ",b[i]);
97    	            strcat(out,num);
98    	        }
99    	        strcat(out,"]");
100   	
101   	        sat_stack_push_string(out);
102   	        sat_push_real(0);
103   	    }
104   	    else {
105   	        if(cholsol)
106   	            sat_stack_push_string("Matrix is not Positive Definite or Out of Memory.");
107   	        else
108   	            sat_stack_push_string("Bogus Inputs or Out of Memory.");
109   	        sat_push_real(1);
110   	    }
111   	
112   	    free(b);
113   	
114   	    sys_slowOn();
115   	
116   	    return(0);
117   	}
118   	
119   	double *load_vector(char *s) {
120   	    int i=0;
121   	    double x;
122   	    double *b = NULL;
123   	
124   	    int k=0;
125   	    char c[100];
126   	
127   	    s++;
128   	    while(*s++ != '\0') {
129   	        if(*s == ' ') {
130   	            c[k] = '\0';
131   	            if(c > 0) {
132   	                if(sscanf(c,"%lg",&x) == 1) {
133   	                    b = realloc(b,(i+1)*sizeof(double));
134   	                    b[i++] = x;
135   	                }
136   	            }
137   	            k=0;
138   	        }
139   	        c[k++] = *s;
140   	    }
141   	
142   	    return(b);
143   	}
144   	
145   	cs *load_matrix(char *s) {
146   	    int i,j;
147   	    double x;
148   	    cs *T;
149   	
150   	    int k=0;
151   	    char c[100];
152   	
153   	    T = cs_spalloc(0,0,1,1,1);
154   	
155   	    while(*s++ != '\0') {
156   	        if(*s == '{') {
157   	            s++;
158   	            k=0;
159   	            continue;
160   	        }
161   	        if(*s == '}') {
162   	            c[--k] = '\0';
163   	            if(k > 0) {
164   	                sscanf(c,"%d %d %lg",&i,&j,&x);
165   	                if(!cs_entry(T, i, j, x))
166   	                    return(cs_spfree(T));
167   	            }
168   	            s+=3;
169   	            k=0;
170   	            continue;
171   	        }
172   	        c[k++] = *s;
173   	    }
174   	
175   	    return(T);
176   	}
