#include <hpgcc49.h>
#include <cs.h>
#include <hpstack.h>

double *load_vector(array_t *);
cs *load_matrix(list_t *);

int main (void)
{
	cs *A;
	double *b; 
	int ok = 0;
	char *c;
	list_t *t = NULL;
	array_t *v = NULL;
	int lusol, qrsol, cholsol, return_vector;

	lusol = qrsol = cholsol = return_vector = 0;

	c = sat_stack_pop_string_alloc();

	if(strcmp(c,"LUSOL") == 0)
		lusol = 1;
	else if(strcmp(c,"QRSOL") == 0)
		qrsol = 1;
	else if(strcmp(c,"CHOLSOL") == 0)
		cholsol = 1;
	else {
		char error[40];
		char opt[20];

		strncpy(opt,c,19);
		opt[19] = '\0';
		sprintf(error,"Unknown option: %s",opt);
		sat_stack_push_string(error);
		sat_push_real(1);
	    return(0);
	}

	if(lusol || qrsol || cholsol) {
		int order;
		double tol = 1e-14;

		order = sat_pop_zint_llong();

		if(order < 0 || order > 3) {
			char error[40];
			sprintf(error,"Unknown order: %d",order);
			sat_stack_push_string(error);
			sat_push_real(1);
	    	return(0);
		}

		if (hps_pop_array(&v) != HPS_OK) {
			sat_stack_push_string("b is not a Vector.");
			sat_push_real(1);
	    	return(0);
		}

		if (hps_pop_list(&t) != HPS_OK) {
			sat_stack_push_string("A is not a Matrix Triplet List.");
			sat_push_real(1);
	    	return(0);
		}

		sys_slowOff();

		b = load_vector(v);
		hps_array_destroy(&v);
		A = cs_compress(load_matrix(t));
		hps_list_destroy(&t);

		if(lusol)
			ok = cs_lusol(order, A, b, tol);
		if(qrsol)
			ok = cs_qrsol(order, A, b);
		if(cholsol)
			ok = cs_cholsol(order, A, b);

		return_vector = 1;
	}

	if(ok == 1 && return_vector) {
#ifdef USE_HPSTACK_OUT
		array_t *x = NULL;
		int n;

		n = A->n;
		cs_free(A);

		hps_array_build(&x,b,SAT_DOREAL,1,n);
		hps_push_array(x);
		sat_push_real(0);
#else
		int i;
		char num[21];
		char *out;
		int n;

		n = A->n;
		cs_free(A);

		out = malloc((n + 4)*sizeof(char));
		out[0] = '\0';

		strcat(out,"[ ");
		for(i=0;i < n;i++) {
			sprintf(num,"%0.12E ",b[i]);
			strcat(out,num);
		}
		strcat(out,"]");

		sat_stack_push_string(out);
		sat_push_real(0);
#endif
	}
	else {
		if(cholsol)
			sat_stack_push_string("Matrix is not Positive Definite or Out of Memory.");
		else
			sat_stack_push_string("Bogus Inputs or Out of Memory.");
		sat_push_real(1);
	}

	free(b);

	sys_slowOn();

    return (0);
}

double *load_vector(array_t *v) {
	int index=1;
	double x;
	double *b = NULL;
	
	while(hps_list_get_real(v ,index ,&x) == HPS_OK) {
		b = realloc(b,index*sizeof(double));
		b[index-1] = x;
		index++;
	}

	return(b);
}

cs *load_matrix(list_t *t) {
	list_t *e;
	int index=1;
	cs *T;

	T = cs_spalloc(0,0,1,1,1);

	while(hps_list_get_list(t ,index++ ,&e) == HPS_OK) {
		LONGLONG i,j;
		double x;

		hps_list_get_int(e, 1, &i);
		hps_list_get_int(e, 2, &j);
		hps_list_get_real(e, 3, &x);

		if(!cs_entry(T, i, j, x)) {
			return(cs_spfree(T));
		}
	}

	return(T);
}
