1     	#include <hpgcc49.h>
2     	#include <cs.h>
3     	#include <hpstack.h>
4     	
5     	double *load_vector(array_t *);
6     	cs *load_matrix(list_t *);
7     	
8     	int main (void)
9     	{
10    	    cs *A;
11    	    double *b; 
12    	    int ok = 0;
13    	    char *c;
14    	    list_t *t = NULL;
15    	    array_t *v = NULL;
16    	    int lusol, qrsol, cholsol, return_vector;
17    	
18    	    lusol = qrsol = cholsol = return_vector = 0;
19    	
20    	    c = sat_stack_pop_string_alloc();
21    	
22    	    if(strcmp(c,"LUSOL") == 0)
23    	        lusol = 1;
24    	    else if(strcmp(c,"QRSOL") == 0)
25    	        qrsol = 1;
26    	    else if(strcmp(c,"CHOLSOL") == 0)
27    	        cholsol = 1;
28    	    else {
29    	        char error[40];
30    	        char opt[20];
31    	
32    	        strncpy(opt,c,19);
33    	        opt[19] = '\0';
34    	        sprintf(error,"Unknown option: %s",opt);
35    	        sat_stack_push_string(error);
36    	        sat_push_real(1);
37    	        return(0);
38    	    }
39    	
40    	    if(lusol || qrsol || cholsol) {
41    	        int order;
42    	        double tol = 1e-14;
43    	
44    	        order = sat_pop_zint_llong();
45    	
46    	        if(order < 0 || order > 3) {
47    	            char error[40];
48    	            sprintf(error,"Unknown order: %d",order);
49    	            sat_stack_push_string(error);
50    	            sat_push_real(1);
51    	            return(0);
52    	        }
53    	
54    	        if (hps_pop_array(&v) != HPS_OK) {
55    	            sat_stack_push_string("b is not a Vector.");
56    	            sat_push_real(1);
57    	            return(0);
58    	        }
59    	
60    	        if (hps_pop_list(&t) != HPS_OK) {
61    	            sat_stack_push_string("A is not a Matrix Triplet List.");
62    	            sat_push_real(1);
63    	            return(0);
64    	        }
65    	
66    	        sys_slowOff();
67    	
68    	        b = load_vector(v);
69    	        hps_array_destroy(&v);
70    	        A = cs_compress(load_matrix(t));
71    	        hps_list_destroy(&t);
72    	
73    	        if(lusol)
74    	            ok = cs_lusol(order, A, b, tol);
75    	        if(qrsol)
76    	            ok = cs_qrsol(order, A, b);
77    	        if(cholsol)
78    	            ok = cs_cholsol(order, A, b);
79    	
80    	        return_vector = 1;
81    	    }
82    	
83    	    if(ok == 1 && return_vector) {
84    	#ifdef USE_HPSTACK_OUT
85    	        array_t *x = NULL;
86    	        int n;
87    	
88    	        n = A->n;
89    	        cs_free(A);
90    	
91    	        hps_array_build(&x,b,SAT_DOREAL,1,n);
92    	        hps_push_array(x);
93    	        sat_push_real(0);
94    	#else
95    	        int i;
96    	        char num[21];
97    	        char *out;
98    	        int n;
99    	
100   	        n = A->n;
101   	        cs_free(A);
102   	
103   	        out = malloc((n + 4)*sizeof(char));
104   	        out[0] = '\0';
105   	
106   	        strcat(out,"[ ");
107   	        for(i=0;i < n;i++) {
108   	            sprintf(num,"%0.12E ",b[i]);
109   	            strcat(out,num);
110   	        }
111   	        strcat(out,"]");
112   	
113   	        sat_stack_push_string(out);
114   	        sat_push_real(0);
115   	#endif
116   	    }
117   	    else {
118   	        if(cholsol)
119   	            sat_stack_push_string("Matrix is not Positive Definite or Out of Memory.");
120   	        else
121   	            sat_stack_push_string("Bogus Inputs or Out of Memory.");
122   	        sat_push_real(1);
123   	    }
124   	
125   	    free(b);
126   	
127   	    sys_slowOn();
128   	
129   	    return (0);
130   	}
131   	
132   	double *load_vector(array_t *v) {
133   	    int index=1;
134   	    double x;
135   	    double *b = NULL;
136   	    
137   	    while(hps_list_get_real(v ,index ,&x) == HPS_OK) {
138   	        b = realloc(b,index*sizeof(double));
139   	        b[index-1] = x;
140   	        index++;
141   	    }
142   	
143   	    return(b);
144   	}
145   	
146   	cs *load_matrix(list_t *t) {
147   	    list_t *e;
148   	    int index=1;
149   	    cs *T;
150   	
151   	    T = cs_spalloc(0,0,1,1,1);
152   	
153   	    while(hps_list_get_list(t ,index++ ,&e) == HPS_OK) {
154   	        LONGLONG i,j;
155   	        double x;
156   	
157   	        hps_list_get_int(e, 1, &i);
158   	        hps_list_get_int(e, 2, &j);
159   	        hps_list_get_real(e, 3, &x);
160   	
161   	        if(!cs_entry(T, i, j, x)) {
162   	            return(cs_spfree(T));
163   	        }
164   	    }
165   	
166   	    return(T);
167   	}
