#include <hpgcc49.h>
#include "hpspriv.h"
#include <hpstack.h>

int hps_array_create (array_t **array)
{
	return (hps_list_create ((list_t **)array));
}

int hps_array_destroy (array_t **array)
{
	return (hps_list_destroy ((list_t **)array));
}

int hps_array_add (array_t *array, hpobj_t *obj)
{
	return (hps_list_add ((list_t *)array, obj));
}

int hps_array_add_type (array_t *array, int type, void *data)
{
	return (hps_list_add_type ((list_t *)array, type, data));
}

int hps_array_add_int (array_t *array, LONGLONG val)
{
	return (hps_list_add_int ((list_t *)array, val));
}

int hps_array_add_uint (array_t *array, ULONGLONG val)
{
	return (hps_list_add_uint ((list_t *)array, val));
}

int hps_array_add_real (array_t *array, double val)
{
	return (hps_list_add_real ((list_t *)array, val));
}

int hps_array_add_complex (array_t *array, complex_t *val)
{
	return (hps_list_add_complex ((list_t *)array, val));
}

int hps_array_add_str (array_t *array, char *val)
{
	return (hps_list_add_str ((list_t *)array, val));
}

int hps_array_add_list (array_t *array, list_t *val)
{
	return (hps_list_add_list ((list_t *)array, val));
}

int hps_array_add_array (array_t *array, array_t *val)
{
	return (hps_list_add_array ((list_t *)array, val));
}

int hps_array_add_ident (array_t *array, char *val)
{
	return (hps_list_add_ident ((list_t *)array, val));
}

int hps_array_get (array_t *array, hpobj_t **obj, int nb, ...)
{
	int Ret = HPS_OK;
	int i, dim;
	array_t *elem = array;
	va_list ind;

	va_start (ind, nb);

	for (i = 0; i < nb; i++)
	{
		int j;

		dim = va_arg (ind, int);

		if (i > 0)
		{
			if (elem)
			{
				if (elem->obj->type == DOMATRIX)
				{
					elem = elem->obj->data.domatrix;
				}
				else
				{
					Ret = HPS_ERROR_PAR;
					break;
				}
			}
			else
			{
				Ret = HPS_ERROR_PAR;
				break;
			}
		}

		for (j = 1; j < dim; j++)
		{
			elem = elem->next;
			if (! elem)
			{
				Ret = HPS_ERROR_PAR;
				break;
			}
		}

		if (j < dim)
		{
			Ret = HPS_ERROR_PAR;
			break;
		}
	}

	if (i == nb)
	{
		*obj = elem->obj;
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}

	va_end (ind);

	return (Ret);
}

static int _sizeof (int type)
{
	int Ret = 0;

	switch (type)
	{
		case DOINT :
		{
			Ret = sizeof (LONGLONG);
			break;
		}

		case DOHXS :
		{
			Ret = sizeof (ULONGLONG);
			break;
		}

		case DOREAL :
		{
			Ret = sizeof (double);
			break;
		}

		case DOCMP :
		{
			Ret = sizeof (complex_t);
			break;
		}

		case DOCSTR :
		{
			Ret = sizeof (char *);
			break;
		}

		case DOLIST :
		{
			Ret = sizeof (list_t *);
			break;
		}

		case DOMATRIX :
		{
			Ret = sizeof (array_t *);
			break;
		}
	}

	return (Ret);
}

static int _hps_array_build (array_t ** array, void *data, int type, int size, int nb, int lev, int ind[])
{
	int Ret = HPS_OK;

	// Creates the array
	if ((Ret = hps_array_create (array)) == HPS_OK)
	{
		int dim = ind[lev - 1];

		// Checks for the end of recursion
		if (lev == nb)
		{
			int i;

			// Fills the array with elementary data
			for (i = 0; i < dim; i++)
			{
				(void) hps_array_add_type (*array, type, (void *)((int)data + i*size));
			}
		}
		else
		{
			int j, k;
			array_t *ar;
			int n = 1;

			for (k = lev; k < nb; k++)
			{
				n *= ind[k];
			}

			for (j = 0; j < dim; j++)
			{
				_hps_array_build (&ar, (void *)((unsigned long)data + n*j*size), type, size, nb, lev + 1, ind);
				hps_array_add_array (*array, ar);
			}
		}
	}

	return (Ret);
}

int hps_array_build (array_t **array, void *data, int type, int nb, ...)
{
	va_list ind;
	int Ret = HPS_OK;

	va_start (ind, nb);

	if (array && data)
	{
		int *tab;
		if (tab = (int *) malloc (nb * sizeof(int)))
		{
			int i;

			for (i = 0; i < nb; i++)
			{
				tab[i] = va_arg(ind, int);
			}
			va_end (ind);

			Ret = _hps_array_build (array, data, type, _sizeof (type), nb, 1, tab);
		}
		else
		{
			Ret = HPS_ERROR_ARM_MEMORY;
		}
	}
	else
	{
		Ret = HPS_ERROR_PAR;
	}

	return (Ret);
}
